@@ -9,4 +9,6 @@
 #undef PACKAGE_SOURCE_DIR
 #undef CVS_SOURCE 
 #undef CORBA_INTERFACE
-#undef MOZILLA_MILESTONE
+#undef MOZILLA_VERSION
+#undef ENABLE_APPLET
+#undef HAVE__NL_PAPER_WIDTH_HEIGHT
