@@ -17,19 +17,24 @@
  */
 
 #include "galeon.h"
-
-#ifdef CVS_SOURCE
-
+#include "misc.h"
+#include "window.h"
+#include "mozilla.h"
 #include "PromptService.h"
+
 #include "nsCOMPtr.h"
-#include "nsIPromptService.h"
 #include "nsIFactory.h"
 #include "nsString.h"
 #include "nsIServiceManager.h"
-#include "nsIWebBrowserChrome.h"
-#include "nsIEmbeddingSiteWindow.h"
+#include "nsXPComFactory.h"
+
+#include "nsIPromptService.h"
 #include "nsIUnicodeEncoder.h"
-#include "nsIWindowWatcher.h"
+
+#include <gtk/gtkentry.h>
+#include <gtk/gtktogglebutton.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnome/gnome-i18n.h>
 
 /* local function prototypes */
 static void set_title (GtkWidget *dialog, const PRUnichar *title);
@@ -41,8 +46,6 @@ static void set_editable (GtkWidget *entry, PRUnichar **text);
 static void get_check_button (GtkWidget *button, PRBool *value);
 static void get_editable (GtkWidget *editable, PRUnichar **text);
 
-static GtkWidget *promptService_find_window (nsIDOMWindow *domWindow);
-
 /**
  * class CPromptService: an GNOME implementation of prompt dialogs for
  * Mozilla
@@ -82,35 +85,40 @@ NS_IMETHODIMP CPromptService::Alert (nsIDOMWindow *parent,
 				     const PRUnichar *dialogTitle,
 				     const PRUnichar *text)
 {    
-	GtkWidget *parent_window;
+	GtkWidget *label;
+	GtkWidget *title_label;
 	GtkWidget *dialog;
-	gchar *gtext;
-
-	/* convert arguments */
-	gtext = mozilla_unicode_to_locale (text);
+	GladeXML *gxml;
 
 	/* build the dialog */
-	dialog = gnome_message_box_new (gtext, GNOME_MESSAGE_BOX_WARNING,
-					GNOME_STOCK_BUTTON_OK, NULL);
-	set_title (dialog, dialogTitle);
-	
-	/* parent properly */
-	/* FIXME: DONT CUT AND PASTE THIS CODE DAMMIT
-	 * ahem, sorry, please refactor this.... */
-	parent_window = promptService_find_window (parent);	
-	if (parent_window != NULL)
-	{
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog), 
-					 GTK_WINDOW (parent_window));
-	}
+	gxml = glade_widget_new ("prompts.glade", 
+				 "alert_dialog", &dialog, NULL);
+	label       = glade_xml_get_widget (gxml, "label");
+	title_label = glade_xml_get_widget (gxml, "title_label");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
-	/* run the dialog */
+	/* set dynamic attributes */
+	set_title (dialog, dialogTitle);
+	set_label_text (title_label, dialogTitle);
+	set_label_text (label, text);
+
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
+	/* run dialog and capture return values */
 	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
-	/* free allocated strings */
-	g_free (gtext);
-			
 	/* done */
+	gtk_widget_destroy (dialog);
 	return NS_OK;
 }
 
@@ -131,10 +139,12 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 	GladeXML *gxml;
 
 	/* build the dialog */
-	gxml = glade_widget_new ("alert_check_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "alert_check_dialog", 
+				 &dialog, NULL);
 	label        = glade_xml_get_widget (gxml, "label");
 	title_label  = glade_xml_get_widget (gxml, "title_label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -143,6 +153,18 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 	set_check_button_text (check_button, checkMsg);
 	set_check_button (check_button, checkValue);
 
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
 	gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 	get_check_button (check_button, checkValue);
@@ -160,29 +182,42 @@ NS_IMETHODIMP CPromptService::Confirm (nsIDOMWindow *parent,
 				       const PRUnichar *text,
 				       PRBool *_retval)
 {
+	GtkWidget *label;
+	GtkWidget *title_label;
 	GtkWidget *dialog;
-	gchar *gtext;
+	GladeXML *gxml;
 	gint ret;
 
-	/* convert arguments */
-	gtext = mozilla_unicode_to_locale (text);
+	/* build the dialog */
+	gxml = glade_widget_new ("prompts.glade", "confirm_dialog", 
+				 &dialog, NULL);
+	label       = glade_xml_get_widget (gxml, "label");
+	title_label = glade_xml_get_widget (gxml, "title_label");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
-	/* build and show the dialog */
-	dialog = gnome_message_box_new (gtext,
-					GNOME_MESSAGE_BOX_QUESTION,
-					GNOME_STOCK_BUTTON_YES,
-					GNOME_STOCK_BUTTON_NO,
-					NULL);
+	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
-	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	set_label_text (title_label, dialogTitle);
+	set_label_text (label, text);
 
-	/* capture return value */
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
+	/* run dialog and capture return values */
+	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 	*_retval = (ret == 0);
 
-	/* free */
-	g_free (gtext);
-
 	/* done */
+	gtk_widget_destroy (dialog);
 	return NS_OK;
 }
 
@@ -205,10 +240,12 @@ NS_IMETHODIMP CPromptService::ConfirmCheck (nsIDOMWindow *parent,
 	gint ret;
 
 	/* build and show the dialog */
-	gxml = glade_widget_new ("confirm_check_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "confirm_check_dialog", 
+				 &dialog, NULL);
 	label        = glade_xml_get_widget (gxml, "label");
 	title_label  = glade_xml_get_widget (gxml, "title_label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
+	gtk_object_unref (GTK_OBJECT (gxml));
 	
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -217,12 +254,24 @@ NS_IMETHODIMP CPromptService::ConfirmCheck (nsIDOMWindow *parent,
 	set_check_button_text (check_button, checkMsg);
 	set_check_button (check_button, checkValue);
 
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	/* handle return values */
 	get_check_button (check_button, checkValue);
-	*_retval = (ret == 1);
+	*_retval = (ret == 0);
 
 	/* done */
 	gtk_widget_destroy (dialog);
@@ -235,7 +284,11 @@ NS_IMETHODIMP CPromptService::ConfirmCheck (nsIDOMWindow *parent,
 NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent, 
 					 const PRUnichar *dialogTitle,
 					 const PRUnichar *text,
-					 PRUint32 button0And1Flags, 
+					 PRUint32 buttonFlags, 
+#if MOZILLA_VERSION > VERSION2(0,9)
+					 const PRUnichar *button0Title,
+					 const PRUnichar *button1Title,
+#endif
 					 const PRUnichar *button2Title,
 					 const PRUnichar *checkMsg, 
 					 PRBool *checkValue,
@@ -246,14 +299,16 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 	GtkWidget *title_label;
 	GtkWidget *check_button;
 	GladeXML *gxml;
-	gint *ret;
+	gint ret;
 	gint i;
 
 	/* build and show the dialog */
-	gxml = glade_widget_new ("confirmex_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "confirmex_dialog", 
+				 &dialog, NULL);
 	label        = glade_xml_get_widget (gxml, "label");
 	title_label  = glade_xml_get_widget (gxml, "title_label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
+	gtk_object_unref (GTK_OBJECT (gxml));
 	
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -265,7 +320,7 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 	/* create buttons */
 	for (i = 0; i < 4; i++)
 	{
-		switch (button0And1Flags & 0xff)
+		switch (buttonFlags & 0xff)
 		{
 		case BUTTON_TITLE_OK:
 			gnome_dialog_append_button (GNOME_DIALOG(dialog),
@@ -285,25 +340,62 @@ NS_IMETHODIMP CPromptService::ConfirmEx (nsIDOMWindow *parent,
 			break;
 		case BUTTON_TITLE_SAVE:
 			gnome_dialog_append_button (GNOME_DIALOG(dialog),
-						    "Save");
+						    _("Save"));
 			break;
 		case BUTTON_TITLE_REVERT:
 			gnome_dialog_append_button (GNOME_DIALOG(dialog),
-						    "Revert");
+						    _("Revert"));
 			break;
+#if MOZILLA_VERSION > VERSION2(0,9)
+		case BUTTON_TITLE_DONT_SAVE:
+			gnome_dialog_append_button (GNOME_DIALOG(dialog),
+						    _("Don't save"));
+			break;
+		case BUTTON_TITLE_IS_STRING:
+			{
+				gchar *tmpstr;
+				if (i==0)
+					tmpstr = mozilla_unicode_to_locale (
+							button0Title);
+				else if (i==1)
+					tmpstr = mozilla_unicode_to_locale (
+							button1Title);
+				else if (i==2)
+					tmpstr = mozilla_unicode_to_locale (
+							button2Title);
+				else
+					break;
+				gnome_dialog_append_button (
+						GNOME_DIALOG(dialog), tmpstr);
+				g_free (tmpstr);
+			}
+			break;
+#endif
 		default:
 			break;
 		}	
 
-		button0And1Flags >>= 8;
+		buttonFlags >>= 8;
 	}
 
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
-	*ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
+	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	/* handle return values */
 	get_check_button (check_button, checkValue);
-	buttonPressed = ret;
+	*buttonPressed = ret;
 
 	/* done */
 	gtk_widget_destroy (dialog);
@@ -330,11 +422,13 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	gint ret;
 
 	/* build and show the dialog */
-	gxml = glade_widget_new ("prompt_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "prompt_dialog", 
+				 &dialog, NULL);
 	entry        = glade_xml_get_widget (gxml, "entry");
 	label        = glade_xml_get_widget (gxml, "label");
 	title_label  = glade_xml_get_widget (gxml, "title_label");
 	check_button = glade_xml_get_widget (gxml, "check_button");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -344,13 +438,25 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	set_editable (entry, value);
 	set_check_button (check_button, checkValue);
 
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	/* handle return values */
 	get_check_button (check_button, checkValue);
 	get_editable (entry, value);
-	*_retval = (ret == 1);
+	*_retval = (ret == 0);
 
 	/* done */
 	gtk_widget_destroy (dialog);
@@ -380,11 +486,13 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	gint ret;
 
 	/* build and show the dialog */
-	gxml = glade_widget_new ("prompt_user_pass_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "prompt_user_pass_dialog", 
+				 &dialog, NULL);
 	title_label    = glade_xml_get_widget (gxml, "title_label");
 	check_button   = glade_xml_get_widget (gxml, "check_button");
 	username_entry = glade_xml_get_widget (gxml, "username_entry");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -394,6 +502,25 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	set_editable (password_entry, password);
 	set_check_button (check_button, checkValue);
 
+	/* make an enter keypress activate the "ok" button */
+	gnome_dialog_editable_enters (GNOME_DIALOG (dialog),
+				      GTK_EDITABLE (password_entry));
+
+	/* set the keyboard focus to the username entry */
+	gtk_widget_grab_focus (username_entry);
+
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
@@ -401,7 +528,7 @@ NS_IMETHODIMP CPromptService::PromptUsernameAndPassword
 	get_check_button (check_button, checkValue);
 	get_editable (username_entry, username);
 	get_editable (password_entry, password);
-	*_retval = (ret == 1);
+	*_retval = (ret == 0);
 
 	/* done */
 	gtk_widget_destroy (dialog);
@@ -428,10 +555,12 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 	gint ret;
 
 	/* build and show the dialog */
-	gxml = glade_widget_new ("prompt_pass_dialog", &dialog, NULL);
+	gxml = glade_widget_new ("prompts.glade", "prompt_pass_dialog", 
+				 &dialog, NULL);
 	title_label    = glade_xml_get_widget (gxml, "title_label");
 	check_button   = glade_xml_get_widget (gxml, "check_button");
 	password_entry = glade_xml_get_widget (gxml, "password_entry");
+	gtk_object_unref (GTK_OBJECT (gxml));
 
 	/* set dynamic attributes */
 	set_title (dialog, dialogTitle);
@@ -440,13 +569,25 @@ NS_IMETHODIMP CPromptService::PromptPassword (nsIDOMWindow *parent,
 	set_editable (password_entry, password);
 	set_check_button (check_button, checkValue);
 
+	GtkWidget *parentWidget = mozilla_find_gtk_parent (parent);
+	if (parentWidget)
+	{
+		gnome_dialog_set_parent (GNOME_DIALOG (dialog),
+					 GTK_WINDOW(parentWidget));
+	}
+	else
+	{
+	        gtk_widget_show(dialog);
+	        window_set_layer(dialog);
+	};
+
 	/* run dialog and capture return values */
 	ret = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
 
 	/* handle return values */
 	get_check_button (check_button, checkValue);
 	get_editable (password_entry, password);
-	*_retval = (ret == 1);
+	*_retval = (ret == 0);
 
 	/* done */
 	gtk_widget_destroy (dialog);
@@ -468,68 +609,7 @@ NS_IMETHODIMP CPromptService::Select (nsIDOMWindow *parent,
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-/**
- * CPromptServiceFactory: factory class
- */ 
-class CPromptServiceFactory : public nsIFactory
-{
-	public:
-		NS_DECL_ISUPPORTS
-		NS_DECL_NSIFACTORY
-
-		CPromptServiceFactory();
-		virtual ~CPromptServiceFactory();
-};
-
-NS_IMPL_ISUPPORTS1(CPromptServiceFactory, nsIFactory)
-
-/**
- * CPromptServiceFactory::CPromptServiceFactory: factory class constructor
- */ 
-CPromptServiceFactory::CPromptServiceFactory()
-{
-	NS_INIT_ISUPPORTS();
-}
-
-/**
- * CPromptServiceFactory::~CPromptServiceFactory: factory class destructor
- */ 
-CPromptServiceFactory::~CPromptServiceFactory()
-{
-}
-
-/**
- * CPromptServiceFactory::CreateInstance: create a single factory
- */ 
-NS_IMETHODIMP CPromptServiceFactory::CreateInstance(nsISupports *aOuter, 
-						    const nsIID & aIID, 
-						    void **aResult)
-{
-	NS_ENSURE_ARG_POINTER(aResult);
-  
-	*aResult = NULL;  
-	CPromptService *inst = new CPromptService;    
-	if (inst == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	nsresult rv = inst->QueryInterface(aIID, aResult);
-	if (rv != NS_OK)
-	{
-		delete inst;  
-	}  
-    
-	return rv;
-}
-
-/**
- * CPromptServiceFactory::LockFactory:
- */ 
-NS_IMETHODIMP CPromptServiceFactory::LockFactory(PRBool lock)
-{
-	return NS_OK;
-}
+NS_DEF_FACTORY (CPromptService, CPromptService);
 
 /**
  * NS_NewPromptServiceFactory:
@@ -539,7 +619,7 @@ nsresult NS_NewPromptServiceFactory(nsIFactory** aFactory)
 	NS_ENSURE_ARG_POINTER(aFactory);
 	*aFactory = nsnull;
 
-	CPromptServiceFactory *result = new CPromptServiceFactory;
+	nsCPromptServiceFactory *result = new nsCPromptServiceFactory;
 	if (result == NULL)
 	{
 		return NS_ERROR_OUT_OF_MEMORY;
@@ -563,7 +643,8 @@ set_title (GtkWidget *dialog, const PRUnichar *title)
 	gtitle = mozilla_unicode_to_locale (title);
 
 	/* set it */
-	gtk_window_set_title (GTK_WINDOW (dialog), gtitle);
+	gtk_window_set_title (GTK_WINDOW (dialog), 
+			      (gtitle == NULL ? N_("Galeon") : gtitle));
 
 	/* and free */
 	g_free (gtitle);
@@ -685,35 +766,3 @@ get_editable (GtkWidget *editable, PRUnichar **text)
 	/* free */
 	g_free (edited);
 }
-
-/**
- * promptService_find_window: retrives gtkwidget pointer to parent
- * window of domWindow
- * @domWindow: nsIDOMWindow to find gtk window parent of.
- */
-static GtkWidget *
-promptService_find_window (nsIDOMWindow *domWindow)
-{
-	nsresult result;
-	
-        nsCOMPtr<nsIWindowWatcher> wwatch 
-		(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
-        if (!wwatch) return nsnull;
-
-        nsCOMPtr<nsIWebBrowserChrome> windowChrome;
-        result = wwatch->GetChromeForWindow (domWindow,
-        				     getter_AddRefs(windowChrome));
-        if (NS_FAILED(result)) return nsnull;
-
-        nsCOMPtr<nsIEmbeddingSiteWindow> window
-        		(do_QueryInterface(windowChrome, &result));
-        if (NS_FAILED(result)) return nsnull;
-
-        GtkWidget *parentWindow;
-        result = window->GetSiteWindow ((void **)&parentWindow);
-        if (NS_FAILED(result)) return nsnull;
-
-        return parentWindow;
-}
-
-#endif /* CVS_SOURCE */
