@@ -17,6 +17,17 @@
  */
 
 #include "galeon.h"
+#include "auto_completion.h"
+
+#include <string.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkscrolledwindow.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-config.h>
+#include <gdk/gdkkeysyms.h>
 
 /**
  * Total number of completions to offer
@@ -59,16 +70,16 @@ const gchar *common_prefixes[] =
 };
 
 /* Private functions */
-static GList *auto_completion_insert_sorted_nodup(GList *l,char *url, int type);
-static GList *auto_completion_concat(GList *l1, GList *l2);
-static void auto_completion_alternatives_clist_select_row_cb (GtkCList *clist, gint row,
-		gint column, GdkEventButton *event, GtkWidget *entry);
+static GList *auto_completion_insert_sorted_nodup (GList *l, char *url,
+						   int type);
+static GList *auto_completion_concat (GList *l1, GList *l2);
+static void clist_select_row_cb (GtkCList *clist, gint row, gint column,
+				 GdkEventButton *event, GtkWidget *entry);
 static gchar *auto_completion_suggest (gchar *prefix);
-gchar *auto_completion_truncate_url(gchar *url);
-gchar *auto_completion_simplify_url(gchar *url);
+static gchar *auto_completion_simplify_url(gchar *url);
 static gboolean alternatives_win_button_press_cb (GtkWidget *widget,
-						GdkEventButton *event,
-						GtkWidget *alternatives_window);
+						  GdkEventButton *event,
+						  gpointer data); 
 static gboolean alternatives_win_key_press_cb (GtkWidget *widget,
 					       GdkEventKey *event,
 					       GtkWidget *location_entry);
@@ -80,7 +91,7 @@ static gboolean alternatives_win_key_press_cb (GtkWidget *widget,
 void
 auto_completion_add_url (gchar *url)
 {
-	gint len=0;
+	gint len = 0;
 	GList *list = NULL;
 	gshort i;
 
@@ -123,6 +134,8 @@ auto_completion_reset (void)
 
 	if (alternatives_window != NULL)
 	{
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		gtk_grab_remove (alternatives_window);
 		gtk_widget_destroy (alternatives_window);
 	}
 	alternatives_window = NULL;
@@ -148,10 +161,6 @@ auto_completion_complete_url (gchar *prefix)
 	/* try to complete */
 	tmp = g_completion_complete (URLCompletion, prefix, &common);
 
-	/* changed by MattA on 5/2/2001: I think this is a more
-	 * desirable behaviour (it's more like IE) -- what does 
-	 * everyone else think? */
-
 	/* this might be NULL */
 	return common;
 }
@@ -307,8 +316,8 @@ auto_completion_suggest (gchar *prefix)
  * The returned string mustn't be modified nor freed
  */
 
-gchar *
-auto_completion_simplify_url(gchar *url)
+static gchar *
+auto_completion_simplify_url (gchar *url)
 {
 	unsigned char i = 0;
 	unsigned char url_length = 0;
@@ -323,33 +332,6 @@ auto_completion_simplify_url(gchar *url)
 }
 
 
-/**
- * REMOVED If someone wants to readd it, 
- * add a preference for disabling it -- ricardo
- *
- * Strips the url before the first /, ? or #
- * Deals with file:// (does nothing) and the xxx:// suffixes (skips them)
- * You must free the returned string
- */
-
-gchar *
-auto_completion_truncate_url(gchar *url)
-{
-	gchar *newurl;
-
-	newurl = strstr(url, "file:");	
-	if (newurl == NULL) {
-		newurl = g_strdup(auto_completion_simplify_url (url));
-		while ((*newurl != '\0') && (*newurl != '/') && (*newurl != '?') && (*newurl !='#'))
-			newurl++;
-		*newurl = '\0';
-	} else {
-		newurl = g_strdup (url);
-	}
-	return newurl;
-}
-
-
 /**
  * insert the string data into the given glist 
  * the list returned is sorted and without duplicates 
@@ -357,7 +339,7 @@ auto_completion_truncate_url(gchar *url)
  * simplified urls
  */
 static GList *
-auto_completion_insert_sorted_nodup(GList *l, char *url, int type)
+auto_completion_insert_sorted_nodup (GList *l, char *url, int type)
 {
 	GList *tmp = l;
 	int i = 0;
@@ -395,7 +377,7 @@ auto_completion_insert_sorted_nodup(GList *l, char *url, int type)
  * string to the end of the url
  */
 void
-auto_completion_display(GaleonWindow *window, GtkWidget *entry)
+auto_completion_display (GaleonWindow *window, GtkWidget *entry)
 {
 	gchar *text;
 	gint text_len;
@@ -428,13 +410,14 @@ auto_completion_display(GaleonWindow *window, GtkWidget *entry)
 }
 
 /**
- * Displays the list of possible completions under the location box of the given browser
+ * Displays the list of possible completions under the location box of the
+ * given browser
  */
 void 
 auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 {
 	int sort = gnome_config_get_int (CONF_COMPLETION_SORT);
-	gint x, y, height, width, depth;
+	gint x, y, height, width;
 	GtkWidget *scroll = gtk_scrolled_window_new (NULL, NULL);
 	GtkWidget *clist = gtk_clist_new (1); 
 	GtkRequisition r;
@@ -443,7 +426,9 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 	GList *labels = NULL;
 	
 	if (alternatives_window)
+	{
 		gtk_widget_destroy (alternatives_window);
+	}
 	alternatives_window = gtk_window_new (GTK_WINDOW_POPUP);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
@@ -452,13 +437,13 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 	gtk_container_add (GTK_CONTAINER (scroll), clist);
 
 	gtk_signal_connect (GTK_OBJECT(clist), "select_row", 
-			    GTK_SIGNAL_FUNC(auto_completion_alternatives_clist_select_row_cb), 
-			    entry);
+			    GTK_SIGNAL_FUNC (clist_select_row_cb), entry);
 	
 	for (l = possible_completions; (l!=NULL) && (count < MAX_ALTERNATIVES);
 	     l = g_list_next(l), count++) 
 	{
-		labels = auto_completion_insert_sorted_nodup(labels, l->data, sort);
+		labels = auto_completion_insert_sorted_nodup(labels, l->data,
+		                                             sort);
 	};
 
 	for (l =  labels; (l != NULL); l = g_list_next(l)) 
@@ -467,7 +452,12 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 		gtk_clist_append (GTK_CLIST (clist), text);
 	}	
 	g_list_free(labels);
-	gdk_window_get_geometry (entry->window, &x, &y, &width, &height, &depth);
+
+	/* Urgh. As of GTK+ 1.2.10, this isn't getting initialized correctly,
+	   so we have to do it ourselves */
+	GTK_CLIST (clist)->focus_row = -1;
+
+	gdk_window_get_size (entry->window,&width, &height);
 	gdk_window_get_deskrelative_origin (entry->window, &x, &y);
 	y += height;
 	gtk_widget_set_uposition (alternatives_window, x, y);
@@ -505,10 +495,8 @@ auto_completion_display_alternatives (GaleonWindow *window, GtkWidget *entry)
 }
 
 static void 
-auto_completion_alternatives_clist_select_row_cb (GtkCList *clist,
-						  gint row, gint column,
-						  GdkEventButton *event,
-						  GtkWidget *entry)
+clist_select_row_cb (GtkCList *clist, gint row, gint column,
+		     GdkEventButton *event, GtkWidget *entry)
 {
 	GdkEventKey tmp_event;
 
@@ -529,35 +517,14 @@ auto_completion_alternatives_clist_select_row_cb (GtkCList *clist,
 	}
 }
 
-/** 
- * Adds the history of a GnomeEntry to the list of completions 
- */
-/* Actually, I think that this may be unnecessary. */
-void 
-auto_completion_add_from_entry (GnomeEntry *e) 
-{
-	static gboolean done = FALSE;
-	if (!done) { /* do this only once */
-		struct item {
-			gboolean save;
-			gchar *text;
-		};
-		GList *list;
-		for (list = e->items; list != NULL; list = list->next) {
-			struct item *i = list->data;
-			auto_completion_add_url (i->text);
-		}	
-	}
-}
-
 static gboolean
 alternatives_win_button_press_cb (GtkWidget *widget,
 				  GdkEventButton *event,
-				  GtkWidget *alternatives_window)
+				  gpointer data)
 {
 	GtkWidget *event_widget;
 
-	event_widget = gtk_get_event_widget((GdkEvent *)event);
+	event_widget = gtk_get_event_widget ((GdkEvent *)event);
 
 	/* Check to see if button press happened inside the alternatives
 	   window.  If not, destroy the window. */
@@ -587,11 +554,14 @@ alternatives_win_key_press_cb (GtkWidget *widget,
 	if (event->keyval == GDK_Up || event->keyval == GDK_Down ||
 	    event->keyval == GDK_Page_Up ||  event->keyval == GDK_Page_Down ||
 	    event->keyval == GDK_space)
+	{
 		return FALSE;
+	}
 	
 	if (event->keyval == GDK_Return)
 	{
-		if (!GTK_CLIST_CHILD_HAS_FOCUS (clist)) {
+		if (clist->focus_row >= 0)
+		{
 			event->keyval = GDK_space;
 			return FALSE;
 		}
@@ -609,6 +579,11 @@ alternatives_win_key_press_cb (GtkWidget *widget,
 	tmp_event.string = event->string;
 	gtk_widget_event(location_entry, (GdkEvent *)&tmp_event);
 
+	/* hmm... something seems to have changed in the GTK+ signal handling
+	   code.  Returning true no longer seems to stop emission of the key
+	   press signal in the clist window, so we have to do it manually */
+	gtk_signal_emit_stop_by_name (GTK_OBJECT(widget), "key-press-event");
+
 	return TRUE;
 }
 
