@@ -16,7 +16,24 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Galeon includes */
 #include "galeon.h"
+#include "misc.h"
+#include "embed.h"
+#include "auto_completion.h"
+#include "bookmarks.h"
+#include "window.h"
+
+/* GNOME includes */
+#include <stdlib.h>
+#include <string.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnome/gnome-util.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-config.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtkdnd.h>
 
 /** The pixmaps for the bookmarks */
 PixmapData *folder_pixmap_data = NULL;
@@ -27,7 +44,10 @@ PixmapData *site_pixmap_data = NULL;
 BookmarkItem *bookmarks_root = NULL;
 
 /** The root of the temporary bookmark tree */
-BookmarkItem *temp_bookmarks_root = NULL;
+BookmarkItem *default_bookmarks_root = NULL;
+
+/** The bookmarks need to be saved */
+gboolean bookmarks_dirty = FALSE;
 
 /** The next id to assign (for alias saving) */
 static long maxid = 0;
@@ -40,29 +60,54 @@ GHashTable *favicons_cache = NULL;
 
 /* DnD */
 const GtkTargetEntry bookmarks_dnd_targets [] = {
-	{ "GALEON_BOOKMARK", 0, DND_TARGET_GALEON_BOOKMARK},
-	{ "GALEON_URL", 0, DND_TARGET_GALEON_URL },
-	{ "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL},
-	{ "text/uri-list", 0, DND_TARGET_TEXT_URI_LIST},
-	{ "STRING", 0, DND_TARGET_STRING}
+	{ "GALEON_BOOKMARK", 0, DND_TARGET_GALEON_BOOKMARK },
+	{ "GALEON_URL",      0, DND_TARGET_GALEON_URL      },
+	{ "_NETSCAPE_URL",   0, DND_TARGET_NETSCAPE_URL    },
+	{ "text/uri-list",   0, DND_TARGET_TEXT_URI_LIST   },
+	{ "STRING",          0, DND_TARGET_STRING          }
 };
-const gint bookmarks_dnd_targets_num_items = (
-	sizeof (bookmarks_dnd_targets) / sizeof (GtkTargetEntry));
+const gint bookmarks_dnd_targets_num_items = (sizeof (bookmarks_dnd_targets) /
+					      sizeof (GtkTargetEntry));
 
 /* Private prototypes */
+static void bookmarks_load_icons (void);
 static BookmarkItem *bookmarks_xml_read_item (xmlNodePtr item);
 static BookmarkItem *bookmarks_xml_read (xmlNodePtr item);
 static void bookmarks_add_completion_recursively (BookmarkItem *b);
 static void bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root,
 					BookmarkItem *b);
-static BookmarkItem *bookmarks_find_by_nick (const gchar *nick,
-					     BookmarkItem *root,
-					     gboolean wantsmart);
 static void bookmarks_remove_all_alias (BookmarkItem *b);
 static void bookmarks_new_alias_do (BookmarkItem *b, BookmarkItem *bookmark);
 static BookmarkItem *bookmarks_find_by_id (long id,
 					   BookmarkItem *root);
 
+/**
+ * bookmarks_init: intialise bookmarks for the first time
+ */
+void
+bookmarks_init (void)
+{
+	bookmarks_load_icons ();
+	bookmarks_load ();
+}
+
+/**
+ * bookmarks_exit: shut down bookmarks
+ */
+void
+bookmarks_exit (void)
+{
+	bookmarks_save ();
+
+	g_free (folder_open_pixmap_data);
+	g_free (folder_pixmap_data);
+	g_free (site_pixmap_data);
+
+	folder_open_pixmap_data = NULL;
+	folder_pixmap_data = NULL;
+	site_pixmap_data = NULL;
+}
+
 /**
  * bookmarks_load: load bookmarks from xml file
  */
@@ -71,8 +116,6 @@ bookmarks_load (void)
 {
 	gchar *bookfile;
 	
-	bookmarks_load_icons ();
-	
 	bookfile = g_concat_dir_and_file (g_get_home_dir (), 
 					  "/.galeon/bookmarks.xml");
 	
@@ -82,6 +125,10 @@ bookmarks_load (void)
 		bookmarks_root = bookmarks_new_bookmark 
 			(BM_CATEGORY, TRUE, _("Bookmarks"), NULL, NULL, NULL, NULL);
 	}
+	
+	if (default_bookmarks_root == NULL) {
+		default_bookmarks_root = bookmarks_root;
+	}
 
 	/* generate the autobookmarks category */
 	autobookmarks_generate();
@@ -94,19 +141,32 @@ bookmarks_load (void)
 /**
  * bookmarks_load_icons: load the bookmarks icons
  */
-void 
+static void 
 bookmarks_load_icons (void) 
 {
-	folder_pixmap_data = pixmap_data_new_from_file (
-						SHARE_DIR "/dir.xpm");
-	
-	folder_open_pixmap_data = pixmap_data_new_from_file (
-						SHARE_DIR "/dir_open.xpm");
-	
-	site_pixmap_data = pixmap_data_new_from_file (
-						SHARE_DIR "/i-bookmark.xpm");
-			
+	folder_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/dir.xpm");
+	folder_open_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/dir_open.xpm");
+	site_pixmap_data = 
+		pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
 	/* TODO: add icons for alias */
+
+	/* if we didn't manage to load the icons, make an empty structure
+	 * anyway, this allows the browser to run even when not fully 
+	 * installed.... */
+	if (folder_pixmap_data == NULL)
+	{
+		folder_pixmap_data = g_new0 (PixmapData, 1);
+	}
+	if (folder_open_pixmap_data == NULL)
+	{
+		folder_open_pixmap_data = g_new0 (PixmapData, 1);
+	}
+	if (site_pixmap_data == NULL)
+	{
+		site_pixmap_data = g_new0 (PixmapData, 1);
+	}
 }
 
 /**
@@ -192,8 +252,8 @@ bookmarks_xml_read_item (xmlNodePtr item)
 {
 	BookmarkItem *b;
 	BookmarkItem *b2;
-	gchar *ct, *expanded, *ccm;
-	
+	gchar *ct, *expanded, *ccm, *default_root;
+
 	gchar *name = xmlGetPropISOLatin1 (item, "name");	
 	gchar *notes = xmlGetProp (item, "notes");
 	gchar *nick = xmlGetProp (item, "nick");
@@ -205,33 +265,48 @@ bookmarks_xml_read_item (xmlNodePtr item)
   
 	g_return_val_if_fail(item != NULL, NULL);
 
-	if (!strcmp (item->name, "category")) {
-		b = bookmarks_new_bookmark (BM_CATEGORY, FALSE, name, NULL, nick, notes,
-					    pixmap_file);
+	if (!strcmp (item->name, "category")) 
+	{
+		b = bookmarks_new_bookmark (BM_CATEGORY, FALSE, name, NULL,
+					    nick, notes, pixmap_file);
 		ct = xmlGetProp (item, "create_toolbar");
 		ccm = xmlGetProp (item, "create_context_menu");
 		expanded = xmlGetProp (item, "expanded");
-		b->create_toolbar = (ct && !strcmp (ct, "TRUE")) ? TRUE : FALSE;
-		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE")) ? TRUE : FALSE;
-		b->expanded = (expanded && !strcmp (expanded, "TRUE")) ? TRUE : FALSE;
-		xmlFree (ct);
+	        default_root = xmlGetProp (item, "default_bookmarks_root");
+		b->create_toolbar = (ct && !strcmp (ct, "TRUE") ? 
+				     TRUE : FALSE);
+		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE") ? 
+					  TRUE : FALSE);
+		b->expanded = (expanded && !strcmp (expanded, "TRUE") ? 
+			       TRUE : FALSE);
+		if (default_root && !strcmp (default_root, "TRUE"))
+		{
+			default_bookmarks_root = b;
+		}
+	        xmlFree (ct);		
 		xmlFree (ccm);
 		xmlFree (expanded);
+		xmlFree (default_root);
 		item = item->childs;
-		while (item != NULL) {
+		while (item != NULL)
+		{
 			xmlNodePtr item2 = item;
 			b2 = bookmarks_xml_read_item (item2);
 			b->list = g_list_append (b->list, b2);
 			b2->parent = b;
 			item = item->next;
 		}
-	} else if (!strcmp (item->name, "separator")) {
-		b = bookmarks_new_bookmark (BM_SEPARATOR, FALSE, name, NULL, nick, notes,
-					    pixmap_file);
-	} else if (!strcmp (item->name, "autobookmarks")) {
+	} 
+	else if (!strcmp (item->name, "separator"))
+	{
+		b = bookmarks_new_bookmark (BM_SEPARATOR, FALSE, name, NULL,
+					    nick, notes, pixmap_file);
+	} 
+	else if (!strcmp (item->name, "autobookmarks"))
+	{
 		/* FIXME: remove code duplication */
-		b = bookmarks_new_bookmark (BM_AUTOBOOKMARKS, FALSE, name, NULL, nick,
-					    notes, pixmap_file);
+		b = bookmarks_new_bookmark (BM_AUTOBOOKMARKS, FALSE, name, 
+					    NULL, nick, notes, pixmap_file);
 		ct = xmlGetProp (item, "create_toolbar");
 		ccm = xmlGetProp (item, "create_context_menu");
 		expanded = xmlGetProp (item, "expanded");
@@ -242,27 +317,36 @@ bookmarks_xml_read_item (xmlNodePtr item)
 		xmlFree (ccm);
 		xmlFree (expanded);
 		autobookmarks_root = b;
-	} else if (!strcmp (item->name, "alias")) {
+	} 
+	else if (!strcmp (item->name, "alias")) 
+	{
 		b = bookmarks_new_alias (NULL);
 		unresolved_aliases = g_list_prepend (unresolved_aliases, b);
-	} else { /* site */
+	} 
+	else 
+	{
+		/* site */
 		gchar *url = xmlGetProp (item, "url");
-		b = bookmarks_new_bookmark (BM_SITE, FALSE, name, url, nick, notes,
-					    pixmap_file);
+		b = bookmarks_new_bookmark (BM_SITE, FALSE, name, url, 
+					    nick, notes, pixmap_file);
 		ccm = xmlGetProp (item, "create_context_menu");
-		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE")) ? TRUE : FALSE;
+		b->create_context_menu = (ccm && !strcmp (ccm, "TRUE") ? 
+					  TRUE : FALSE);
 		xmlFree (ccm);
 		if (url) g_free (url);
 	}
-	if (time_added_str) {
+	if (time_added_str)
+	{
 		b->time_added = strtol (time_added_str, NULL, 10);
 		g_free (time_added_str);
 	}
-	if (time_visited_str) {
+	if (time_modified_str)
+	{
 		b->time_modified = strtol (time_modified_str, NULL, 10);
 		g_free (time_modified_str);
 	}
-	if (time_visited_str) {
+	if (time_visited_str)
+	{
 		b->time_visited = strtol (time_visited_str, NULL, 10);
 		g_free (time_visited_str);
 	}
@@ -277,31 +361,64 @@ bookmarks_xml_read_item (xmlNodePtr item)
 	return b;
 }
 
+/**
+ * bookmarks_updated: called when one or more change has taken place in 
+ * the bookmarks which we wish to record, i.e. save to disk and display
+ * in the menus and toolbars.
+ */
 void
-temp_bookmarks_save (void)
+bookmarks_updated (void)
 {
-	char *file = g_strconcat (g_get_home_dir(), "/.galeon/temp_bookmarks.xml", NULL);
-	if (temp_bookmarks_root != NULL) {
-		bookmarks_save_as (temp_bookmarks_root, file);
+	GList *l;
+
+	/* force a save */
+	bookmarks_dirty = TRUE;
+	bookmarks_save ();
+
+	/* update each window */
+	for (l = all_windows; l != NULL; l = g_list_next (l))
+	{
+		GaleonWindow *window = (GaleonWindow *)(l->data);
+
+		/* save toolbar layout */
+		window_save_layout (window);
+	
+		/* update bookmarks menubar */
+		bookmarks_menu_recreate (window);
+	
+		/* update bookmarks toolbars */
+		bookmarks_toolbars_recreate (window);
+
+		/* restore toolbar layout */
+		window_restore_layout (window);
 	}
-	g_free (file);
+
+	/* FIXME: update applet... */
 }
 
 /**
  * bookmarks_save: save all bookmarks to the xml file
  */
 void
-bookmarks_save (void) 
+bookmarks_save (void)
 {
-	bookmarks_update_menu ();
-	bookmarks_update_tb ();
-	if ((!bookmarks_editor) || (bookmarks_editor->dirty)) {
-		char *savefile = g_strconcat (g_get_home_dir(), 
-					      "/.galeon/bookmarks.xml", NULL);
-		bookmarks_save_as (bookmarks_root, savefile);
-		if (bookmarks_editor) bookmarks_editor->dirty = FALSE;
-		g_free (savefile);
+	gchar *filename;
+
+	if (!bookmarks_dirty)
+	{
+		return;
 	}
+
+	/* FIXME: refactor into bookmarks_filename ... */
+	filename = g_strconcat (g_get_home_dir (), 
+				"/.galeon/bookmarks.xml", NULL);
+
+	/* save the bookmarks */
+	bookmarks_save_as (bookmarks_root, filename);
+	bookmarks_dirty = FALSE;
+
+	/* free allocated string */
+	g_free (filename);
 }
 
 /**
@@ -336,37 +453,54 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 	gchar *idstr = g_strdup_printf ("%ld", b->id);
 	GString *s = g_string_new ("");
 
-	if (b->alias_of) {
+	if (b->alias_of)
+	{
 		node = xmlNewDocNode (doc, NULL, "alias", NULL);
 		xmlSetProp (node, "id", idstr);
-		xmlSetProp (node, "create_toolbar", 
-			    b->create_toolbar ? "TRUE" : "FALSE");
-		xmlSetProp (node, "create_context_menu", 
-			    b->create_context_menu ? "TRUE" : "FALSE");
-		g_string_sprintf (s, "%d", b->time_added);
-		xmlSetProp (node, "time_added", s->str);
+		if (b->create_toolbar)
+			xmlSetProp (node, "create_toolbar", "TRUE");
+		if (b->create_context_menu)
+			xmlSetProp (node, "create_context_menu", "TRUE");
+		if (b->time_added) {
+			g_string_sprintf (s, "%d", b->time_added);
+			xmlSetProp (node, "time_added", s->str);
+		}
 		xmlAddChild (root, node);
-	} else {
-		switch (b->type) {
+	} 
+	else
+	{
+		switch (b->type)
+		{
 		case BM_CATEGORY:
 			node = xmlNewDocNode (doc, NULL, "category", NULL);
 			xmlSetProp (node, "id", idstr);
 			xmlSetPropISOLatin1 (node, "name", b->name);
-			xmlSetProp (node, "create_toolbar", 
-				    b->create_toolbar ? "TRUE" : "FALSE");
-			xmlSetProp (node, "create_context_menu", 
-				    b->create_context_menu ? "TRUE" : "FALSE");
-			xmlSetProp (node, "expanded", 
-				    b->expanded ? "TRUE" : "FALSE");
-			xmlSetProp(node, "notes", b->notes);
-			if (b->pixmap_file)
+			if (b->create_toolbar)
+				xmlSetProp (node, "create_toolbar", "TRUE");
+			if (b->create_context_menu)
+				xmlSetProp (node, "create_context_menu",  "TRUE");
+			if (b->expanded)
+				xmlSetProp (node, "expanded", "TRUE");
+			if (*b->notes)
+				xmlSetProp (node, "notes", b->notes);
+			if (default_bookmarks_root == b) {
+				xmlSetProp (node, "default_bookmarks_root", 
+					    "TRUE");
+			}
+			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp(node, "pixmap", b->pixmap_file);
-			g_string_sprintf (s, "%d", b->time_added);
-			xmlSetProp (node, "time_added", s->str);
-			g_string_sprintf (s, "%d", b->time_modified);
-			xmlSetProp (node, "time_modified", s->str);
-			g_string_sprintf (s, "%d", b->time_visited);
-			xmlSetProp (node, "time_visited", s->str);
+			if (b->time_added) {
+				g_string_sprintf (s, "%d", b->time_added);
+				xmlSetProp (node, "time_added", s->str);
+			}
+			if (b->time_modified) {
+				g_string_sprintf (s, "%d", b->time_modified);
+				xmlSetProp (node, "time_modified", s->str);
+			}
+			if (b->time_visited) {
+				g_string_sprintf (s, "%d", b->time_visited);
+				xmlSetProp (node, "time_visited", s->str);
+			}
 			xmlAddChild (root, node);
 			
 			for (li = b->list; li != NULL; li = li->next) {
@@ -385,18 +519,26 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 			xmlSetProp (node, "id", idstr);
 			xmlSetPropISOLatin1 (node, "name", b->name);
 			xmlSetProp(node, "url", b->url);    
-			xmlSetProp(node, "nick", b->nick);
-			xmlSetProp(node, "notes", b->notes);
-			xmlSetProp (node, "create_context_menu", 
-				    b->create_context_menu ? "TRUE" : "FALSE");
-			if (b->pixmap_file)
+			if (*b->nick)
+				xmlSetProp(node, "nick", b->nick);
+			if (*b->notes)
+				xmlSetProp(node, "notes", b->notes);
+			if (b->create_context_menu)
+				xmlSetProp (node, "create_context_menu", "TRUE");
+			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp(node, "pixmap", b->pixmap_file);
-			g_string_sprintf (s, "%d", b->time_added);
-			xmlSetProp (node, "time_added", s->str);
-			g_string_sprintf (s, "%d", b->time_modified);
-			xmlSetProp (node, "time_modified", s->str);
-			g_string_sprintf (s, "%d", b->time_visited);
-			xmlSetProp (node, "time_visited", s->str);
+			if (b->time_added) {
+				g_string_sprintf (s, "%d", b->time_added);
+				xmlSetProp (node, "time_added", s->str);
+			}
+			if (b->time_modified) {
+				g_string_sprintf (s, "%d", b->time_modified);
+				xmlSetProp (node, "time_modified", s->str);
+			}
+			if (b->time_visited) {
+				g_string_sprintf (s, "%d", b->time_visited);
+				xmlSetProp (node, "time_visited", s->str);
+			}
 			xmlAddChild(root, node);
 			break;
 			
@@ -404,14 +546,15 @@ bookmarks_save_recursively (xmlDocPtr doc, xmlNodePtr root, BookmarkItem *b)
 			node = xmlNewDocNode(doc, NULL, "autobookmarks", NULL);
 			xmlSetProp (node, "id", idstr);
 			xmlSetPropISOLatin1 (node, "name", b->name);
-			xmlSetProp (node, "create_toolbar", 
-				    b->create_toolbar ? "TRUE" : "FALSE");
-			xmlSetProp (node, "create_context_menu", 
-				    b->create_context_menu ? "TRUE" : "FALSE");
-			xmlSetProp (node, "expanded", 
-				    b->expanded ? "TRUE" : "FALSE");
-			xmlSetProp(node, "notes", b->notes);
-			if (b->pixmap_file)
+			if (b->create_toolbar)
+				xmlSetProp (node, "create_toolbar", "TRUE");
+			if (b->create_context_menu)
+				xmlSetProp (node, "create_context_menu", "TRUE");
+			if (b->expanded)
+				xmlSetProp (node, "expanded", "TRUE");
+			if (*b->notes)
+				xmlSetProp(node, "notes", b->notes);
+			if (b->pixmap_file && *b->pixmap_file)
 				xmlSetProp(node, "pixmap", b->pixmap_file);
 			xmlAddChild(root, node);
 			break;
@@ -433,7 +576,7 @@ bookmarks_remove_recursively (BookmarkItem *b) {
 		position = g_list_find (b->parent->list, b);
 		b->parent->list = 
 			g_list_remove_link (b->parent->list, position);
-		bookmarks_update_alias (NULL, b->parent);
+		bookmarks_update_alias (b->parent);
 	}
 
 	if (!b->alias_of) 
@@ -448,6 +591,8 @@ bookmarks_remove_recursively (BookmarkItem *b) {
 	if ((b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS) && !b->alias_of)
 		for (li = b->list; li != NULL; li = li->next) 
 			bookmarks_remove_recursively (li->data);
+	
+	if (b == default_bookmarks_root) default_bookmarks_root = bookmarks_root;
 
 	bookmarks_free_bookmark (b);
 }
@@ -509,7 +654,7 @@ bookmarks_new_bookmark (BookmarkType type, gboolean escape_name,
 		break;
 	}
 	bookmark->list = NULL;
-	bookmark->tree_item = NULL;
+	bookmark->tree_items = NULL;
 	bookmark->parent = NULL;
 	bookmark->create_toolbar = FALSE;
 	bookmark->create_context_menu = FALSE;
@@ -549,7 +694,7 @@ BookmarkItem *
 bookmarks_new_alias (BookmarkItem *b)
 {
 	BookmarkItem *bookmark = g_new0 (BookmarkItem, 1);
-	bookmark->tree_item = NULL;
+	bookmark->tree_items = NULL;
 	bookmark->parent = NULL;
 	bookmark->create_toolbar = FALSE;
 	bookmark->create_context_menu = FALSE;
@@ -585,7 +730,7 @@ bookmarks_new_alias_do (BookmarkItem *b, BookmarkItem *bookmark)
  * Updates all aliases of a bookmark
  */
 void
-bookmarks_update_alias (BookmarksEditorControls *c, BookmarkItem *b)
+bookmarks_update_alias (BookmarkItem *b)
 {
 	BookmarkItem *alias = b->alias;
 	if (!alias) return;
@@ -599,9 +744,8 @@ bookmarks_update_alias (BookmarksEditorControls *c, BookmarkItem *b)
 	alias->id = b->id;
 	alias->time_visited = b->time_visited;
 	alias->time_modified = b->time_modified;
-	if (c)
-		bookmarks_editor_update_tree_item (c, alias);
-	bookmarks_update_alias (c, alias);
+	bookmarks_editor_update_tree_items (alias);
+	bookmarks_update_alias (alias);
 }
 
 /**
@@ -694,10 +838,13 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 	gint position;
 	
 	if (near == NULL)
+	{
 		parent = bookmarks_root;
+	}
 	else {
-		if (insert_pos == GTK_CLIST_DRAG_INTO &&
-		    (near->type == BM_CATEGORY || near->type == BM_AUTOBOOKMARKS))
+		if (insert_pos == GTK_CLIST_DRAG_INTO 
+		    && (near->type == BM_CATEGORY || near->type == BM_AUTOBOOKMARKS)
+		    && !near->alias_of)
 			parent = near;
 		else
 			parent = near->parent;
@@ -731,7 +878,7 @@ bookmarks_insert_bookmark (BookmarkItem *b, BookmarkItem *near,
 		}
 		parent->list = g_list_insert (parent->list, b, position);
 	}
-	bookmarks_update_alias (NULL, parent);
+	bookmarks_update_alias (parent);
 }
 
 /**
@@ -768,8 +915,9 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 		if (other->type == BM_CATEGORY) {
 			/* expand the category and move b into it */
 			parent->list = g_list_remove_link (parent->list, pos);
-			bookmarks_update_alias (controls, parent);
-			gtk_ctree_expand (GTK_CTREE (controls->ctree), other->tree_item);
+			bookmarks_update_alias (parent);
+			gtk_ctree_expand (GTK_CTREE (controls->ctree), 
+					bookmarks_get_tree_item (other, controls));
 			
 			if (!where) { 
 				/* up */
@@ -791,7 +939,7 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 			return 1;
 		
 		parent->list = g_list_remove_link(parent->list, pos);
-		bookmarks_update_alias (controls, parent);
+		bookmarks_update_alias (parent);
 		
 		if (!where) {
 			/* up */
@@ -809,22 +957,27 @@ bookmarks_move_bookmark (BookmarksEditorControls *controls,
 			b->parent = grandparent;
 		}
 	}
-	bookmarks_editor_place_tree_item (controls, b);
-	bookmarks_update_alias (controls, b->parent);
-	gtk_ctree_select ( GTK_CTREE (controls->ctree), b->tree_item);
+	bookmarks_editor_place_tree_items (b);
+	bookmarks_update_alias (b->parent);
+	gtk_ctree_select ( GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls));
 
 	return 0;
 }
 
 /**
- * Renames a bookmark (wich has just been added) or deletes it if the user cancelled 
+ * Renames a bookmark (which has just been added) or deletes it if the 
+ * user cancelled
  */
 void 
 bookmarks_string_request_callback (gchar *string, BookmarkItem *b) 
 {
-	if (!string) { /* User canceled */
+	if (string == NULL) 
+	{ 
+                /* user canceled */
 		bookmarks_remove_recursively (b);
-	} else { 
+	} 
+	else 
+	{ 
 		g_free (b->name);
 		b->name = string;
 		
@@ -836,17 +989,17 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
 
 		if (b->url &&
 		    gnome_config_get_bool (CONF_GENERAL_FAVICONS_ENABLED))
+		{
 			get_siteicon (NULL, b->url);
+		}
 		
-		if (bookmarks_editor)
-			bookmarks_editor_place_tree_item (bookmarks_editor, b);
-		bookmarks_update_menu ();
-		bookmarks_save (); 
+		bookmarks_editor_place_tree_items (b);
+		bookmarks_updated (); 
 	}
 }
 
 /**
- * add_temp_bookmark: Adds a temporal bookmark
+ * add_bookmark_default: Adds a bookmark to the default bookmark folder
  * @type: The type of the bookmark
  * @name: The name of the bookmark, if NULL or "" the url will be used instead.
  * @url: The url of the bookmark
@@ -855,31 +1008,54 @@ bookmarks_string_request_callback (gchar *string, BookmarkItem *b)
  * Return value: The new bookmark
  **/
 BookmarkItem *
-add_temp_bookmark (BookmarkType type, const gchar *name, const gchar *url, 
-		   BookmarkItem *parent)
+add_bookmark_default (BookmarkType type, const gchar *name, const gchar *url, 
+		      BookmarkItem *parent)
 {
 	BookmarkItem *bookmark;
 
 	bookmark = bookmarks_new_bookmark (type, TRUE, name, url, NULL, NULL, NULL);
 
-	if (!temp_bookmarks_root) {
-		temp_bookmarks_init ();
-	}
-	if (!parent) parent = temp_bookmarks_root;
+	if (!parent) parent = default_bookmarks_root;
 	
 	bookmark->parent = parent;
 	parent->list = g_list_append (parent->list, bookmark);
 	bookmark->time_added = time (NULL);
 
-	if (temp_bookmarks_window != NULL) 
-	{
-		bookmarks_editor_place_tree_item (temp_bookmarks_window, 
-						  bookmark);
-		temp_bookmarks_window->dirty = TRUE;
-	}
+	bookmarks_editor_place_tree_items (bookmark);
+	bookmarks_updated ();
 
-	/* Save the temp bookmarks, for safety in case of a crash */
-	temp_bookmarks_save ();
+	return bookmark;
+}
+
+/**
+ * add_temp_link_bookmark: Adds a temporal bookmark of a link
+ * @name: The link text
+ * @url: The url of the bookmark
+ * @parent: The parent bookmark or NULL if the bookmark should be added to the 
+ * main temp category
+ * Return value: The new bookmark
+ **/
+BookmarkItem *
+add_temp_link_bookmark (const gchar *name, const gchar *url, BookmarkItem *parent)
+{
+	BookmarkItem *bookmark;
+	gchar *newname;
+	gint shorttext = gnome_config_get_int ("/galeon/TempBookmarks/shorttext=16");
+
+	/**
+	 * if shorttext is < 0, use old url only behavior,
+	 * if >=0, it is the length above which only the link text will be used
+	 */
+	
+	if (shorttext < 0)
+		newname = g_strdup (url);
+	else if (strlen(name) <= shorttext)
+		newname = g_strdup_printf ("%s - %s", name, url);
+	else
+		newname = g_strdup (name);
+
+	bookmark = add_bookmark_default (BM_SITE, newname, url, parent);
+	g_free (newname);
 
 	return bookmark;
 }
@@ -1068,7 +1244,7 @@ bookmarks_substitute_argument (BookmarkItem *b, const gchar *arguments)
  * 
  * Return value: the found BookmarkItem or NULL if nothing found
  **/
-static BookmarkItem *
+BookmarkItem *
 bookmarks_find_by_nick (const gchar *nick, BookmarkItem *r, gboolean wantsmart)
 {
 	GList *l;
@@ -1076,13 +1252,19 @@ bookmarks_find_by_nick (const gchar *nick, BookmarkItem *r, gboolean wantsmart)
 	if (!r) r = bookmarks_root;
 
 	if ((r->nick) && !strcmp (r->nick, nick) &&
-			(wantsmart ^ (strstr(r->url,"%s")==NULL)))
+	    (wantsmart ^ (r->url == NULL || strstr(r->url,"%s") == NULL)))
+	{
 		return r;
+	}
+
 	if ((r->type == BM_CATEGORY) && !r->alias_of)
-		for (l = r->list; l != NULL; l = g_list_next (l)) {
+	{
+		for (l = r->list; l != NULL; l = g_list_next (l))
+		{
 			b = bookmarks_find_by_nick (nick, l->data, wantsmart);
 			if (b) return b;
 		}
+	}
 	return NULL;
 }
 
@@ -1096,18 +1278,15 @@ gchar *
 bookmarks_item_to_string (BookmarkItem *bookmark)
 {
 	xmlNodePtr node;
-	xmlDocPtr doc = xmlNewDoc ("1.0");
+	xmlDocPtr doc;
 	xmlChar *mem;
 	int size;
 
+	doc = xmlNewDoc ("1.0");
 	node = xmlNewDocNode (doc, NULL, "single_bookmark", NULL);
-
 	bookmarks_save_recursively (doc, node, bookmark);
-
 	xmlDocSetRootElement (doc, node);
-
 	xmlDocDumpMemory (doc, &mem, &size);
-
 	xmlFreeDoc (doc);
 
 	return mem;
@@ -1127,11 +1306,14 @@ bookmarks_item_from_string (char *string)
 
 	doc = xmlParseMemory (string, strlen (string));	
 	
-	if (doc) {
+	if (doc)
+	{
 		item = doc->root->childs;
 		b = bookmarks_xml_read_item (item);
 		xmlFreeDoc (doc);
-	} else {
+	} 
+	else
+	{
 		g_warning ("Error parsing: %s", string);
 	}
 	return b;
@@ -1204,10 +1386,8 @@ bookmarks_folder_open_all_items (GaleonEmbed *embed, BookmarkItem *bi,
 //	else
 //		embed = embed_create_from_url (embed, child->url, TRUE);
 
-	if (reuse)
-		embed = embed_create_from_url (embed, child->url, new_window);
-	else
-		embed = embed_create_from_url (embed, child->url, TRUE);	
+	embed = embed_create_from_url (embed, child->url, FALSE,
+				       (new_window || !reuse));
 
 	/* process the remaining bookmarks */
 	l = l->next;
@@ -1215,7 +1395,8 @@ bookmarks_folder_open_all_items (GaleonEmbed *embed, BookmarkItem *bi,
 	{
 		BookmarkItem *child = l->data;
 		if (child->url)
-			embed_create_from_url (embed, child->url, new_window);
+			embed_create_from_url (embed, child->url, FALSE,
+					       new_window);
 		l = l->next;
 	}
 }
@@ -1286,10 +1467,10 @@ static BookmarkItem *bookmarks_find_by_id (long id, BookmarkItem *r)
 /**
  * bookmarks_get_siteicon: Parse an http URL ans return the appropriate icon 
  */
-PixmapData *
+const PixmapData *
 bookmarks_get_siteicon (const gchar *url)
 {
-	GnomeVFSURI *vfs_uri, *vfs_parent_uri;
+	GnomeVFSURI *vfs_uri, *vfs_parent_uri, *favicon_uri;
 	gchar *favicon_url, *favicon_path;
 	PixmapData *result = NULL;
 
@@ -1309,7 +1490,7 @@ bookmarks_get_siteicon (const gchar *url)
 		if (result != NULL) return result;
 	}
 	
-	vfs_uri = gnome_vfs_uri_new(url);
+	vfs_uri = gnome_vfs_uri_new (url);
 	if (vfs_uri == NULL)
 	{
 		return site_pixmap_data;
@@ -1318,17 +1499,18 @@ bookmarks_get_siteicon (const gchar *url)
 	if (gnome_vfs_uri_has_parent (vfs_uri))
 	{
 		vfs_parent_uri = gnome_vfs_uri_get_parent (vfs_uri);
+		gnome_vfs_uri_unref (vfs_uri);
 	} 
 	else
 	{
 		vfs_parent_uri = vfs_uri;
 	}
-	vfs_parent_uri = gnome_vfs_uri_append_file_name (vfs_parent_uri, 
-							 "favicon.ico");
+	favicon_uri = gnome_vfs_uri_append_file_name (vfs_parent_uri, 
+						      "favicon.ico");
 	favicon_url = gnome_vfs_uri_to_string (vfs_parent_uri, 0);
 
 	gnome_vfs_uri_unref (vfs_parent_uri);
-	gnome_vfs_uri_unref (vfs_uri);
+	gnome_vfs_uri_unref (favicon_uri);
 	
 	favicon_path = favicon_filename (favicon_url);
 	
@@ -1380,3 +1562,47 @@ bookmarks_get_siteicon (const gchar *url)
 	return result;
 }
 
+GtkCTreeNode *bookmarks_get_tree_item (BookmarkItem *b, BookmarksEditorControls *c)
+{
+	GList *l;
+	for (l = b->tree_items; l; l = l->next) {
+		BookmarkTreeItem *ti = l->data;
+		if (ti->controls == c) return ti->tree_item;
+	}
+	return NULL;
+}
+
+void bookmarks_set_tree_item (BookmarkItem *b, BookmarksEditorControls *c, GtkCTreeNode *n)
+{
+	GList *l;
+	BookmarkTreeItem *ti;		
+	for (l = b->tree_items; l; l = l->next) {
+		ti = l->data;
+		if (ti->controls == c) {
+			if (n)
+				ti->tree_item = n;
+			else  {
+				b->tree_items = g_list_remove_link (b->tree_items, l);
+				g_free (ti);
+			}	
+			return;
+		}
+	}
+	/* not found, append */
+	ti = g_new0 (BookmarkTreeItem, 1);
+	ti->tree_item = n;
+	ti->controls = c;
+	b->tree_items = g_list_prepend (b->tree_items, ti);
+}
+
+/**
+ * Returns TRUE is "a" is an ancestor of "b"
+ */
+gboolean
+bookmarks_is_ancestor (BookmarkItem *a, BookmarkItem *b)
+{
+	for (; b; b = b->parent)
+		if (a == b) return TRUE;
+	return FALSE;
+}
+
