@@ -17,8 +17,23 @@
  */
 
 #include "galeon.h"
+#include "misc.h"
+#include "embed.h"
+#include "window.h"
+#include "bookmarks.h"
 #include "bookmarks_editor_callbacks.h"
 
+#include <string.h>
+#include <time.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-popup-menu.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkselection.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtktext.h>
+#include <gdk/gdkkeysyms.h>
+
 /* internal helper functions */
 static void fetch_icon_recursively (BookmarkItem *b,
 				    BookmarksEditorControls *controls);
@@ -49,6 +64,8 @@ static void bookmarks_editor_set_pixmap (BookmarksEditorControls *controls,
 					 gboolean active);
 static void bookmarks_editor_set_times (BookmarksEditorControls *controls, 
 					 BookmarkItem *bookmark);
+static void bookmarks_selection_prepare (GList *l,
+					BookmarksEditorControls *controls);
 
 
 void
@@ -237,21 +254,46 @@ bookmarks_editor_edited_cb (GtkEditable *editable,
 
 		b_real->time_modified = b_alias->time_modified = time (NULL);
 		bookmarks_editor_set_times (controls, b_alias);
-		bookmarks_editor_update_tree_item (controls, b_alias);
-		bookmarks_update_alias (controls, b_real);
-		controls->dirty = TRUE;
+		bookmarks_editor_update_tree_items (b_real);
+		bookmarks_update_alias (b_real);
+		bookmarks_dirty = TRUE;
 	}
 }
 
 static void
 bookmarks_editor_apply (BookmarksEditorControls *controls)
 {
-	if (controls->dirty && controls->save_function != NULL)
+	if (bookmarks_dirty)
 	{
-		controls->save_function();
+		bookmarks_updated ();
 	}
 }
 
+void
+bookmarks_editor_revert_clicked_cb (GtkWidget *widget,
+				    BookmarksEditorControls *controls)
+{
+	/* delete old bookmarks view */
+	bookmarks_editor_remove_tree_item (controls, controls->root_bookmark);
+
+	/* delete the old bookmarks */
+	/* FIXME: is this enough to delete them all */
+	bookmarks_remove_recursively (controls->root_bookmark);
+
+	/* load bookmarks from disk */
+	bookmarks_load ();
+	
+	/* view new bookmarks */
+	/* FIXME: any implications for temp bookmarks editor? */
+	controls->root_bookmark = bookmarks_root;
+	controls->selection = NULL;
+	controls->last_pressed = NULL;
+	bookmarks_editor_place_tree_item (controls, controls->root_bookmark);
+
+	/* refresh everywhere else */
+	bookmarks_editor_apply (controls);
+}
+
 void 
 bookmarks_editor_apply_clicked_cb (GtkButton *button,
 				   BookmarksEditorControls *controls)
@@ -312,21 +354,24 @@ bookmarks_editor_remove_clicked_cb (GtkButton *button,
 			for (alias = b->alias; alias; alias = alias->alias) {
 				if (alias == next_to_select) 
 					next_to_select = NULL;
-				gtk_ctree_remove_node (ctree, alias->tree_item);
+				bookmarks_editor_remove_tree_items (alias);
 			}
 		}
 
-		gtk_ctree_remove_node (ctree, b->tree_item);
+		bookmarks_editor_remove_tree_items (b);
 		bookmarks_remove_recursively (b);
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
         }
 	gtk_clist_thaw (clist);
 
 	if (autobookmarks_deleted)
+	{
 		autobookmarks_root = NULL;
+		gnome_config_set_bool (CONF_AUTOBOOKMARKS_ENABLED, FALSE);
+	}
 
 	if (next_to_select)
-		gtk_ctree_select (ctree, next_to_select->tree_item);
+		gtk_ctree_select (ctree, bookmarks_get_tree_item (next_to_select, controls));
 
 	g_list_free (selection_list);
 }
@@ -351,10 +396,10 @@ bookmarks_editor_move_up_clicked_cb (GtkButton *button,
 						GTK_CTREE_NODE(selection->data));
 		if (bookmarks_move_bookmark(controls, b, 0))
 		{
-			controls->dirty = TRUE;
+			bookmarks_dirty = TRUE;
 			break;
 		}
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
 	gtk_clist_thaw(clist);
@@ -382,10 +427,10 @@ bookmarks_editor_move_down_clicked_cb (GtkButton *button,
 						GTK_CTREE_NODE(selection->data));
 		if (bookmarks_move_bookmark(controls, b, 1))
 		{
-			controls->dirty = TRUE;
+			bookmarks_dirty = TRUE;
 			break;
 		}
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
 		selection = selection->next;
 	}
 	gtk_clist_thaw(clist);
@@ -436,7 +481,7 @@ void fetch_icon_recursively (BookmarkItem *b,
 		
 	case BM_SITE:
 		get_siteicon (NULL, b->url);
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
 		break;
 
 	case BM_SEPARATOR:
@@ -445,8 +490,8 @@ void fetch_icon_recursively (BookmarkItem *b,
 }
 
 void
-compact_bookmarks_editor_edit_toggled_cb (GtkWidget *toggle, 
-					  BookmarksEditorControls *controls) 
+bookmarks_editor_edit_toggled_cb (GtkWidget *toggle, 
+				  BookmarksEditorControls *controls) 
 {
 	if (controls->edit_frame) {
 		gboolean visible = TRUE;
@@ -473,22 +518,39 @@ bookmarks_editor_add_alias_cb (GtkButton *button,
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 	if (new->type != BM_SITE)
 		insert_pos = GTK_CLIST_DRAG_BEFORE;
+
+	if (near)
+	{
+		new->time_added = time (NULL);
+		bookmarks_insert_bookmark (new, near, insert_pos);
+		bookmarks_editor_place_tree_items (new);
+		bookmarks_editor_select_bookmark (controls, new);
+	}
+}
+
+void
+bookmarks_editor_add_alias_root_cb (GtkButton *button,
+			       BookmarksEditorControls *controls) 
+{
+	BookmarkItem *orig = controls->root_bookmark;
+	BookmarkItem *new = bookmarks_new_alias (orig);
+	BookmarkItem *near = controls->selection;
+	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 	if (!near) near = controls->root_bookmark;
 	new->time_added = time (NULL);
 	bookmarks_insert_bookmark (new, near, insert_pos);
-	bookmarks_editor_place_tree_item (controls, new);
-	controls->dirty = TRUE;
-	gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
+	bookmarks_editor_place_tree_items (new);
+	bookmarks_editor_select_bookmark (controls, new);
 }
 
+
 void 
 bookmarks_editor_new_item_clicked_cb (GtkButton *button,
 				      BookmarksEditorControls *controls) 
 {
 	BookmarkItem *new = bookmarks_new_bookmark 
-		(BM_SITE, TRUE, _("New bookmark item"), 
-		 _("where://do.you.want.to.go"), NULL, NULL, NULL);
+		(BM_SITE, TRUE, _("New bookmark"),  "http://",
+		 NULL, NULL, NULL);
 	BookmarkItem *near = controls->selection;
 	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
 
@@ -496,10 +558,8 @@ bookmarks_editor_new_item_clicked_cb (GtkButton *button,
 
 	if (!near) near = controls->root_bookmark;
 	bookmarks_insert_bookmark (new, near, insert_pos);
-	bookmarks_editor_place_tree_item (controls, new);
-	controls->dirty = TRUE;
-	gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
+	bookmarks_editor_place_tree_items (new);
+	bookmarks_editor_select_bookmark (controls, new);
 
 	/* Don't grab the focus if we're in the temp bookmarks window and the
 	   edit frame isn't visible */
@@ -524,10 +584,8 @@ bookmarks_editor_new_category_clicked_cb (GtkButton *button,
 
 	if (!near) near = controls->root_bookmark;
 	bookmarks_insert_bookmark (new, near, insert_pos);
-	bookmarks_editor_place_tree_item (controls, new);
-	controls->dirty = TRUE;
-	gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
+	bookmarks_editor_place_tree_items (new);
+	bookmarks_editor_select_bookmark (controls, new);
 
 	/* Don't grab the focus if we're in the temp bookmarks window and the
 	   edit frame isn't visible */
@@ -552,17 +610,13 @@ bookmarks_editor_new_separator_clicked_cb (GtkButton *button,
 
 	if (!near) near = controls->root_bookmark;
 	bookmarks_insert_bookmark (new, near, insert_pos);
-	bookmarks_editor_place_tree_item (controls, new);
-	controls->dirty = TRUE;
-	gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
+	bookmarks_editor_place_tree_items (new);
+	bookmarks_editor_select_bookmark (controls, new);
 }
 
-/* TODO: create bookmarks_editor_new_alias_clicked_cb (after adding the button) */
-
 void
-compact_bookmarks_editor_go_clicked_cb (GtkButton *button, 
-					BookmarksEditorControls *controls)
+bookmarks_editor_go_clicked_cb (GtkButton *button, 
+				BookmarksEditorControls *controls)
 {
 	GList *pos;
 	gchar *url;
@@ -588,7 +642,7 @@ compact_bookmarks_editor_go_clicked_cb (GtkButton *button,
 		if (!controls->window)
 		{
 			controls->window = 
-				embed_create_from_url (NULL, url, TRUE)->parent_window;
+				embed_create_from_url (NULL, url, FALSE, TRUE)->parent_window;
 		}
 		else
 		{
@@ -597,48 +651,6 @@ compact_bookmarks_editor_go_clicked_cb (GtkButton *button,
 	}
 }
 
-void
-compact_bookmarks_editor_go_forward_clicked_cb (GtkButton *button,
-						BookmarksEditorControls *controls)
-{
-	if (controls->selection && controls->selection->parent) {
-		GList *dest_pos = g_list_next (
-			g_list_find (controls->selection->parent->list, 
-				     controls->selection));
-		if (dest_pos) {
-			BookmarkItem *dest = dest_pos->data;
-			if (dest) {
-				gtk_ctree_unselect (GTK_CTREE (controls->ctree),
-						controls->selection->tree_item);
-				gtk_ctree_select (GTK_CTREE (controls->ctree),
-						  dest->tree_item);
-				compact_bookmarks_editor_go_clicked_cb (button, controls);
-			}
-		}
-	}
-}
-
-void
-compact_bookmarks_editor_go_back_clicked_cb (GtkButton *button,
-					     BookmarksEditorControls *controls)
-{
-	if (controls->selection && controls->selection->parent) {
-		GList *dest_pos = g_list_previous (
-			g_list_find (controls->selection->parent->list, 
-				     controls->selection));
-		if (dest_pos) {
-			BookmarkItem *dest = dest_pos->data;
-			if (dest) {
-				gtk_ctree_unselect (GTK_CTREE (controls->ctree),
-						controls->selection->tree_item);
-				gtk_ctree_select (GTK_CTREE (controls->ctree),
-						  dest->tree_item);
-				compact_bookmarks_editor_go_clicked_cb (button, controls);
-			}
-		}
-	}
-}
-
 gboolean
 bookmarks_editor_copy_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 				       BookmarksEditorControls *controls)
@@ -655,18 +667,14 @@ bookmarks_editor_copy_clicked_cb (GtkButton *button,
 {
 	if (controls->selection) {
 		GtkMenu *destination_menu;
-		if (!bookmarks_editor)
-			bookmarks_editor_show_dialog (
-						NULL,
-						STANDARD_BOOKMARKS_EDITOR);
+
 		destination_menu = 
-			bookmarks_create_copy_menu (bookmarks_editor, 
-						    controls->selection);
+			bookmarks_create_copy_menu (controls->selection);
 		gnome_popup_menu_do_popup_modal
 			(GTK_WIDGET (destination_menu), menu_position_under_widget, 
 			 GTK_WIDGET (button), NULL, NULL);
 		gtk_widget_destroy (GTK_WIDGET (destination_menu));
-		bookmarks_save ();
+		bookmarks_dirty = TRUE;
 	}
 }
 
@@ -688,7 +696,7 @@ bookmarks_editor_ctree_tree_move_cb (GtkCTree *ctree, GtkCTreeNode *node,
 		new_parent = controls->root_bookmark;
 		
 	source->parent->list = g_list_remove (source->parent->list, source);
-	bookmarks_update_alias (controls, source->parent);
+	bookmarks_update_alias (source->parent);
 	source->parent = new_parent;
 	
 	new_sib_list_node = g_list_find (new_parent->list, new_sib);
@@ -698,7 +706,9 @@ bookmarks_editor_ctree_tree_move_cb (GtkCTree *ctree, GtkCTreeNode *node,
 	} else
 		new_parent->list = g_list_append (new_parent->list, source);
 
-	bookmarks_update_alias (controls, source->parent);
+	bookmarks_update_alias (source->parent);
+	bookmarks_editor_place_tree_items (source);
+
 	/* DEBUG only */
 #if 0
 	g_assert (g_list_find (source->parent->list, source));
@@ -706,7 +716,6 @@ bookmarks_editor_ctree_tree_move_cb (GtkCTree *ctree, GtkCTreeNode *node,
 		  (source->parent->list, g_list_find (source->parent->list, source)) 
 		  == new_pos);
 #endif
-	controls->dirty = TRUE;
 }
 
 void
@@ -719,8 +728,7 @@ bookmarks_editor_bookmark_copy_cb (GtkMenuItem *menuitem, BookmarkItem *dest)
 	new = bookmarks_copy_bookmark (item);
 	new->parent = dest;
 	dest->list = g_list_append (dest->list, new);
-	bookmarks_editor->dirty = TRUE;
-	bookmarks_editor_place_tree_item (bookmarks_editor, new);
+	bookmarks_editor_place_tree_items (new);
 }
 
 gboolean 
@@ -734,14 +742,15 @@ bookmarks_editor_ctree_button_press_event_cb (GtkWidget *widget,
 	controls->last_pressed = gtk_ctree_node_get_row_data 
 		(GTK_CTREE (widget), gtk_ctree_node_nth (GTK_CTREE (widget), row));
 
-	/* Double click goes to the link */
+	/* Double click goes to the link, as well as ignore the scrollbutton */
 	if (event->type == GDK_2BUTTON_PRESS && 
+	    !(event->button == 4 || event->button == 5) &&
 	    controls->last_pressed != NULL)
 	{
 		/* make sure that this entry is selected */
 		gtk_ctree_select (GTK_CTREE (controls->ctree),
-				  controls->last_pressed->tree_item);
-		compact_bookmarks_editor_go_clicked_cb(NULL, controls);
+				  bookmarks_get_tree_item (controls->last_pressed, controls));
+		bookmarks_editor_go_clicked_cb(NULL, controls);
 	}
 
 	return FALSE;
@@ -779,7 +788,7 @@ bookmarks_editor_tree_expand_cb (GtkCTree *ctree, GtkCTreeNode *node,
 	BookmarkItem *b = gtk_ctree_node_get_row_data (ctree, node);
 	if (b) {
 		b->expanded = TRUE;
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
 	}
 }
 
@@ -790,7 +799,7 @@ bookmarks_editor_tree_collapse_cb (GtkCTree *ctree, GtkCTreeNode *node,
 	BookmarkItem *b = gtk_ctree_node_get_row_data (ctree, node);
 	if (b) {
 		b->expanded = FALSE;
-		controls->dirty = TRUE;
+		bookmarks_dirty = TRUE;
 	}
 }
 
@@ -803,7 +812,7 @@ bookmarks_editor_expand_all_cb (GtkButton *button,
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
 		gtk_ctree_expand_recursive (GTK_CTREE (controls->ctree), 
-					    b->tree_item);
+					    bookmarks_get_tree_item (b, controls));
 	}
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
@@ -817,7 +826,7 @@ bookmarks_editor_collapse_all_clicked_cb (GtkButton *button,
 	for (l = controls->root_bookmark->list; l != NULL; l = l->next) {
 		BookmarkItem *b = l->data;
 		gtk_ctree_collapse_recursive (GTK_CTREE (controls->ctree), 
-					      b->tree_item);
+					      bookmarks_get_tree_item (b, controls));
 	}
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
@@ -980,19 +989,184 @@ bookmarks_editor_set_times (BookmarksEditorControls *controls, BookmarkItem *boo
 	}
 }
 
-void
-bookmarks_editor_add_alias_root_cb (GtkButton *button,
-			       BookmarksEditorControls *controls) 
+void 
+bookmarks_editor_show_only_this_category_cb (GtkButton *button,
+				             BookmarksEditorControls *controls) 
 {
-	BookmarkItem *orig = controls->root_bookmark;
-	BookmarkItem *new = bookmarks_new_alias (orig);
-	BookmarkItem *near = controls->selection;
-	GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
-	if (!near) near = controls->root_bookmark;
-	new->time_added = time (NULL);
-	bookmarks_insert_bookmark (new, near, insert_pos);
-	bookmarks_editor_place_tree_item (controls, new);
-	controls->dirty = TRUE;
-	gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), new->tree_item);
+	if (controls->selection) {
+		BookmarkItem *new_root = 
+			((controls->selection->type == BM_CATEGORY)
+			 || (controls->selection->type == BM_AUTOBOOKMARKS)) ?
+			controls->selection :
+			controls->selection->parent;
+		if (new_root) bookmarks_editor_set_root (controls, new_root);
+			
+	}
+}
+
+void 
+bookmarks_editor_show_all_bookmarks_cb (GtkButton *button,
+				        BookmarksEditorControls *controls) 
+{
+	bookmarks_editor_set_root (controls, bookmarks_root);
+}
+
+void
+bookmarks_editor_cut_cb (GtkButton *button,
+			 BookmarksEditorControls *controls) 
+{
+	bookmarks_editor_copy_cb (button, controls);
+	bookmarks_editor_remove_clicked_cb (button, controls);
+}
+
+void
+bookmarks_editor_copy_cb (GtkButton *button,
+			  BookmarksEditorControls *controls) 
+{
+	if (controls->selection) {
+		BookmarkItem *dummy = bookmarks_new_bookmark 
+			(BM_CATEGORY, TRUE, "Dummy category", 
+			 NULL, NULL, NULL, NULL);
+		GtkCList *clist = GTK_CLIST(controls->ctree);
+		GList *selection_list = 
+			bookmarks_get_selections_sorted (clist);
+		bookmarks_selection_prepare (selection_list, controls);
+		dummy->list = selection_list;
+		if (controls->clipboard) g_free (controls->clipboard);
+		controls->clipboard = bookmarks_item_to_string (dummy);
+		gtk_selection_owner_set (GTK_WIDGET (clist),
+					 GDK_SELECTION_PRIMARY,
+					 GDK_CURRENT_TIME);
+		dummy->list = NULL;
+		bookmarks_free_bookmark (dummy);
+		g_list_free (selection_list);
+	}
+}
+
+void
+bookmarks_editor_paste_cb (GtkButton *button,
+			   BookmarksEditorControls *controls) 
+{
+	gtk_selection_convert (GTK_WIDGET (controls->ctree), 
+			       GDK_SELECTION_PRIMARY,
+			       GDK_SELECTION_TYPE_STRING,
+			       GDK_CURRENT_TIME);
+}
+
+void
+bookmarks_editor_selection_get_cb (GtkWidget *widget, 
+					GtkSelectionData *selection_data,
+					guint info, guint time_stamp,
+					BookmarksEditorControls *controls)
+{
+	if (controls->clipboard)
+		gtk_selection_data_set (selection_data, 
+					GDK_SELECTION_TYPE_STRING, // FIXME
+					8, controls->clipboard, 
+					strlen (controls->clipboard));
+}
+
+void 
+bookmarks_selection_prepare (GList *l, BookmarksEditorControls *controls) 
+{
+	GtkCTree *ctree = GTK_CTREE (controls->ctree);
+	GList *li;
+	for (li = l; li; li = li->next) {
+		BookmarkItem *b =  gtk_ctree_node_get_row_data 
+			(ctree, li->data);
+		li->data = b;
+	}
+}
+
+
+void 
+bookmarks_editor_selection_received_cb (GtkWidget *widget, 
+					GtkSelectionData *selection_data,
+					guint xtime)
+{
+	BookmarkItem *b;
+	BookmarksEditorControls *controls = gtk_object_get_data 
+		(GTK_OBJECT (widget), "controls");
+
+	if (selection_data->length < 0)	{
+		g_warning ("Selection retrieval failed\n");
+		return;
+	}
+
+	b = bookmarks_item_from_string (selection_data->data);
+
+	if (b) {
+		BookmarkItem *near = controls->selection;
+		GtkCListDragPos insert_pos = GTK_CLIST_DRAG_INTO;
+		GList *li;
+		if (!near) near = controls->root_bookmark;
+		
+		gtk_clist_freeze (GTK_CLIST(controls->ctree));
+		gtk_clist_unselect_all (GTK_CLIST(controls->ctree));
+		
+		/* b is the dummy bookmark, we have to add all it's children */
+		for (li = b->list; li; li = li->next)
+		{
+			BookmarkItem *new = li->data;
+			new->time_modified = time (NULL);
+			bookmarks_insert_bookmark (new, near, insert_pos);
+			bookmarks_editor_place_tree_items (new);
+			bookmarks_editor_select_bookmark (controls, new);
+			near = new;
+			insert_pos = GTK_CLIST_DRAG_AFTER;
+		}
+		bookmarks_free_bookmark (b);
+		gtk_clist_thaw (GTK_CLIST(controls->ctree));
+	}
+}
+
+void
+bookmarks_dock_close_clicked_cb (GtkWidget *button, 
+				 BookmarksEditorControls *controls)
+{
+	bookmarks_editor_apply (controls);
+	bookmarks_editor_remove_tree_item (controls, 
+			                   controls->root_bookmark);
+	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
+	window_undock (controls->window);
+}
+
+void
+bookmarks_dock_destroy_cb (GtkWidget *dialog, 
+			   BookmarksEditorControls *controls)
+{
+	GaleonWindow *window;
+
+	window = controls->window;
+	return_if_not_window (window);
+	(GTK_CHECK_MENU_ITEM (window->view_bookmarks_dock))->active = FALSE;
+}
+
+void 
+bookmarks_dock_detach_button_clicked_cb (GtkButton *button,
+					 BookmarksEditorControls *controls)
+{
+	GaleonWindow *window;
+
+	window = controls->window;
+	return_if_not_window (window);
+	bookmarks_editor_apply (controls);
+	bookmarks_editor_remove_tree_item (controls, 
+			                   controls->root_bookmark);
+	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
+	window_undock (window);
+	bookmarks_editor_show_dialog (window, STANDARD_BOOKMARKS_EDITOR);
+}
+
+void
+bookmarks_editor_show_attach_cb (GtkWidget *toggle, 
+				 BookmarksEditorControls *controls) 
+{
+	GaleonWindow *window;
+
+	window = controls->window;
+	return_if_not_window (window);
+
+	bookmarks_editor_hide_dialog (controls);
+	bookmarks_editor_show_dock (window);
 }
