@@ -17,6 +17,11 @@
  */
 
 #include "galeon.h"
+#include "bookmarks.h"
+
+#include <string.h>
+#include <gtk/gtkselection.h>
+#include <gtk/gtkdnd.h>
 
 /* Internal function used for multiple selection DnD */
 static gint bookmarks_row_compare (GList *a, GList *b);
@@ -48,7 +53,9 @@ void bookmarks_editor_ctree_drag_leave (GtkWidget *widget,
 					GdkDragContext *context,
 					guint time,
 					BookmarksEditorControls *controls);
-gboolean vscroll_timeout (gpointer data);
+static gboolean vscroll_timeout (gint direction, BookmarksEditorControls *controls);
+static gboolean vscroll_timeout_up (BookmarksEditorControls *controls);
+static gboolean vscroll_timeout_down (BookmarksEditorControls *controls);
 
 /* Internal helper functions from gtkctree.c */
 #define GTK_CLIST_CLASS_FW(_widget_) \
@@ -195,8 +202,9 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 		if (!clist->vtimer)
 		{
 			DRAW_DRAG_HIGHLIGHT (clist, dest_info);
-			clist->vtimer = g_timeout_add (50, vscroll_timeout,
-						       GINT_TO_POINTER (1));
+			clist->vtimer = g_timeout_add (50, (GSourceFunc)
+						       vscroll_timeout_down,
+						       controls);
 		}
 	}
 	else if (y < (offset + clist->row_height + 1))
@@ -204,8 +212,9 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 		if (!clist->vtimer)
 		{
 			DRAW_DRAG_HIGHLIGHT (clist, dest_info);
-			clist->vtimer = g_timeout_add (50, vscroll_timeout,
-						       GINT_TO_POINTER (-1));
+			clist->vtimer = g_timeout_add (50, (GSourceFunc)
+						       vscroll_timeout_up,
+						       controls);
 		}
 	}
 	else
@@ -220,12 +229,23 @@ bookmarks_editor_ctree_drag_motion (GtkWidget *widget,
 	return TRUE;
 }
 
-gboolean
-vscroll_timeout (gpointer data)
+static gboolean 
+vscroll_timeout_up (BookmarksEditorControls *controls)
 {
-	GtkCList *clist = GTK_CLIST (bookmarks_editor->ctree);
+	return vscroll_timeout (-1, controls);
+}
+
+static gboolean
+vscroll_timeout_down (BookmarksEditorControls *controls)
+{
+	return vscroll_timeout (1, controls);
+}
+
+static gboolean
+vscroll_timeout (gint direction, BookmarksEditorControls *controls)
+{
+	GtkCList *clist = GTK_CLIST (controls->ctree);
 	gfloat scroll_step, newval;
-	int direction = GPOINTER_TO_INT (data);
 
 	g_return_val_if_fail (clist != NULL, FALSE);
 
@@ -425,7 +445,6 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 					      guint info, guint time, 
 					      BookmarksEditorControls *controls)
 {
-	/* TODO: check for alias loops */
 	GtkCTree *ctree;
 	GtkCList *clist;
 	BookmarkItem *b = NULL;
@@ -555,27 +574,27 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 			dest_info.insert_pos = GTK_CLIST_DRAG_INTO;
 
 		bookmarks_insert_bookmark (b, near, dest_info.insert_pos);
-		bookmarks_update_alias (controls, b->parent);
-		bookmarks_editor_place_tree_item (controls, b);
-		controls->dirty = TRUE;
+		bookmarks_update_alias (b->parent);
+		bookmarks_editor_place_tree_items (b);
 
 		/* set the focus so it matches the selection */
 		if (gtk_ctree_node_nth (GTK_CTREE (controls->ctree),
 				        dest_info.cell.row)
-		    != b->tree_item)
+		    != bookmarks_get_tree_item (b, controls))
 		{
-			if (GTK_CTREE_NODE_NEXT(b->tree_item) == NULL)
+			if (GTK_CTREE_NODE_NEXT (bookmarks_get_tree_item (b, controls)) == NULL)
 				row = g_list_length(g_list_first(
-				                    (GList *)b->tree_item)) - 1;
+				                    (GList *) bookmarks_get_tree_item (b, controls))) - 1;
 			else
 				row = g_list_position(g_list_first(
-						      (GList *)b->tree_item),
-						      (GList *)b->tree_item);
+						      (GList *)bookmarks_get_tree_item (b, controls)),
+						      (GList *)bookmarks_get_tree_item (b, controls));
 		}
 		if (row >= -1)
 			GTK_CLIST(controls->ctree)->focus_row = row;
-		gtk_ctree_select (GTK_CTREE (controls->ctree), b->tree_item);
+		gtk_ctree_select (GTK_CTREE (controls->ctree), bookmarks_get_tree_item (b, controls));
 	}
+	bookmarks_dirty = TRUE;
 }
 
 /* CTree reordering does not work if DnD is enabled, since the reordering
