@@ -17,8 +17,20 @@
  */
 
 #include "galeon.h"
+#include "misc.h"
+#include "embed.h"
+#include "bookmarks.h"
 
-static GtkWidget *bookmarks_create_menu_item (BookmarkItem *bm, GtkMenu *menu);
+#include <string.h>
+#include <libgnomeui/gnome-preferences.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gtkpixmapmenuitem.h>
+
+
+static void bookmarks_menu_destroy (GaleonWindow *window);
+static GtkWidget *bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu);
 static void bookmarks_create_file_menu (GtkMenuItem *file_menuitem,
 					BookmarkItem *bookmark);
 static void bookmarks_create_file_submenu (GtkMenuItem *file_menuitem, 
@@ -26,89 +38,111 @@ static void bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 static void bookmarks_menu_create_extra_items (GtkMenu *menu,
 					       BookmarkItem *bookmark,
 					       gint opts_location);
-
-static void bookmarks_create_menu_recursively_cb (GtkMenuItem *menuitem, gpointer data);
+static void bookmarks_menu_create_recursively_cb (GtkMenuItem *menuitem,
+						  gpointer data);
 static void bookmarks_create_file_menu_cb (GtkMenuItem *mi, BookmarkItem *b);
-
-GtkMenu *bookmarks_create_copy_menu_recursively (BookmarksEditorControls *destination_window,
-						 BookmarkItem *source, BookmarkItem *b);
-static gint
-bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window, 
-				  BookmarkItem *bi, gboolean new_window);
-static gint
-bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonWindow *window, 
-				 BookmarkItem *bi, gboolean new_window);
+GtkMenu *bookmarks_create_copy_menu_recursively (BookmarkItem *source,
+						 BookmarkItem *b);
+static gint bookmarks_smart_bm_dialog_single (GtkWidget *dialog, 
+					      GaleonWindow *window, 
+					      BookmarkItem *bi,
+					      gboolean new_window);
+static gint bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, 
+					     GaleonWindow *window, 
+					     BookmarkItem *bi,
+					     gboolean new_window);
 static gboolean bookmark_has_subcategories (BookmarkItem *b);
 
-void
-bookmarks_create_menu_global (void)
-{
-	GList *sl;
-
-	for (sl = all_windows; sl != NULL; sl = sl->next) 
-	{
-		/* Create the menus and the toolbars for this window */
-		bookmarks_create_menu (sl->data);
-	}
-}
-
-void
-bookmarks_remove_menu_global (void)
-{
-	GList *sl;
-
-	/* Destroys the menus */
-	for (sl = all_windows; sl != NULL; sl = sl->next) {
-		GtkWidget *WMain = ( (GaleonWindow*) sl->data)->WMain;
-		GtkWidget *separator = gtk_object_get_data (GTK_OBJECT(WMain),
-							"bookmarks_separator");
-		GList *l = gtk_container_children (GTK_CONTAINER (
-				gtk_object_get_data (GTK_OBJECT(WMain),
-						     "bookmarks_menu")));
-		GList *li;
-		/* first look for the separator */
-		for (li = l; li != NULL; li = li->next) {
-			if (li->data == separator) break;
-		}
-		/* then, destroy every widget after it */
-		for (li = li->next; li != NULL; li = li->next) {
-			if (GTK_IS_WIDGET (li->data))
-			    gtk_widget_destroy (GTK_WIDGET (li->data));
-		}
-		g_list_free(l);
-	}
-}
-
 /**
- * bookmarks_update_menu: destroy and recreate the bookmarks menu
+ * bookmarks_menu_create: create the bookmarks menu for a single window
  */
-void 
-bookmarks_update_menu (void) 
-{
-	bookmarks_remove_menu_global ();
-	bookmarks_create_menu_global ();
-}
-
 void
-bookmarks_create_menu (GaleonWindow *window) 
+bookmarks_menu_create (GaleonWindow *window) 
 {
-	GtkWidget *app = window->WMain;
-	GtkMenu *menu = GTK_MENU
-		(gtk_object_get_data (GTK_OBJECT(app), "bookmarks_menu"));
-	GtkMenuItem *file_menuitem = GTK_MENU_ITEM 
-		(gtk_object_get_data (GTK_OBJECT(app), "file_bookmark"));
-	if (window->bookmarks_tooltips) 
-		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
+	GtkMenuItem *file_menuitem;
+	GtkWidget *app;
+	GtkMenu *menu;
+	
+	return_if_not_window (window);
 
+	/* skip if no menu on this window */
+	if (!(window->menuBarOn))
+	{
+		return;
+	}
+
+	app = window->WMain;
+	menu = GTK_MENU (gtk_object_get_data (GTK_OBJECT(app), 
+					      "bookmarks_menu"));
+	file_menuitem = GTK_MENU_ITEM (gtk_object_get_data (GTK_OBJECT(app), 
+							    "file_bookmark"));
+
+	if (window->bookmarks_tooltips != NULL)
+	{
+		gtk_object_destroy (GTK_OBJECT (window->bookmarks_tooltips));
+	}
+	
 	window->bookmarks_tooltips = gtk_tooltips_new ();
 	bookmarks_create_file_submenu (file_menuitem, bookmarks_root);
-	bookmarks_create_menu_recursively (bookmarks_root, menu, file_menuitem,
-					   window->bookmarks_tooltips, TRUE, 
+	bookmarks_menu_create_recursively (bookmarks_root, menu, file_menuitem,
+					   GTK_TOOLTIPS(window->bookmarks_tooltips), 
+					   TRUE, 
 					   FALSE);
 }
 
 /**
- * bookmarks_create_menu_recursively: Create menus for bookmarks
+ * bookmarks_menu_recreate: recreate the bookmarks menu for a single window
+ */
+void
+bookmarks_menu_recreate (GaleonWindow *window)
+{
+	bookmarks_menu_destroy (window);
+	bookmarks_menu_create (window);
+}
+
+static void
+bookmarks_menu_destroy (GaleonWindow *window)
+{
+	GtkWidget *WMain = GTK_WIDGET (window->WMain);
+	GtkWidget *bookmarks_menu;
+	GtkWidget *separator;
+	GList *li, *l;
+	
+	/* skip if no menu on this window */
+	if (!(window->menuBarOn))
+	{
+		return;
+	}
+	
+	separator = gtk_object_get_data (GTK_OBJECT (WMain),
+					 "bookmarks_separator");
+	bookmarks_menu = gtk_object_get_data (GTK_OBJECT (WMain),
+					      "bookmarks_menu");
+	l = gtk_container_children (GTK_CONTAINER (bookmarks_menu));
+	
+	/* first look for the separator */
+	for (li = l; li != NULL; li = li->next)
+	{
+		if (li->data == separator)
+		{
+			break;
+		}
+	}
+	
+	/* then, destroy every widget after it */
+	for (li = li->next; li != NULL; li = li->next)
+	{
+		if (GTK_IS_WIDGET (li->data))
+		{
+			gtk_widget_destroy (GTK_WIDGET (li->data));
+		}
+	}
+	//FIXME: right???
+	g_list_free (l);
+}
+
+/**
+ * bookmarks_menu_create_recursively: Create menus for bookmarks
  * @bookmark: the root bookmark of the menu
  * @menu: the menu to fill with @bookmark children
  * @file_menuitem: the menu to fill with categories (to add bookmarks)
@@ -120,7 +154,7 @@ bookmarks_create_menu (GaleonWindow *window)
  * (usefull for the toolbar menus).
  **/
 void
-bookmarks_create_menu_recursively (BookmarkItem *bookmark, GtkMenu *menu, 
+bookmarks_menu_create_recursively (BookmarkItem *bookmark, GtkMenu *menu, 
 				   GtkMenuItem *file_menuitem,
 				   GtkTooltips *tooltips, gboolean options, 
 				   gboolean disable_tearoffs)
@@ -144,7 +178,7 @@ bookmarks_create_menu_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 		}
 		else if (opts_location == 2)
 		{
-			mi = bookmarks_create_menu_item (bookmark, menu);
+			mi = bookmarks_menu_create_item (bookmark, menu);
 			gtk_signal_connect (GTK_OBJECT(mi),
 					    "button-release-event",
 					    GTK_SIGNAL_FUNC (
@@ -169,7 +203,7 @@ bookmarks_create_menu_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 		case BM_AUTOBOOKMARKS:
 			submenu = gtk_menu_new ();
 			gtk_menu_set_title (GTK_MENU (submenu), b->name);
-			mi = bookmarks_create_menu_item (b, menu);
+			mi = bookmarks_menu_create_item (b, menu);
 			gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), submenu);
 			gtk_signal_connect 
 				(GTK_OBJECT (submenu), "button-release-event", 
@@ -185,11 +219,11 @@ bookmarks_create_menu_recursively (BookmarkItem *bookmark, GtkMenu *menu,
 					     GINT_TO_POINTER (disable_tearoffs));
 			
 			gtk_signal_connect (GTK_OBJECT (mi), "map", 
-					    bookmarks_create_menu_recursively_cb,
+					    bookmarks_menu_create_recursively_cb,
 					    NULL);
 			break;
 		case BM_SITE:
-			mi = bookmarks_create_menu_item (b, menu);
+			mi = bookmarks_menu_create_item (b, menu);
 			gtk_signal_connect_after
 				(GTK_OBJECT (mi), "button_release_event", 
 				 GTK_SIGNAL_FUNC (
@@ -250,7 +284,7 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 	for (l = bookmark->list; l ; l = l->next) {
 		BookmarkItem *b = l->data;
 		if ((b->type == BM_CATEGORY) || (b->type == BM_AUTOBOOKMARKS)) {
-			GtkWidget *file = bookmarks_create_menu_item (b,
+			GtkWidget *file = bookmarks_menu_create_item (b,
 							GTK_MENU (file_menu));
 			gtk_widget_show (file);
 			gtk_menu_append (GTK_MENU (file_menu), file);
@@ -276,7 +310,7 @@ bookmarks_create_file_menu (GtkMenuItem *file_menuitem, BookmarkItem *bookmark)
 }
 
 static void
-bookmarks_create_menu_recursively_cb (GtkMenuItem *mi, gpointer data)
+bookmarks_menu_create_recursively_cb (GtkMenuItem *mi, gpointer data)
 {
 	BookmarkItem *b = gtk_object_get_data (GTK_OBJECT (mi), "bookmark");
 	GtkMenu *submenu = gtk_object_get_data (GTK_OBJECT (mi), "submenu");
@@ -288,7 +322,7 @@ bookmarks_create_menu_recursively_cb (GtkMenuItem *mi, gpointer data)
 						      "disable_tearoffs"));
 
 	gtk_signal_disconnect_by_func (GTK_OBJECT (mi),  
-			    bookmarks_create_menu_recursively_cb,
+			    bookmarks_menu_create_recursively_cb,
 			    NULL);
 
 	gtk_object_remove_data (GTK_OBJECT (mi), "bookmark");
@@ -297,7 +331,7 @@ bookmarks_create_menu_recursively_cb (GtkMenuItem *mi, gpointer data)
 	gtk_object_remove_data (GTK_OBJECT (mi), "options");
 	gtk_object_remove_data (GTK_OBJECT (mi), "disable_tearoffs");
 
-	bookmarks_create_menu_recursively (b, GTK_MENU (submenu), NULL, 
+	bookmarks_menu_create_recursively (b, GTK_MENU (submenu), NULL, 
 					   tooltips, options, disable_tearoffs);
 
 }
@@ -311,7 +345,9 @@ bookmarks_create_file_menu_cb (GtkMenuItem *mi, BookmarkItem *b)
 	bookmarks_create_file_menu (mi, b);
 }
 
-static gboolean bookmark_has_subcategories (BookmarkItem *b) {
+static gboolean 
+bookmark_has_subcategories (BookmarkItem *b) 
+{
 	GList *l;
 	for (l = b->list; l; l = l->next) {
 		BookmarkItem *bi = l->data;
@@ -337,8 +373,10 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	if (opts_location == 2)
 		return;
 
-	add_item = gtk_menu_item_new_with_label
-				(_("Add bookmark to this folder"));
+	add_item = gtk_menu_item_new_with_label ("");
+	label_set_accel_text (_("Add _bookmark to this folder"), 
+			GTK_BIN (add_item)->child, GTK_WIDGET (menu),
+			add_item);
 	gtk_signal_connect (GTK_OBJECT (add_item), "button-release-event",
 			    GTK_SIGNAL_FUNC(bookmarks_button_release_event_cb),
 			    bookmark);
@@ -347,8 +385,10 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	gtk_object_set_data (GTK_OBJECT (add_item), "add_item",
 			     GINT_TO_POINTER (1));
 
-	open_all_item = gtk_menu_item_new_with_label
-				(_("Open all bookmarks in this folder"));
+	open_all_item = gtk_menu_item_new_with_label ("");
+	label_set_accel_text (_("_Open all bookmarks in this folder"),
+			GTK_BIN (open_all_item)->child, GTK_WIDGET (menu),
+			open_all_item);
 	gtk_signal_connect (GTK_OBJECT(open_all_item), "button-release-event",
 			    GTK_SIGNAL_FUNC(bookmarks_button_release_event_cb),
 			    bookmark);
@@ -357,8 +397,10 @@ bookmarks_menu_create_extra_items (GtkMenu *menu, BookmarkItem *bookmark,
 	gtk_object_set_data (GTK_OBJECT (open_all_item), "open_all_item",
 			     GINT_TO_POINTER (1));
 
-	create_toolbar_item = gtk_check_menu_item_new_with_label
-				(_("Create toolbar from this folder"));
+	create_toolbar_item = gtk_check_menu_item_new_with_label ("");
+	label_set_accel_text (_("_Create toolbar from this folder"),
+			GTK_BIN (create_toolbar_item)->child,
+			GTK_WIDGET (menu), create_toolbar_item);
 	gtk_check_menu_item_set_active 
 				(GTK_CHECK_MENU_ITEM(create_toolbar_item), 
 				 bookmark->create_toolbar);
@@ -424,7 +466,7 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 		(GTK_MENU_ITEM (file_menuitem),
 		 file_menu);
 
-	w = bookmarks_create_menu_item (bookmark, GTK_MENU (file_menu));
+	w = bookmarks_menu_create_item (bookmark, GTK_MENU (file_menu));
 	gtk_signal_connect (GTK_OBJECT (w), "activate", 
 			    bookmarks_file_bookmark_cb, bookmark);
 	gtk_widget_show (w);
@@ -441,17 +483,19 @@ bookmarks_create_file_submenu (GtkMenuItem *file_menuitem,
 	}
 }
 
+/* FIXME: obsoleted
+ * These two functions need rewriting or just dumping them...
+ * They are useful now just for dropping utls on the bookmark menu
+ */
 GtkMenu*
-bookmarks_create_copy_menu (BookmarksEditorControls *destination_window, 
-			    BookmarkItem *source)
+bookmarks_create_copy_menu (BookmarkItem *source)
 {
 	GtkMenu *res;
-	g_assert (destination_window->root_bookmark->type == BM_CATEGORY);
-	res = bookmarks_create_copy_menu_recursively (destination_window, source,
-						      destination_window->root_bookmark);
+	res = bookmarks_create_copy_menu_recursively (source,
+						      bookmarks_root);
 	if (! res) {
-		BookmarkItem *item = destination_window->root_bookmark;
-		GtkWidget *menu_item = bookmarks_create_menu_item (item, res);
+		BookmarkItem *item = bookmarks_root;
+		GtkWidget *menu_item = bookmarks_menu_create_item (item, res);
 		res = GTK_MENU (gtk_menu_new ());
 		gtk_widget_show (menu_item);
 		gtk_menu_prepend (res, menu_item);
@@ -460,8 +504,8 @@ bookmarks_create_copy_menu (BookmarksEditorControls *destination_window,
 		gtk_object_set_user_data (GTK_OBJECT (menu_item), 
 					  source);
 	} else {
-		BookmarkItem *item = destination_window->root_bookmark;
-		GtkWidget *menu_item = bookmarks_create_menu_item (item, res);
+		BookmarkItem *item = bookmarks_root;
+		GtkWidget *menu_item = bookmarks_menu_create_item (item, res);
 		GtkWidget *sep = gtk_menu_item_new ();
 		gtk_widget_show (menu_item);
 		gtk_widget_show (sep);
@@ -476,8 +520,7 @@ bookmarks_create_copy_menu (BookmarksEditorControls *destination_window,
 }
 
 GtkMenu*
-bookmarks_create_copy_menu_recursively (BookmarksEditorControls *destination_window, 
-					BookmarkItem *source, BookmarkItem *b)
+bookmarks_create_copy_menu_recursively (BookmarkItem *source, BookmarkItem *b)
 {
 	GtkMenu *res = GTK_MENU (gtk_menu_new ());
 	GList *l;
@@ -486,17 +529,15 @@ bookmarks_create_copy_menu_recursively (BookmarksEditorControls *destination_win
 	for (l = b->list; l != NULL; l = g_list_next (l)) {
 		BookmarkItem *item = l->data;
 		GtkWidget *menu_item, *sub_menu_item, *sep;
-		if (item->type == BM_CATEGORY) {
-			GtkMenu *submenu = 
-				bookmarks_create_copy_menu_recursively 
-				(destination_window, source, item);
+		if (item->type == BM_CATEGORY && !(item->alias_of)) {
+			GtkMenu *submenu = bookmarks_create_copy_menu_recursively (source, item);
 			has_sub_categories = TRUE;
-			sub_menu_item = bookmarks_create_menu_item (item, res);
+			sub_menu_item = bookmarks_menu_create_item (item, res);
 			if (submenu) {
 				gtk_menu_item_set_submenu 
 					(GTK_MENU_ITEM (sub_menu_item), 
 					 GTK_WIDGET (submenu));
-				menu_item = bookmarks_create_menu_item (item, submenu);
+				menu_item = bookmarks_menu_create_item (item, submenu);
 				gtk_widget_show (menu_item);
 				sep = gtk_menu_item_new ();
 				gtk_widget_show (sep);
@@ -515,7 +556,7 @@ bookmarks_create_copy_menu_recursively (BookmarksEditorControls *destination_win
 			gtk_widget_show (sub_menu_item);
 			gtk_menu_append (res, sub_menu_item);
 		}
-
+		
 	}
 	if (has_sub_categories) {
 		return res;
@@ -532,10 +573,10 @@ bookmarks_create_copy_menu_recursively (BookmarksEditorControls *destination_win
  */
 /* This may be improved */
 static GtkWidget *
-bookmarks_create_menu_item (BookmarkItem *bm, GtkMenu *menu)
+bookmarks_menu_create_item (BookmarkItem *bm, GtkMenu *menu)
 {
 	GtkWidget *pixmap;
-	PixmapData *bm_icon;
+	const PixmapData *bm_icon;
 	
 	GtkWidget *m = gtk_pixmap_menu_item_new ();
 	GtkWidget *l;
@@ -543,7 +584,7 @@ bookmarks_create_menu_item (BookmarkItem *bm, GtkMenu *menu)
 
 	g_return_val_if_fail (bm->name != NULL, NULL);
 
-	if (bm->pixmap_data == NULL)
+	if (bm->pixmap_data == NULL || bm->pixmap_data->pixmap == NULL)
 	{
 		l = gtk_label_new ("");
 		if (strlen (bm->name) > BOOKMARKS_MENU_MAX_LENGTH)
@@ -635,13 +676,13 @@ bookmarks_smart_bm_dialog (GaleonWindow *window, BookmarkItem *bi,
 
 	if (num_parameters == 1)
 	{
-		dlg_status = bookmarks_smart_bm_dialog_single(dialog, window,
-							      bi, new_window);
+		dlg_status = bookmarks_smart_bm_dialog_single (dialog, window,
+							       bi, new_window);
 	}
 	else if (num_parameters > 1)
 	{
-		dlg_status = bookmarks_smart_bm_dialog_multi(dialog, window,
-							     bi, new_window);
+		dlg_status = bookmarks_smart_bm_dialog_multi (dialog, window,
+							      bi, new_window);
 	}
 	if (dlg_status != -1)
 		gnome_dialog_close (GNOME_DIALOG (dialog));
@@ -683,7 +724,7 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonWindow *window,
 		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
 		if (new_window)
 			embed_create_from_url(window->active_embed, result,
-					      !tabbed_mode);
+					      TRUE, !tabbed_mode);
 		else
 			embed_load_url (window->active_embed, result);
 
@@ -798,7 +839,7 @@ bookmarks_smart_bm_dialog_multi (GtkWidget *dialog, GaleonWindow *window,
 		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
 		if (new_window)
 			embed_create_from_url(window->active_embed, result,
-					      !tabbed_mode);
+					      TRUE, !tabbed_mode);
 		else
 			embed_load_url (window->active_embed, result);
 
