@@ -17,17 +17,51 @@
  */
 
 #include "galeon.h"
+#include "bookmarks.h"
+#include "embed.h"
+#include "window.h"
+#include "misc.h"
+#include "find.h"
+
+#include <string.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-preferences.h>
+#include <libgnomeui/gnome-popup-menu.h>
+#include <libgnomeui/gnome-stock.h>
+#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtktoolbar.h>
+#include <gtk/gtkbox.h>
+
+extern const GtkTargetEntry bookmarks_dnd_targets[];
+
+/* local function prototypes */
+static void bookmarks_toolbars_create_from_root (GaleonWindow *window, 
+						 BookmarkItem *bi);
+static void  bookmarks_toolbar_create_from_category (GaleonWindow *window,
+						     BookmarkItem *bi);
+static void bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, 
+						    GtkToolbar *tb,
+						    BookmarkItem *b);
+static gint bookmarks_folder_show_menu_cb (GtkWidget *menu);
+static void  bookmarks_nick_entry_activate_cb (GtkEntry *entry, 
+					       BookmarkItem *bi);
+static gboolean bookmarks_toolbar_item_button_release_event_cb 
+                                       (GtkWidget *item, 
+					GdkEventButton *event,
+					BookmarkItem *bm);
+static gboolean bookmarks_toolbar_button_release_event_cb 
+                                       (GtkWidget *item, 
+					GdkEventButton *event,
+					BookmarkItem *bm);
+static void bookmarks_toolbar_drag_data_received_cb 
+                                       (GtkWidget *widget, 
+					GdkDragContext *drag_context,
+					gint x, gint y,
+					GtkSelectionData *selection_data,
+					guint info, guint time,
+					BookmarkItem *bm);
 
-static void bookmarks_create_toolbars (GaleonWindow *window, 
-				       BookmarkItem *bi);
-static void bookmarks_create_toolbar_from (GaleonWindow *window, 
-					   BookmarkItem *bi);
-static gint bookmarks_folder_show_menu (GtkWidget *menu);
-gboolean
-bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item, 
-						GdkEventButton *event,
-						BookmarkDragItem *bookmark_info);
- 
 /* this magic number is used to hack around a problem with
  * gnome-libs (I think) which causes multiple toolbars with
  * the same number not to display properly -- MattA
@@ -37,124 +71,216 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 static gint magic_number;
 
 /**
- * bookmarks_create_toolbars: create personal toolbars
+ * bookmarks_toolbars_create: create personal toolbars
  */
-static void 
-bookmarks_create_toolbars (GaleonWindow *window, BookmarkItem *bi)
+void 
+bookmarks_toolbars_create (GaleonWindow *window)
 {
-	GList *l;
+	gboolean visible;
 
-	/* skip if we're not a category */
-	if ((bi->type != BM_CATEGORY) && (bi->type != BM_AUTOBOOKMARKS))
+	/* if no toolbars, skip this */
+	if (!(window->toolBarOn))
+	{
 		return;
-	
-	/* if we're meant to create a toolbar... */
-	if (bi->create_toolbar)	{
-		/* do so */
-		bookmarks_create_toolbar_from (window, bi);
 	}
 
-	/* now iterate to find any marked subfolders, unless this is an alias */
-	if (! bi->alias_of)
-		for (l = bi->list; l != NULL; l = l->next) {
-			bookmarks_create_toolbars (window, l->data);
-		}
-}
-
-/**
- * bookmarks_create_toolbar_from: create a toolbar from a folder
- */
-static void 
-bookmarks_create_toolbar_from (GaleonWindow *window, BookmarkItem *bi)
-{
-	GnomeDockItemBehavior props;
-	gboolean toolbar_detachable;
-	GtkToolbar *tb;
-	GList *l;
-	gchar *unique_name;
-
-	/* make the toolbar */
-	tb = GTK_TOOLBAR(gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
-					  GTK_TOOLBAR_TEXT));
-
-	/* add the toolbar it to the main window */
-	/* this has to be done here for the relief to get set */
-	toolbar_detachable = gnome_preferences_get_toolbar_detachable ();
-	props = toolbar_detachable ? 0 : GNOME_DOCK_ITEM_BEH_LOCKED;
-	props |= GNOME_DOCK_ITEM_BEH_EXCLUSIVE;
-
-	/* make sure each toolbar has a unique name */
-	unique_name = g_strdup_printf("%s%d", bi->name, magic_number);
-
-	gnome_app_add_toolbar (GNOME_APP (window->WMain), tb, unique_name,
-			       props, GNOME_DOCK_TOP, magic_number++, 0, 0);
-
-	/* set border to match button toolbar */
-	gtk_container_set_border_width (GTK_CONTAINER (tb), 2);
-
-	/* override global setting */
-	gtk_toolbar_set_space_style (tb, GTK_TOOLBAR_SPACE_EMPTY);
-	gtk_toolbar_set_space_size (tb, 6);
-
-	/* iterate over the contents of the folder */
-	for (l = bi->list; l != NULL; l = l->next)
+	/* hack, we need probably to unify not created / not visible case */
+	/* FIXME: shouldnt need menu bar on to know visiblity of toolbar */
+	if (window->menuBarOn && window->view_toolbar)
 	{
-		BookmarkItem *b = l->data;
-
-		/* build the appropriate element */
-		if (b->type != BM_SEPARATOR)
-		{
-			bookmarks_create_toolbar_from_bm(window, tb, b, -1);
-		} 
-		else 
-		{
-			gtk_toolbar_append_space(tb);
-		}
-	}
-
-	/* show it */
-	gtk_widget_show_all (GTK_WIDGET(tb));
-
-	/* add it to the per-window list of toolbars */
-	window->bookmarks_toolbars = 
-		g_list_append (window->bookmarks_toolbars, tb);
-}
-
-/**
- * bookmarks_create_toolbar_from_bm:
- *
- * drag_info should not be freed
- *
- */
-void
-bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
-				  BookmarkItem *b, gint insert_pos)
-{
-	GtkWidget *button, *hbox, *pixmap, *entry;
-	GtkReliefStyle relief;
-	GtkWidget *dnd_dest;
-	BookmarkDragItem *drag_info = g_malloc0 (sizeof(BookmarkDragItem));
-	PixmapData *bm_icon;
-	
-	/* get the button relief setting */
-	relief = gtk_toolbar_get_button_relief (tb);
-
-	if (b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS)
-	{
-		pixmap = gtk_pixmap_new (folder_pixmap_data->pixmap,
-					 folder_pixmap_data->mask);
+		visible = GTK_CHECK_MENU_ITEM (window->view_toolbar)->active;
 	}
 	else
 	{
-		bm_icon = bookmarks_get_siteicon (b->url);
-		pixmap = gtk_pixmap_new (bm_icon->pixmap, bm_icon->mask);
+		visible = TRUE;
 	}
 
+	/* reset the magic number */
+	magic_number = 99;
+
+	/* build all the toolbars */
+	bookmarks_toolbars_create_from_root (window, bookmarks_root);
+
+	/* setup visibility of them all */
+	bookmarks_toolbars_set_visibility (window, visible);
+}
+
+/**
+ * bookmarks_toolbars_recreate: recreate personal toolbars
+ */
+void 
+bookmarks_toolbars_recreate (GaleonWindow *window)
+{
+	GList *l;
+
+	/* destroy all the toolbars */
+	for (l = window->bookmarks_toolbars; l != NULL; l = g_list_next (l))
+	{
+		GtkToolbar *toolbar = GTK_TOOLBAR (l->data);
+		gtk_widget_destroy (GTK_WIDGET (toolbar)->parent);
+	}
+
+	/* list is now empty */
+	g_list_free (window->bookmarks_toolbars);
+	window->bookmarks_toolbars = NULL;
+
+	/* now create toolbars for all of these */
+	bookmarks_toolbars_create (window);
+}
+
+/**
+ * bookmarks_toolbars_create_from_root: create toolbars from a given
+ * root bookmark
+ */
+static void
+bookmarks_toolbars_create_from_root (GaleonWindow *window, BookmarkItem *bi)
+{
+	GList *l;
+
+	switch (bi->type)
+	{
+	case BM_SITE:
+	case BM_SEPARATOR:
+		/* can't create a toolbar from these */
+		return;
+
+	case BM_CATEGORY:
+	case BM_AUTOBOOKMARKS:
+		/* continue... */
+		break;
+	}
+
+	/* if we're meant to create a toolbar... */
+	if (bi->create_toolbar)
+	{
+		/* do so */
+		bookmarks_toolbar_create_from_category (window, bi);
+	}
+	
+	/* now iterate to find any marked subfolders, 
+	 * unless this is an alias */
+	if (!(bi->alias_of))
+	{
+		for (l = bi->list; l != NULL; l = g_list_next (l))
+		{
+			bookmarks_toolbars_create_from_root (window, l->data);
+		}
+	}
+}
+
+/**
+ * bookmarks_toolbar_create_from_category: create a toolbar from a folder
+ */
+static void 
+bookmarks_toolbar_create_from_category (GaleonWindow *window, BookmarkItem *bi)
+{
+	gboolean toolbar_relief;
+	gboolean toolbar_detachable;
+	GnomeDockItemBehavior props;
+	GtkWidget *toolbar;
+	gchar *unique_name;
+	GList *l;
+
+	/* read gnome prefs */
+	toolbar_relief = gnome_preferences_get_toolbar_relief ();
+	toolbar_detachable = gnome_preferences_get_toolbar_detachable ();
+
+	/* make the toolbar */
+	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, 
+				   GTK_TOOLBAR_TEXT);
+
+	/* check whether it should be detachable or not */
+	props = (GNOME_DOCK_ITEM_BEH_EXCLUSIVE |
+		 (toolbar_detachable ? 0 : GNOME_DOCK_ITEM_BEH_LOCKED));
+
+	/* make sure each toolbar has a unique name */
+	unique_name = g_strdup_printf("%ld", bi->id);
+
+	/* add the toolbar it to the main window */
+	gnome_app_add_docked (GNOME_APP (window->WMain), toolbar, unique_name,
+			      props, GNOME_DOCK_TOP, magic_number++, 0, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (toolbar->parent),
+					toolbar_relief ? 2 : 0);
+	gnome_dock_item_set_shadow_type (GNOME_DOCK_ITEM (toolbar->parent), 
+					 toolbar_relief ? 
+					 GTK_SHADOW_OUT : GTK_SHADOW_NONE);
+	g_free (unique_name);
+
+	/* override global setting */
+	gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar),
+				     GTK_TOOLBAR_SPACE_EMPTY);
+	gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar), 6);
+
+	/* iterate over the contents of the folder */
+	for (l = bi->list; l != NULL; l = g_list_next (l))
+	{
+		BookmarkItem *bm = l->data;
+
+		/* build the appropriate element */
+		if (bm->type != BM_SEPARATOR)
+		{
+			bookmarks_toolbar_create_from_bookmark
+				(window, GTK_TOOLBAR (toolbar), bm);
+		} 
+		else 
+		{
+			gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+		}
+	}
+
+	gtk_signal_connect (GTK_OBJECT (toolbar), "drag_data_received",
+			    GTK_SIGNAL_FUNC 
+			    (bookmarks_toolbar_drag_data_received_cb), bi);
+	gtk_drag_dest_set (toolbar,
+			   GTK_DEST_DEFAULT_DROP,
+			   bookmarks_dnd_targets,
+			   bookmarks_dnd_targets_num_items,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE |
+			   GDK_ACTION_LINK);
+
+	/* the click handler */
+	gtk_signal_connect (GTK_OBJECT (toolbar->parent), 
+			    "button-release-event",
+			    GTK_SIGNAL_FUNC 
+			    (bookmarks_toolbar_button_release_event_cb), bi);
+
+	/* show it */
+	gtk_widget_show_all (GTK_WIDGET (toolbar));
+
+	/* add it to the per-window list of toolbars */
+	window->bookmarks_toolbars = 
+		g_list_append (window->bookmarks_toolbars,
+			       GTK_TOOLBAR (toolbar));
+}
+
+/**
+ * bookmarks_toolbar_create_from_bookmark:
+ */
+static void
+bookmarks_toolbar_create_from_bookmark (GaleonWindow *window, GtkToolbar *tb,
+					BookmarkItem *b)
+{
+        GtkWidget *button, *hbox, *pixmap, *entry;
+	const PixmapData *bm_icon;
+	GtkWidget *dnd_dest;
+	gchar *name;
+	
 	/* build a box, and pack the pixmap and label inside */
 	hbox = gtk_hbox_new (FALSE, 2);
 	if (b->pixmap_data == NULL)
 	{
-		gchar *name = strip_uline_accel (b->name);
+		if (b->type == BM_CATEGORY || b->type == BM_AUTOBOOKMARKS)
+		{
+			pixmap = gtk_pixmap_new (folder_pixmap_data->pixmap,
+						 folder_pixmap_data->mask);
+		}
+		else
+		{
+			bm_icon = bookmarks_get_siteicon (b->url);
+			pixmap = gtk_pixmap_new (bm_icon->pixmap, 
+						 bm_icon->mask);
+		}
+		name = strip_uline_accel (b->name);
 		gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (name),
 				    TRUE, TRUE, 0);
@@ -162,9 +288,9 @@ bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
 	}
 	else
 	{
-		GtkWidget *image  = gtk_pixmap_new (b->pixmap_data->pixmap,
-						    b->pixmap_data->mask);
-		gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (image),
+		pixmap = gtk_pixmap_new (b->pixmap_data->pixmap, 
+					 b->pixmap_data->mask);
+		gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (pixmap),
 				    TRUE, FALSE, 0);
 	}
 
@@ -174,12 +300,8 @@ bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
 		/* yes, build a button and pack it */
 		dnd_dest = button = gtk_button_new ();
 		gtk_container_add(GTK_CONTAINER(button), hbox);
-		if (insert_pos == -1)
-			gtk_toolbar_append_widget(tb, button, b->url, NULL);
-		else
-			gtk_toolbar_insert_widget(tb, button, b->url, NULL,
-						  insert_pos);
-		gtk_button_set_relief (GTK_BUTTON (button), relief);
+		gtk_toolbar_append_widget(tb, button, b->url, NULL);
+		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
 	}
 	else
@@ -192,7 +314,7 @@ bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
 			("/galeon/Appearance/smart_bm_entry_width=100");
 		gtk_container_add (GTK_CONTAINER (button), 
 				   GTK_WIDGET (hbox));
-		gtk_button_set_relief (GTK_BUTTON (button), relief);
+		gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
 		gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
 		/* otherwise build an entry field and pack it */
@@ -207,16 +329,12 @@ bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
 		gtk_object_set_data (GTK_OBJECT (entry), "GaleonWindow", 
 				     window);
 		gtk_signal_connect (GTK_OBJECT (entry), "activate",
-				    bookmarks_nick_entry_activate, b);
+				    bookmarks_nick_entry_activate_cb, b);
 	}
 
-	/* dnd */
-	drag_info->window = window;
-	drag_info->bookmark = b;
-	drag_info->toolbar = tb;
 	gtk_signal_connect (GTK_OBJECT (dnd_dest), "drag_data_received",
-		       GTK_SIGNAL_FUNC (bookmarks_toolbar_drag_data_received_cb),
-		       drag_info);
+			    GTK_SIGNAL_FUNC 
+			    (bookmarks_toolbar_drag_data_received_cb), b);
 	gtk_drag_dest_set (dnd_dest,
 			   GTK_DEST_DEFAULT_ALL,
 			   bookmarks_dnd_targets,
@@ -226,143 +344,33 @@ bookmarks_create_toolbar_from_bm (GaleonWindow *window, GtkToolbar *tb,
 			   
 	/* the click handler */
 	gtk_signal_connect (GTK_OBJECT (dnd_dest), "button-release-event",
-		GTK_SIGNAL_FUNC (bookmarks_toolbar_item_button_release_event_cb),
-		drag_info);				
-}
-
-void
-bookmarks_create_tb_global (void)
-{
-	GList *sl;
-	gchar *layout;
-
-	/* get the saved layout string */
-	layout = gnome_config_get_string ("/galeon/Placement/Dock");
-
-	for (sl = all_windows; sl != NULL; sl = sl->next) 
-	{
-		GaleonWindow *window = (GaleonWindow *) sl->data;
-		GnomeApp *app = GNOME_APP (window->WMain);
-		GList *lp;
-		GnomeDockLayoutItem *item;
-
-		/* Create new layout object, so gnome_app_toolbar will add the
-		   toolbars to the layout instead of directly to the dock */
-		app->layout = gnome_dock_layout_new();
-
-		/* Create the menus and the toolbars for this window */
-		bookmarks_create_tb (sl->data);
-
-		/* Restore the saved layout */
-		gnome_dock_layout_parse_string (app->layout, layout);
-
-		/* Have to do this manually to get the ordering right */
-		lp = app->layout->items;
-		while (lp)
-		{
-			item = lp->data;
-
-			if (item->placement == GNOME_DOCK_FLOATING)
-			{
-				gnome_dock_add_floating_item
-					(GNOME_DOCK (app->dock),
-					 item->item,
-					 item->position.floating.x,
-					 item->position.floating.y,
-					 item->position.floating.orientation);
-			}
-			else
-			{
-				gnome_dock_add_item
-					(GNOME_DOCK (app->dock),
-					 item->item,
-					 item->placement,
-					 item->position.docked.band_num,
-					 0,
-					 item->position.docked.offset,
-					 TRUE);
-			}
-			lp = lp->next;
-		}
-
-		/* Get rid of the layout object, it's no longer needed */
-		gtk_object_unref (GTK_OBJECT (app->layout));
-		app->layout = NULL;
-	}
-	g_free (layout);
-}
-
-void
-bookmarks_remove_tb_global (void)
-{
-	GList *sl;
-	GList *li;
-
-	/* Destroys the toolbars */
-	for (sl = all_windows; sl != NULL; sl = sl->next)
-	{
-		GaleonWindow *window = (GaleonWindow *) sl->data;
-		GList *toolbars = window->bookmarks_toolbars;
-
-		/* Destroy the dock band containing the toolbar */
-		for (li = toolbars; li != NULL; li = li->next)
-		{
-			GtkWidget *toolbar = li->data;
-			GtkWidget *dock_item, *dock_band;
-
-			/* Probably overkill, but check to be safe */
-			if (toolbar == NULL)
-				continue;
-
-			dock_item = toolbar->parent;
-			if (dock_item == NULL)
-			{
-				gtk_widget_destroy (toolbar);
-				continue;
-			}
-
-			dock_band = dock_item->parent;
-			if (dock_band == NULL)
-			{
-				gtk_widget_destroy (dock_item);
-				continue;
-			}
-
-			gtk_widget_destroy (dock_band);
-		}
-		g_list_free (toolbars);
-		window->bookmarks_toolbars = NULL;
-	}
+			    GTK_SIGNAL_FUNC 
+			    (bookmarks_toolbar_item_button_release_event_cb),
+			    b);				
 }
 
+#if 0
 /**
- * bookmarks_update_tb: destroy and recreate the boomarks toolbars
- */
-void 
-bookmarks_update_tb (void) 
-{
-	bookmarks_remove_tb_global ();
-	bookmarks_create_tb_global ();
-}
-
-/**
- * bookmarks_tb_update_entries_global: search the bookmarks toolbars for
+ * bookmarks_toolbars_update_entries_global: search the bookmarks toolbars for
  *	smart bookmark entry widgets and sync their sizes with the
  *	preferences setting
  */
 void
-bookmarks_tb_update_entries_global (void)
+bookmarks_toolbars_update_entries_global (void)
 {
-	GList *wlist = all_windows;
-	gint entry_width = gnome_config_get_int
-				("/galeon/Appearance/smart_bm_entry_width=100");
+	gint entry_width;
+	GList *toolbars;
+	GList *l;
 
-	while (wlist)
+	entry_width = gnome_config_get_int
+		("/galeon/Appearance/smart_bm_entry_width=100");
+
+	for (l = all_windows; l != NULL; l = g_list_next (l))
 	{
-		GaleonWindow *window = (GaleonWindow *) wlist->data;
-		GList *toolbars = window->bookmarks_toolbars;
+		GaleonWindow *window = (GaleonWindow *)(l->data);
 
-		while (toolbars)
+		for (toolbars = window->bookmarks_toolbars; toolbars != NULL;
+		     toolbars = g_list_next (toolbars))
 		{
 			GtkToolbar *toolbar = GTK_TOOLBAR (toolbars->data);
 			GList *children = toolbar->children;
@@ -378,34 +386,31 @@ bookmarks_tb_update_entries_global (void)
 				}
 				children = children->next;
 			}
-			toolbars = toolbars->next;
 		}
-		wlist = wlist->next;
 	}			
 }
+#endif
 
 void 
-bookmarks_create_tb (GaleonWindow *window)
-{
-	gboolean visible = GTK_CHECK_MENU_ITEM (window->view_toolbar)->active;
-
-	magic_number = 99;
-	bookmarks_create_toolbars (window, bookmarks_root);
-	bookmarks_toolbar_set_visible (window, visible);
-}
-void 
-bookmarks_toolbar_set_visible (GaleonWindow *window, gboolean visible)
+bookmarks_toolbars_set_visibility (GaleonWindow *window, gboolean visible)
 {
 	GList *l;
-	for (l = window->bookmarks_toolbars; l != NULL; l = l->next) {
-		if (GTK_IS_WIDGET (l->data)) {
-			GtkWidget *w = NULL;
-			if (GTK_IS_WIDGET (GTK_WIDGET (l->data)->parent)) {
-				w = GTK_WIDGET (l->data)->parent;
-				if (visible) 
-					gtk_widget_show (w);
-				else
-					gtk_widget_hide (w);
+
+	for (l = window->bookmarks_toolbars; l != NULL; l = g_list_next (l))
+	{
+		GtkToolbar *toolbar = GTK_TOOLBAR (l->data);
+
+		if (GTK_IS_WIDGET (GTK_WIDGET (toolbar)->parent))
+		{
+			GtkWidget *w = GTK_WIDGET (toolbar)->parent;
+
+			if (visible)
+			{
+				gtk_widget_show (w);
+			}
+			else
+			{
+				gtk_widget_hide (w);
 			}
 		}
 	}
@@ -423,16 +428,16 @@ bookmarks_toolbar_set_visible (GaleonWindow *window, gboolean visible)
  * Button 2: Load bookmark in new tab/window
  * Button 3: Show context menu
  */
-gboolean
+static gboolean
 bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item, 
 						GdkEventButton *event,
-						BookmarkDragItem *bookmark_info)
+						BookmarkItem *bi)
 {
 	static GnomeUIInfo toolbar_item_context_menu_uiinfo[] =
 	{
 		GNOMEUIINFO_ITEM_STOCK (N_("Edit..."), NULL, NULL,
 				        GNOME_STOCK_MENU_PROP),
-		GNOMEUIINFO_ITEM_STOCK (N_("Remove"), NULL, NULL,
+		GNOMEUIINFO_ITEM_STOCK (N_("Remove bookmark"), NULL, NULL,
 				        GNOME_STOCK_MENU_CLOSE),
 		GNOMEUIINFO_SEPARATOR,
 		GNOMEUIINFO_ITEM_STOCK (N_("Hide toolbar"), NULL, NULL,
@@ -442,12 +447,12 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 	GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(item), "WMain");
 	GaleonWindow *window = gtk_object_get_data(GTK_OBJECT(WMain),
 						   "GaleonWindow");
-	BookmarkItem *bi = bookmark_info->bookmark;	
 	GtkWidget *entry;
 	GtkWidget *popup;
 	gint item_action;
 	gboolean autobookmarks_deleted = FALSE;
 	gchar *text;
+	BookmarksEditorControls *controls;
 	
 	if (item->window != event->window)
 		return FALSE;
@@ -472,6 +477,19 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 				{
 					find_next (window->active_embed, text);
 				}
+				else
+				{
+					BookmarkItem *b = 
+						bookmarks_find_by_nick (
+								bi->nick,
+								bookmarks_root,
+								FALSE);
+					if (b != NULL && b->url != NULL)
+					{
+						window_load_url (window, 
+								 b->url);
+					}
+				}
 			}
 		}
 		else if ((bi->type == BM_CATEGORY) || (bi->type == BM_AUTOBOOKMARKS))
@@ -479,7 +497,7 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 			GtkWidget *menu = gtk_menu_new();
 			GtkTooltips *tooltips = gtk_tooltips_new ();
 
-			bookmarks_create_menu_recursively
+			bookmarks_menu_create_recursively
 				(bi, GTK_MENU (menu), NULL, tooltips, 
 				 TRUE, TRUE);
 
@@ -493,7 +511,7 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 			   clean up any pointer grabs before displaying the
 			   popup menu */
 			gtk_timeout_add (50,
-				(GtkFunction) bookmarks_folder_show_menu,
+				(GtkFunction) bookmarks_folder_show_menu_cb,
 				menu);
 		}
 	}
@@ -509,7 +527,7 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 		if ((bi->type == BM_SITE) && !GTK_IS_EVENT_BOX(item))
 		{
 			embed_create_from_url (window->active_embed, bi->url,
-					       !tabbed_mode);
+					       FALSE, !tabbed_mode);
 
 		}
 		else if (bi->type == BM_AUTOBOOKMARKS || bi->type == BM_CATEGORY)
@@ -526,30 +544,76 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 							      NULL, NULL, NULL);
 		gtk_widget_destroy(popup);
 		
-		switch (item_action) {
+		switch (item_action) 
+		{
 		case TOOLBAR_ITEM_CONTEXT_MENU_EDIT_POS:
-			bookmarks_editor_show_dialog (
-						window, 
-						STANDARD_BOOKMARKS_EDITOR);
-			bookmarks_editor_goto_bookmark (bookmark_info->bookmark);
+			controls = bookmarks_editor_show_dialog 
+				(window, STANDARD_BOOKMARKS_EDITOR);
+			bookmarks_editor_select_bookmark (controls, bi);
 			break;
 		case TOOLBAR_ITEM_CONTEXT_MENU_REMOVE_POS:
-			if (bookmark_info->bookmark->type == BM_AUTOBOOKMARKS)
-				autobookmarks_deleted = TRUE;
-			if (bookmarks_editor)
+			if (bi->type == BM_AUTOBOOKMARKS)
 			{
-				bookmarks_editor_remove_tree_item
-						       (bookmarks_editor,
-							bookmark_info->bookmark);
-			}
-			bookmarks_remove_recursively (bookmark_info->bookmark);
+				autobookmarks_deleted = TRUE;
+			}			
+			bookmarks_editor_remove_tree_items (bi);
+			bookmarks_remove_recursively (bi);
 			if (autobookmarks_deleted)
+			{
 				autobookmarks_root = NULL;
-			bookmarks_save();
+			}
+			bookmarks_updated ();
 			break;
 		case TOOLBAR_ITEM_CONTEXT_MENU_HIDE_TOOLBAR_POS:
-			bookmark_info->bookmark->parent->create_toolbar = FALSE;
-			bookmarks_save();
+			bi->parent->create_toolbar = FALSE;
+			bookmarks_updated ();
+			break;
+		}
+		return FALSE;
+	}
+	return TRUE;
+}
+
+#define TOOLBAR_CONTEXT_MENU_EDIT_POS 0
+#define TOOLBAR_CONTEXT_MENU_HIDE_TOOLBAR_POS 2
+
+static gboolean
+bookmarks_toolbar_button_release_event_cb (GtkWidget *item, 
+					   GdkEventButton *event,
+					   BookmarkItem *bm)
+{
+	static GnomeUIInfo toolbar_item_context_menu_uiinfo[] =
+	{
+		GNOMEUIINFO_ITEM_STOCK (N_("Edit..."), NULL, NULL,
+				        GNOME_STOCK_MENU_PROP),
+		GNOMEUIINFO_SEPARATOR,
+		GNOMEUIINFO_ITEM_STOCK (N_("Hide toolbar"), NULL, NULL,
+					GNOME_STOCK_MENU_BLANK),
+		GNOMEUIINFO_END
+	};
+	GtkWidget *popup;
+	gint item_action;
+	BookmarksEditorControls *controls;
+	GtkWidget *WMain = window_lookup_widget (GTK_WIDGET(item), "WMain");
+	GaleonWindow *window = gtk_object_get_data(GTK_OBJECT(WMain),
+						   "GaleonWindow");
+
+	if (event->button == 3)
+	{
+		popup = gnome_popup_menu_new(toolbar_item_context_menu_uiinfo);
+		item_action = gnome_popup_menu_do_popup_modal(popup, NULL,
+							      NULL, NULL, NULL);
+		gtk_widget_destroy(popup);
+		
+		switch (item_action) {
+		case TOOLBAR_CONTEXT_MENU_EDIT_POS:
+			controls = bookmarks_editor_show_dialog 
+				(window, STANDARD_BOOKMARKS_EDITOR);
+			bookmarks_editor_select_bookmark (controls, bm);
+			break;
+		case TOOLBAR_CONTEXT_MENU_HIDE_TOOLBAR_POS:
+			bm->create_toolbar = FALSE;
+			bookmarks_updated ();
 			break;
 		}
 		return FALSE;
@@ -558,7 +622,7 @@ bookmarks_toolbar_item_button_release_event_cb (GtkWidget *item,
 }
 
 static int
-bookmarks_folder_show_menu (GtkWidget *menu)
+bookmarks_folder_show_menu_cb (GtkWidget *menu)
 {
 	GtkTooltips *tooltips = gtk_object_get_data (GTK_OBJECT (menu),
 						     "tooltips");
@@ -578,12 +642,12 @@ bookmarks_folder_show_menu (GtkWidget *menu)
 }
 
 /**
- * bookmarks_nick_entry_activate: called when the user hits return
+ * bookmarks_nick_entry_activate_cb: called when the user hits return
  * on an entry field on the toolbar (i.e. one created by a bookmark
  * which has %s in the url)
  */
-void 
-bookmarks_nick_entry_activate (GtkEntry *entry, BookmarkItem *bookmark)
+static void 
+bookmarks_nick_entry_activate_cb (GtkEntry *entry, BookmarkItem *bi)
 {
 	gchar *text, *url, *translated_text;
 	GaleonWindow *window;
@@ -595,10 +659,10 @@ bookmarks_nick_entry_activate (GtkEntry *entry, BookmarkItem *bookmark)
 	text = gtk_entry_get_text (entry);
 
 	/* translate non-alphanumeric characters into %{} format */
-	translated_text = bookmarks_translate_string(text);
+	translated_text = bookmarks_translate_string (text);
 
 	/* get a completed url */
-	url = bookmarks_substitute_argument (bookmark, translated_text);
+	url = bookmarks_substitute_argument (bi, translated_text);
 
 	/* load it in this window */
 	window_load_url (window, url);
@@ -609,82 +673,41 @@ bookmarks_nick_entry_activate (GtkEntry *entry, BookmarkItem *bookmark)
 }
 
 /** 
- * bookmarks_toolbar_drag_data_received_cb::
+ * bookmarks_toolbar_drag_data_received_cb:
  */
-void
+static void
 bookmarks_toolbar_drag_data_received_cb (GtkWidget *widget, 
 					 GdkDragContext *drag_context,
 					 gint x, gint y,
 					 GtkSelectionData *selection_data,
 					 guint info, guint time,
-					 BookmarkDragItem *drag_info)
+					 BookmarkItem *drag_bm)
 {
 	BookmarkItem *b = NULL; 
 	gchar *mem = selection_data->data;
 	
-	if ( !drag_info ) return; /* should not happen */
+	g_return_if_fail (drag_bm != NULL);
 
-	switch (info) {
+	switch (info)
+	{
 	case DND_TARGET_GALEON_BOOKMARK:
 		b = bookmarks_item_from_string (mem);
 		break;
 	case DND_TARGET_STRING:
 	case DND_TARGET_NETSCAPE_URL:
 	case DND_TARGET_GALEON_URL:
-		b = bookmarks_new_bookmark (BM_SITE, TRUE, NULL, mem, NULL, NULL, NULL);
+		b = bookmarks_new_bookmark (BM_SITE, TRUE, NULL, mem, 
+					    NULL, NULL, NULL);
 		break;
 	default:
 		g_warning ("Unknown DND type");
 		break;
 	}
-	if (b) {
-		extern BookmarksEditorControls *bookmarks_editor; /*FIXME!*/
-		gchar *savefile;
-
-		bookmarks_insert_bookmark (b, drag_info->bookmark,
-					   GTK_CLIST_DRAG_INTO);
-		if (bookmarks_editor)
-			bookmarks_editor_place_tree_item(bookmarks_editor, b);
-
-		if (!(drag_info->bookmark->type == BM_CATEGORY ||
-		      drag_info->bookmark->type == BM_AUTOBOOKMARKS))
-		{
-			GList *children = drag_info->toolbar->children;
-			GtkToolbarChild *child = NULL;
-			gint insert_pos = 0;
-
-			/* Find the position of the widget we're dragging onto
-			   in the toolbar and insert after it */
-			while (children)
-			{
-				insert_pos++;
-				child = children->data;
-
-				if (child->widget == widget)
-					break;
-
-				children = children->next;
-			}
-
-			/* Make sure the new bookmark is inserted after a
-			   smart bookmark entry widget */
-			if (strstr(drag_info->bookmark->url, "%s"))
-				insert_pos++;
-
-			bookmarks_create_toolbar_from_bm (drag_info->window,
-							  drag_info->toolbar,
-							  b, insert_pos);
-		}
-		gtk_widget_show_all(GTK_WIDGET(drag_info->toolbar));
-
-		/* Save the bookmarks */
-		savefile = g_strconcat (g_get_home_dir(), 
-					"/.galeon/bookmarks.xml", NULL);
-		bookmarks_save_as (bookmarks_root, savefile);
-		g_free (savefile);
-
-		/* update the menus */
-		bookmarks_update_menu ();
+	if (b)
+	{
+		bookmarks_insert_bookmark (b, drag_bm, GTK_CLIST_DRAG_INTO);
+		bookmarks_editor_place_tree_items (b);
+		bookmarks_updated ();
 	 }
 }
 
