@@ -16,7 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <galeon.h>
+#include "galeon.h"
+#include "mozilla.h"
+
+#include <gtkmozembed.h>
 #include <gtkmozembed_internal.h>
 
 #include "regex.h"
@@ -31,19 +34,18 @@
 #include "nsIDocument.h"
 #include "nsFileSpec.h"
 #include "nsISHEntry.h"
-#ifdef CVS_SOURCE
 #include "nsISHistoryInternal.h"
 #include "nsIWebBrowserPrint.h"
-#else
-#include "nsINetDataCacheManager.h"
-#include "nsICachedNetData.h"
-#endif
 #include "nsIURI.h"
 #include "nsIURL.h"
 #include "nsIPresShell.h"
 #include "nsTextServicesCID.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIDOMHTMLElement.h"
+#include "nsIDOMHTMLInputElement.h"
+#if MOZILLA_VERSION > VERSION2(0,9)
+#include "nsIDOMNSHTMLElement.h"
+#endif
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMNamedNodeMap.h"
@@ -57,11 +59,26 @@
 #include "nsIChannel.h"
 #include "nsILoadGroup.h"
 #include "nsIWebBrowserPersist.h"
+#include "nsIStreamTransfer.h"
 #include "nsNetUtil.h"
 #include "nsPIDOMWindow.h"
 #include "nsIChromeEventHandler.h"
 #include "galeon-wrapper.h"
 
+#include "GlobalHistory.h"
+#include "nsIGlobalHistory.h"
+
+/* type of drag and drop for links */
+static const GtkTargetEntry link_drag_types[] =
+{
+        { "GALEON_URL",    0, DND_TARGET_GALEON_URL   },
+        { "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
+        { "STRING",        0, DND_TARGET_STRING       }
+};
+static const gint link_drag_types_num_items = (sizeof (link_drag_types) / 
+					       sizeof (GtkTargetEntry));
+static GtkTargetList *link_drag_types_tl = NULL;
+
 static NS_DEFINE_CID(kCTextServicesDocumentCID, NS_TEXTSERVICESDOCUMENT_CID);
 static NS_DEFINE_CID(kStandardURLCID, NS_STANDARDURL_CID);
 
@@ -80,7 +97,7 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 	return_val_if_not_embed (galeon_embed, NS_ERROR_FAILURE);
 
 	embed = galeon_embed;
-	mGtkMozEmbed = embed->mozEmbed;
+	mGtkMozEmbed = GTK_MOZ_EMBED(embed->mozEmbed);
 
 	gtk_moz_embed_get_nsIWebBrowser (mGtkMozEmbed,
 					 getter_AddRefs(mWebBrowser));
@@ -93,37 +110,56 @@ nsresult GaleonWrapper::Init (GaleonEmbed *galeon_embed)
 	nsCOMPtr<nsIDocShellHistory> dsHistory = do_QueryInterface (DocShell);
 	if (!dsHistory) return NS_ERROR_FAILURE;
 
-	NS_WITH_SERVICE(nsIGlobalHistory, history,
-			NS_GLOBALHISTORY_CONTRACTID, &result);  
-	if (NS_FAILED(result) || !history) return NS_ERROR_FAILURE;
-	dsHistory->SetGlobalHistory(history);
+	static NS_DEFINE_CID(kGlobalHistoryCID, GALEON_GLOBALHISTORY_CID);
 
-#if MOZILLA_MILESTONE != 81
-  	mChromeNav = do_QueryInterface(mWebBrowser);
-	if (!mChromeNav) return NS_ERROR_FAILURE;
-#else
-	nsCOMPtr<nsIDOMWindow> domWindow;
-	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-	if (!domWindow) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIFactory> GHFactory;
+	result = NS_NewGlobalHistoryFactory(getter_AddRefs(GHFactory));
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
+	result = nsComponentManager::RegisterFactory(kGlobalHistoryCID,
+						     "Global history",
+						     NS_GLOBALHISTORY_CONTRACTID,
+						     GHFactory,
+						     PR_TRUE);
+
+	NS_WITH_SERVICE(nsIGlobalHistory, inst, NS_GLOBALHISTORY_CONTRACTID, &result);
+ 	dsHistory->SetGlobalHistory(inst);
+
+	 nsCOMPtr<nsIDOMWindow> domWindow;
+        mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
+        if (!domWindow) return NS_ERROR_FAILURE;
+
+        nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
 	nsCOMPtr<nsIDOMWindowInternal> rootWindow;
-	domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
+        domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
   
-	nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-	nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
-	piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
+        nsCOMPtr<nsIChromeEventHandler> chromeHandler;
+        nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
+        piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
   
 	mEventReceiver = do_QueryInterface(chromeHandler);
 
-	mEventListener = new GaleonEventListener();
-	mEventListener->Init(this);
-#endif
+        mEventListener = new GaleonEventListener();
+        mEventListener->Init(this);
+
 	if (attachListener)
-	return AddListener ();
+	{
+		return AddListener ();
+	}
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::SetOffline(PRBool aOffline)
+{
+	nsresult rv;
+	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID, &rv);
+	if (NS_FAILED(rv))
+		return NS_ERROR_FAILURE;
+
+	return io->SetOffline(aOffline);
 }
 
-#ifdef CVS_SOURCE
 nsresult GaleonWrapper::Print (nsIPrintOptions *options)
 {
 	nsresult rv = NS_OK;
@@ -143,7 +179,6 @@ nsresult GaleonWrapper::Print (nsIPrintOptions *options)
 
 	return rv;
 }
-#endif
 
 nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 {
@@ -163,18 +198,12 @@ nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 
 	*aSHistory = SessionHistory.get();
 	NS_IF_ADDREF (*aSHistory);
+
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
 {
-#if MOZILLA_MILESTONE == 81
-	nsCOMPtr<nsIDocShell> DocShell (do_GetInterface(mWebBrowser));
-	if (!DocShell) return NS_ERROR_FAILURE;
-
-	*aDocShell = DocShell.get();
-
-	NS_IF_ADDREF(*aDocShell);
-#else
 	nsCOMPtr<nsIDocShellTreeItem> browserAsItem;
 	browserAsItem = do_QueryInterface(mWebBrowser);
 	if (!browserAsItem) return NS_ERROR_FAILURE;
@@ -197,7 +226,8 @@ nsresult GaleonWrapper::GetDocShell (nsIDocShell **aDocShell)
 	*aDocShell = DocShell.get();
 
 	NS_IF_ADDREF(*aDocShell);
-#endif
+	
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::Destroy ()
@@ -205,85 +235,42 @@ nsresult GaleonWrapper::Destroy ()
  	if (attachListener)
   		return RemoveListener ();
       	mWebBrowser = nsnull;
-#if MOZILLA_MILESTONE == 81
-	mEventReceiver = nsnull;
-#else
 	mChromeNav = nsnull;
-#endif
+	
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::AddListener ()
 {
 	nsresult rv = NS_OK;
 
-#if MOZILLA_MILESTONE == 81
-	if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
+        if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
 
-	nsIDOMEventListener *eventListener =
-		NS_STATIC_CAST(nsIDOMEventListener *,
-			       NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
+        nsIDOMEventListener *eventListener =
+                NS_STATIC_CAST(nsIDOMEventListener *,
+                               NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
 
-	rv = mEventReceiver->AddEventListenerByIID(eventListener,
-						 NS_GET_IID(nsIDOMDragListener));
-	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-#else
-	nsCOMPtr <nsIDOMDocument> domDoc;
-	mChromeNav->GetDocument(getter_AddRefs(domDoc));
-	if (!domDoc) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIDOMEventReceiver> eventReceiver;
-	eventReceiver = do_QueryInterface(domDoc);
-	if (!eventReceiver) return NS_ERROR_FAILURE;
-
-	GaleonEventListener *newListener = new GaleonEventListener();
-	NS_ADDREF(newListener);
-	if (!newListener) return NS_ERROR_FAILURE;
-
-	newListener->Init (this);
-	nsCOMPtr<nsIDOMEventListener> mEventListener = 
-				do_QueryInterface(NS_STATIC_CAST(nsISupports *,
-					NS_STATIC_CAST(nsIDOMDragListener *,
-					newListener)));
-	if (!mEventListener) return NS_ERROR_FAILURE;
-	NS_RELEASE(newListener);
-
-	rv = eventReceiver->AddEventListenerByIID(mEventListener,
-						 NS_GET_IID(nsIDOMDragListener));
-	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-#endif
+        rv = mEventReceiver->AddEventListenerByIID(eventListener,
+                                                 NS_GET_IID(nsIDOMDragListener));
+        if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+	
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::RemoveListener ()
 {
 	nsresult rv = NS_OK;
 
-#if MOZILLA_MILESTONE == 81
 	if (!mEventReceiver || !mEventListener) return NS_ERROR_FAILURE;
-	
-	nsIDOMEventListener *eventListener =
-		NS_STATIC_CAST(nsIDOMEventListener *,
-			       NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
+        
+        nsIDOMEventListener *eventListener =
+                NS_STATIC_CAST(nsIDOMEventListener *,
+                               NS_STATIC_CAST(nsIDOMDragListener *, mEventListener));
 
-	rv = mEventReceiver->RemoveEventListenerByIID(eventListener,
-						      NS_GET_IID(nsIDOMDragListener));
-	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-#else
-	if (!mEventListener) return NS_ERROR_FAILURE;
+        rv = mEventReceiver->RemoveEventListenerByIID(eventListener,
+                                                      NS_GET_IID(nsIDOMDragListener));
 
-	nsCOMPtr <nsIDOMDocument> domDoc;
-	mChromeNav->GetDocument(getter_AddRefs(domDoc));
-	if (!domDoc) return NS_ERROR_FAILURE;;
-
-	nsCOMPtr<nsIDOMEventReceiver> eventReceiver;
-	eventReceiver = do_QueryInterface(domDoc);
-	if (!eventReceiver) return NS_ERROR_FAILURE;
-
-	rv = eventReceiver->RemoveEventListenerByIID(mEventListener,
-						NS_GET_IID(nsIDOMDragListener));
-	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
-
-	mEventListener = NULL;
-#endif
+	return rv;
 }
 
 nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
@@ -301,21 +288,6 @@ nsresult GaleonWrapper::GoToHistoryIndex (PRInt16 index)
 	return  ContentNav->GotoIndex (index);
 }
 
-#ifndef CVS_SOURCE
-
-nsresult GaleonWrapper::SetViewSourceMode (PRInt32 mode)
-{
-	nsresult result;
-
-	nsCOMPtr<nsIDocShell> DocShell;
-	result = GetDocShell (getter_AddRefs(DocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
-
-	return DocShell->SetViewMode (mode);
-}
-
-#endif
-
 nsresult GaleonWrapper::SetZoom (float aZoom)
 {
 	nsresult result;
@@ -364,7 +336,7 @@ nsresult GaleonWrapper::GetDOMDocument (nsIDOMDocument **aDOMDocument)
 	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
 	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
 
-	return DOMWindow->GetDocument (getter_AddRefs(aDOMDocument));
+	return DOMWindow->GetDocument (aDOMDocument);
 }
 
 nsresult GaleonWrapper::GetMainDOMDocument (nsIDOMDocument **aDOMDocument)
@@ -379,19 +351,7 @@ nsresult GaleonWrapper::GetMainDOMDocument (nsIDOMDocument **aDOMDocument)
 	result = DocShell->GetContentViewer (getter_AddRefs(contentViewer));
 	if (!NS_SUCCEEDED (result) || !contentViewer) return NS_ERROR_FAILURE;
 
-	return contentViewer->GetDOMDocument (getter_AddRefs(aDOMDocument));
-}
-
-nsresult GaleonWrapper::SaveMainDocument (const char* filename)
-{
-	nsresult result;
-
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-
-	GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
-
-	return SaveDOMDocument (aDOMDocument, filename);
+	return contentViewer->GetDOMDocument (aDOMDocument);
 }
 
 nsresult GaleonWrapper::SaveURI (const char *url, const char *filename)
@@ -406,61 +366,26 @@ nsresult GaleonWrapper::SaveURI (const char *url, const char *filename)
 	return persist->SaveURI(linkURI, nsnull, filename);
 }
 
-nsresult GaleonWrapper::SaveDocument (const char* filename)
+nsresult GaleonWrapper::SaveLink (const char *url)
 {
-	nsresult result;
+	nsresult rv;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMWindow> DOMWindow;
+	rv = mWebBrowser->GetContentDOMWindow (getter_AddRefs(DOMWindow));
+	if (NS_FAILED(rv) || !DOMWindow) return NS_ERROR_FAILURE;
 
-	GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMWindowInternal> DOMWInternal =
+						do_QueryInterface (DOMWindow);
+	if (!DOMWInternal) return NS_ERROR_FAILURE;
 
-	return SaveDOMDocument (aDOMDocument, filename);
+	nsCOMPtr<nsIStreamTransfer> xfer = 
+				do_GetService (NS_ISTREAMTRANSFER_CONTRACTID);
+	if (!xfer) return NS_ERROR_FAILURE;
+
+	return xfer->SelectFileAndTransferLocationSpec (url, DOMWInternal, "",
+							  "", PR_TRUE, nsnull);
 }
 
-nsresult GaleonWrapper::SaveDOMDocument (nsIDOMDocument *aDOMDocument, const char* filename)
-{
-	nsresult result;
-	nsCOMPtr<nsIWebBrowserPersist> persist(do_GetInterface(mWebBrowser));
-	return persist->SaveDocument (aDOMDocument, filename, nsnull);
-}
-
-#if MOZILLA_MILESTONE != 81
-
-nsresult GaleonWrapper::SaveURLFromCache (const char *url, 
-		const char *dir, const char *filename)
-{
-	nsresult result;
-
-	nsCOMPtr<nsINetDataCacheManager> ndcm = 
-		do_GetService(NS_NETWORK_CACHE_MANAGER_CONTRACTID, &result);
-
-	nsCOMPtr<nsICachedNetData> cnd;
-	result = ndcm->GetCachedNetData (url, nsnull, 0, 0, getter_AddRefs(cnd));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIChannel> chan;
-	/* FIXME: dunno if this is needed */
-	nsCOMPtr<nsILoadGroup> buhLoadGroup = null_nsCOMPtr();
-	result = cnd->NewChannel(buhLoadGroup, getter_AddRefs(chan));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIFile> cfile;
-	result = chan->GetLocalFile(getter_AddRefs(cfile));
-	if (NS_FAILED(result) || !cfile) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsILocalFile> localdir = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-	result = localdir->InitWithPath(dir);
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	result = cfile->CopyTo(localdir, filename);
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	return NS_OK;
-}
-
-#endif
-
 nsresult GaleonWrapper::GetSHInfo (PRInt32 *count, PRInt32 *index)
 {
 	nsresult result;
@@ -471,6 +396,8 @@ nsresult GaleonWrapper::GetSHInfo (PRInt32 *count, PRInt32 *index)
 
 	SessionHistory->GetCount (count);
 	SessionHistory->GetIndex (index);	
+
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
@@ -481,29 +408,26 @@ nsresult GaleonWrapper::GetSHTitleAtIndex (PRInt32 index, PRUnichar **title)
 	result = GetSHistory (getter_AddRefs(SessionHistory));
 	if (NS_FAILED(result) || ! SessionHistory) return NS_ERROR_FAILURE;
 
-#ifdef CVS_SOURCE
 	nsCOMPtr<nsIHistoryEntry> he;
-#else
-	nsCOMPtr<nsISHEntry> he;
-#endif
 	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
 						  getter_AddRefs (he));
 	if (!NS_SUCCEEDED(result) || (!he)) return NS_ERROR_FAILURE;
 
 	result = he->GetTitle (title);
 	if (!NS_SUCCEEDED(result) || (!title)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
 }
 
-nsresult GaleonWrapper::Find (const char *exp, PRBool matchcase, 
-			      PRBool search_backwards, PRBool* didFind)
+nsresult GaleonWrapper::Find (const PRUnichar *search_string, 
+			      PRBool matchcase, PRBool search_backwards,
+			      PRBool *didFind)
 {
-	nsCOMPtr<nsIWebBrowserFind> finder(do_GetInterface(mWebBrowser));
+	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
 
-	nsString searchString;
-	searchString.AssignWithConversion(exp);  
-	finder->SetSearchString(searchString.GetUnicode());
-	finder->SetFindBackwards(search_backwards);
-	finder->SetMatchCase(matchcase);
+	finder->SetSearchString (search_string);
+	finder->SetFindBackwards (search_backwards);
+	finder->SetMatchCase (matchcase);
 	return finder->FindNext(didFind);
 }
 
@@ -540,6 +464,26 @@ nsresult GaleonWrapper::ReloadDocument ()
 	result = wn->Reload (nsIWebNavigation::LOAD_FLAGS_BYPASS_CACHE | 
 			     nsIWebNavigation::LOAD_FLAGS_BYPASS_PROXY);
 	if (!NS_SUCCEEDED (result)) return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
+nsresult GaleonWrapper::GetMainDocumentUrl (char **url)
+{
+	nsresult result;
+
+	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+
+	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+
+	nsIURI *uri = doc->GetDocumentURL();
+	uri->GetSpec (url);
+
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::GetDocumentUrl (char **url)
@@ -548,7 +492,7 @@ nsresult GaleonWrapper::GetDocumentUrl (char **url)
 
 	nsCOMPtr<nsIDOMDocument> aDOMDocument;
 
-	GetDOMDocument (getter_AddRefs(aDOMDocument));
+	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
 	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
@@ -556,6 +500,8 @@ nsresult GaleonWrapper::GetDocumentUrl (char **url)
 
 	nsIURI *uri = doc->GetDocumentURL();
 	uri->GetSpec (url);
+
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::GetKeyEventContext (nsIDOMKeyEvent *keyEvent, WrapperKeyEventInfo *info)
@@ -567,29 +513,45 @@ nsresult GaleonWrapper::GetKeyEventContext (nsIDOMKeyEvent *keyEvent, WrapperKey
 	nsCOMPtr<nsIDOMEventTarget>  targetNode;
 	res = keyEvent->GetTarget(getter_AddRefs(targetNode));
 	if (NS_FAILED(res) || !targetNode) return NS_ERROR_FAILURE;
-	if (targetNode)
+	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(targetNode);
+	/* don't grab alt combos, thus you can still access the menus and
+	 *  other alt functions.  */
+	keyEvent->GetAltKey (&mod_key);
+	if (node && !mod_key)
 	{
-		nsCOMPtr<nsIDOMNode> node = do_QueryInterface(targetNode);
-		if (node)
-		{
-	
-			nsCOMPtr<nsIDOMHTMLElement> element;
-	
-			element = do_QueryInterface(node);
-			if (element)
-			{
-				nsAutoString tag;
-				element->GetTagName(tag);
+		nsCOMPtr<nsIDOMHTMLElement> element;
 
-				if (!tag.EqualsWithConversion("a", PR_TRUE) &&
-				    !tag.EqualsWithConversion ("html",PR_TRUE))
+		element = do_QueryInterface(node);
+		if (element)
+		{
+			nsAutoString tag;
+			element->GetTagName(tag);
+
+			nsCOMPtr<nsIDOMHTMLInputElement> inputelement;
+
+			inputelement = do_QueryInterface(element);
+			if (inputelement) {
+				nsAutoString type;
+				inputelement->GetType(type);
+				if (type.EqualsWithConversion 
+						("text", PR_TRUE) ||
+				    type.EqualsWithConversion 
+						("password", PR_TRUE) ||
+				    type.EqualsWithConversion 
+						("file", PR_TRUE))
 				{
 					return NS_ERROR_FAILURE;
 				}
 			}
+			else if (!tag.EqualsWithConversion ("a", PR_TRUE) &&
+				 !tag.EqualsWithConversion ("html", PR_TRUE))
+			{
+				return NS_ERROR_FAILURE;
+			}
+
 		}
 	}
-	
+
 	/* Get the context */
 
 	res = GetEventContext (targetNode, &info->ctx);
@@ -618,9 +580,11 @@ nsresult GaleonWrapper::GetKeyEventContext (nsIDOMKeyEvent *keyEvent, WrapperKey
 	
 	keyEvent->GetCtrlKey(&mod_key);
 	if (mod_key) info->modifier |= CTRL_KEY;
+
+	return NS_OK;
 }
 
-nsresult GaleonWrapper::GetDOMAttribute (nsIDOMNode *node, char *tag,
+nsresult GaleonWrapper::GetDOMAttribute (nsIDOMNode *node, const char *tag,
 					 char **attribute)
 {
 	nsresult result;
@@ -646,6 +610,8 @@ nsresult GaleonWrapper::GetDOMAttribute (nsIDOMNode *node, char *tag,
 	*attribute = g_strdup (cstr);
 			
 	nsMemory::Free (cstr);
+
+	return NS_OK;
 }
 
 nsresult GaleonWrapper::GetEventContext (nsCOMPtr<nsIDOMEventTarget> EventTarget, WrapperContextInfo *info)
@@ -695,12 +661,12 @@ nsresult GaleonWrapper::GetEventContext (nsCOMPtr<nsIDOMEventTarget> EventTarget
 				info->context |= CONTEXT_IMAGE;
 
 				char *src;
-				result = GetDOMAttribute (node, g_strdup("src"),
-							  &src);
+				result = GetDOMAttribute (node, "src", &src);
 				if (NS_FAILED(result) || !src)
 					return NS_ERROR_FAILURE;
 			
 				result = baseURI->Resolve(src, &info->img);
+				g_free(src);
 				if (NS_FAILED(result) || !info->img)
 				return NS_ERROR_FAILURE;
 
@@ -722,14 +688,34 @@ nsresult GaleonWrapper::GetEventContext (nsCOMPtr<nsIDOMEventTarget> EventTarget
 				if (hrefNode)
 				{
 					info->context |= CONTEXT_LINK;
+					nsAutoString linkhtml;
+#if MOZILLA_VERSION > VERSION2(0,9)
+					nsCOMPtr<nsIDOMNSHTMLElement> nsElement;
 
+					nsElement = do_QueryInterface (element);
+					if (nsElement)
+					{
+
+						result = nsElement->GetInnerHTML (linkhtml);
+#else
+						result = element->GetInnerHTML (linkhtml);
+#endif
+						if (NS_SUCCEEDED(result)){
+							info->linktext =
+								mozilla_unicode_to_locale
+									(linkhtml.GetUnicode());
+						}
+#if MOZILLA_VERSION > VERSION2(0,9)
+					}
+#endif
 					char *href;
 					result = GetDOMAttribute (node,
-							g_strdup("href"), &href);
+							"href", &href);
 					if (NS_FAILED(result) || !href)
 						return NS_ERROR_FAILURE;
 				
 					result = baseURI->Resolve(href, &info->link);
+					g_free(href);
 					if (NS_FAILED(result) || !info->link)
 						return NS_ERROR_FAILURE;
 					break;
@@ -837,21 +823,16 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 	nsCOMPtr<nsISHistory> h_dest;
 	result = wn_dest->GetSessionHistory (getter_AddRefs (h_dest));
 	if (!NS_SUCCEEDED (result) || (!h_dest)) return NS_ERROR_FAILURE;
-#ifdef CVS_SOURCE
+
 	nsCOMPtr<nsISHistoryInternal> hi_dest = do_QueryInterface (h_dest);
 	if (!hi_dest) return NS_ERROR_FAILURE;
-#endif
 
 	h_src->GetCount (&count);
 	h_src->GetIndex (&index);
 
 	if (count) {
-#ifdef CVS_SOURCE
 		nsCOMPtr<nsIHistoryEntry> he;
 		nsCOMPtr<nsISHEntry> she;
-#else		
-		nsCOMPtr<nsISHEntry> he;
-#endif
 
 		for (PRInt32 i = 0; i < count; i++) {
 
@@ -859,22 +840,20 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 							 getter_AddRefs (he));
 			if (!NS_SUCCEEDED(result) || (!he))
 				return NS_ERROR_FAILURE;
-#ifdef CVS_SOURCE
+
 			she = do_QueryInterface (he);
 			if (!she) return NS_ERROR_FAILURE;
 
 			result = hi_dest->AddEntry (she, PR_TRUE);
 			if (!NS_SUCCEEDED(result) || (!she))
-#else
-			result = h_dest->AddEntry (he, PR_TRUE);
-			if (!NS_SUCCEEDED(result) || (!he))
-#endif
 				return NS_ERROR_FAILURE;
 		}
 
 		result = wn_dest->GotoIndex(index);
 		if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
 	}
+
+	return NS_OK;
 }
 
 
@@ -882,7 +861,6 @@ nsresult GaleonWrapper::SetSitePermission (gboolean block, PRInt32 type)
 {
 	nsresult result;
 
-#ifdef CVS_SOURCE
 	nsCOMPtr<nsIDOMDocument> DOMDocument;
 	result = GetDOMDocument (getter_AddRefs(DOMDocument));
 	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
@@ -898,23 +876,7 @@ nsresult GaleonWrapper::SetSitePermission (gboolean block, PRInt32 type)
 		permissionManager = do_CreateInstance (PERMISSIONMANAGER_ID);
 	
 	result = permissionManager->Add (url,block ? PR_TRUE : PR_FALSE, type);
-#else
-	nsCOMPtr<nsIWebBrowserFocus> focus (do_GetInterface(mWebBrowser));
 
-	nsCOMPtr<nsIDOMWindow> DOMWindow;
-	result = focus->GetFocusedWindow (getter_AddRefs(DOMWindow));
-	if (NS_FAILED(result) || !DOMWindow) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIDOMWindowInternal> 
-			DOMWindowInternal = do_QueryInterface (DOMWindow);
-	if (!DOMWindowInternal) return NS_ERROR_FAILURE;	
-
-	nsCOMPtr<nsICookieViewer> 
-			cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
-	result = cookieviewer->AddPermission (DOMWindowInternal,
-					      block ? PR_TRUE : PR_FALSE, 
-					      type);
-#endif
 	return result;
 }
 
@@ -965,6 +927,8 @@ nsresult GaleonWrapper::IsPageFramed (PRBool *Framed)
 	contentNode->GetChildCount (&childCount);
 
 	*Framed = (childCount>0);
+
+	return NS_OK;
 }
 
 GaleonEventListener::GaleonEventListener(void)
@@ -1049,6 +1013,7 @@ NS_IMETHODIMP GaleonEventListener::DragGesture(nsIDOMEvent* aMouseEvent)
 				1, NULL);
 	}
 
+	mozilla_free_context_info_sub (&info->ctx);
 	g_free (info);
 
 	return NS_OK;
@@ -1066,7 +1031,7 @@ nsresult GaleonWrapper::GetFaviconURL (char **favicon_url)
 	char *relstr;
 
 	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-	GetMainDOMDocument (getter_AddRefs(aDOMDocument));
+	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
 	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
 
 	nsString strname;
@@ -1106,8 +1071,6 @@ nsresult GaleonWrapper::GetFaviconURL (char **favicon_url)
 				return NS_OK;
 			}
 			
-			gchar *url = NULL;
-			
 			nsCOMPtr<nsIDocument> doc = 
 				do_QueryInterface (aDOMDocument);
 			if(!doc) return NS_ERROR_FAILURE;
