@@ -20,11 +20,21 @@
  * This file contains all the global types used within Galeon.
  */
 
+#ifndef __galeon_types_h
+#define __galeon_types_h
+
+#include <gtk/gtkwidget.h>
+
+/* FIXME Remove these */
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkctree.h>
+
 /* forward declarations of types */
 typedef struct _GaleonWindow GaleonWindow;
 typedef struct _GaleonEmbed  GaleonEmbed;
 typedef struct _BookmarkItem BookmarkItem;
-typedef struct _FindDialog   FindDialog; /* FIXME: remove */
+typedef struct _HistoryHost  HistoryHost;
+typedef struct _HistoryItem  HistoryItem;
 
 /**
  * GaleonEmbed: this is the type of a single web view (e.g. a tab):
@@ -39,7 +49,11 @@ struct _GaleonEmbed
 	GaleonWindow *parent_window;
 
 	/* embedding widget */
-	GtkMozEmbed *mozEmbed;
+	GtkWidget *mozEmbed;
+
+	/* tab widgets */
+	GtkWidget *notebook_close_button;
+	GtkWidget *notebook_close_pixmap;
 
 	/* C++ helper, a "GaleonWrapper" */
         gpointer wrapper;
@@ -54,6 +68,15 @@ struct _GaleonEmbed
 	gint maxBytesLoaded;
 	const gchar *statusMessage;
 
+	/* time when the load was actually started */
+	GTime when_started;
+
+	/* currently set zoom level */
+	gint zoom;
+
+	/* true if the zoom was set automatically */
+	gboolean zoom_auto_set : 1;
+
 	/* true if this is the "active_embed" of the parent window */
 	gboolean is_active : 1;
 
@@ -68,9 +91,6 @@ struct _GaleonEmbed
 
 	/* ignore mime processing of next url */
 	gboolean mime_ignore_once : 1;
-
-	/* FIXME: refactor */
-	FindDialog *find_dialog;
 };
 
 /**
@@ -104,8 +124,13 @@ struct _GaleonWindow
 
 	/* GUI components */
 	GtkWidget *notebook;
+	GtkWidget *hpaned;
+	GtkWidget *hbox;
+	GtkWidget *docked;                /* currently docked control      */
+	const gchar *docked_width_pref;   /* where width of dock is stored */
 
 	/* Toolbar components */
+	GtkWidget *main_toolbar;
 	GtkWidget *spinner; /* the GnomeAnimator spinner */
 	GtkWidget *location_dockitem;
 	GtkWidget *main_dockitem;
@@ -117,6 +142,7 @@ struct _GaleonWindow
 	GtkWidget *BStop;
 	GtkWidget *BStart;
 	GtkWidget *zoom_spin;
+	GtkWidget *drag_event_box;
 	GtkWidget *toolbar_gnomeentry;
 	GtkWidget *toolbar_entry;
 	GtkWidget *go_toolbar;
@@ -142,10 +168,10 @@ struct _GaleonWindow
 	GtkWidget *enable_java;
 	GtkWidget *enable_javascript;
 	GtkWidget *enable_proxy;
-	GtkWidget *use_galeon_mime_handling;
 
-	/* dialogs */
+	/* dialogs and views */
 	GtkWidget *print_dialog;
+	GtkWidget *find_dialog;
 
 	/* list of toolbars */
 	GList *bookmarks_toolbars; 
@@ -153,6 +179,15 @@ struct _GaleonWindow
 
 	/* saved window geometry for fullscreen windows */
 	gint x, y, width, height;
+
+	/* visibility */
+	gboolean menuBarOn;
+	gboolean toolBarOn;
+	gboolean locationBarOn;
+	gboolean statusBarOn;
+
+	/* saved layout */
+	gchar *layout;
 };
 
 /**
@@ -186,6 +221,16 @@ struct _GaleonWindow
   g_return_val_if_fail (embed != NULL, val); \
   g_return_val_if_fail (embed->magic == GALEON_EMBED_MAGIC, val);
 
+/**
+ * WidgetLookup: a mapping from strings to pointers, used to lookup
+ * a group of widgets from a Glade description.
+ */
+typedef struct 
+{ 
+	const gchar *name; 
+	GtkWidget **ptr;
+} WidgetLookup;
+
 /**
  * KeyCode: key pressed char codes, taken from mozilla
  */
@@ -338,20 +383,6 @@ typedef enum
 	DOM_VK_KP_LEFT = 65430   /* added */
 } KeyCode;
 
-/**
- * FindDialog: structure representing a "find in document" dialog
- */
-struct _FindDialog
-{
-	GtkWidget *dialog;
-	GtkWidget *entry;
-	GtkWidget *gnome_entry;
-	GtkWidget *case_checkbutton;
-	GtkWidget *prev_button;
-	GtkWidget *next_button;
-	gboolean string_changed;
-};
-
 /**
  * KeyModifier: key modifiers
  */
@@ -398,12 +429,25 @@ typedef enum
 typedef enum
 {
 	MIME_UNKNOWN,
-	MIME_LEAVE_TO_MOZILLA,
+	MIME_SAVE_WITH_GTM,
 	MIME_SAVE_TO_DISK,
 	MIME_RUN_PROGRAM,      /* should run the GNOME association */
-	MIME_ASK_ACTION
+	MIME_ASK_ACTION,
+	MIME_OVERRIDE_MOZILLA,
+	MIME_IGNORE
 } MimeAction;
 
+/**
+ * FilePickerMode: What mode FilePicker should run in
+ */
+
+typedef enum
+{
+	modeOpen = 0,
+	modeSave = 1,
+	modeGetFolder =2
+} FilePickerMode;
+
 /**
  * AutoResume: how to handling resuming crashed sessions
  */
@@ -492,16 +536,194 @@ typedef struct
 } MimeItem;
 
 /**
- * HistoryItem: an item in the history
+ * PreferencesDialog: structure representing Preferences dialog box
  */
 typedef struct
 {
+	GtkWidget *dialog;
+	GtkWidget *notebook;
+	MimeItem *selected_mime;
+	gboolean enable_edit_mime;
+	GtkWidget *hpaned;
+	GtkWidget *menu_ctree;
+	GtkWidget *themes_page;
+	GtkWidget *iconlist;
+	GtkWidget *smart_bm_example_entry;
+	GtkWidget *mime_list;
+	GtkWidget *mime_action_optionmenu;
+	GtkWidget *mime_name_entry;
+	GtkWidget *scrolledwindow9;/*FIXME*/
+	GtkWidget *lang_entry;/*FIXME*/
+	GtkWidget *autodownload_checkbutton;
+	GtkWidget *ask_download_dir_checkbutton;
+	GtkWidget *location_optionmenu;
+	GtkWidget *toolbar_optionmenu;
+	GtkWidget *startpage_entry;
+	GtkWidget *startpage_optionmenu;
+	GtkWidget *new_optionmenu;
+	GtkWidget *bookmarks_optionmenu;
+	GtkWidget *bm_extra_items_optionmenu;
+	GtkWidget *favicons_enabled_check;
+	GtkWidget *autobookmarks_enabled_check;
+	GtkWidget *autobookmarks_count_spin;
+	GtkWidget *autobookmarks_shorten_spin;
+	GtkWidget *smart_bm_entry_width_spin;
+	GtkWidget *java_checkbutton;
+	GtkWidget *javascript_checkbutton;
+	GtkWidget *download_dir_entry;
+	GtkWidget *tabbed_enable_check;
+	GtkWidget *tabbed_popups_enable_check;
+	GtkWidget *tabbed_autojump_check;
+	GtkWidget *tabbed_insert_new_tabs_check;
+	GtkWidget *tabbed_always_show_check;
+	GtkWidget *tabbed_closebutton_check;
+	GtkWidget *tabbed_shorten_spin;
+	GtkWidget *tabbed_shorten_spin2;
+	GtkWidget *tabbed_position_optionmenu;
+	GtkWidget *tabbed_style_radiobutton1;
+	GtkWidget *tabbed_separator_check;
+	GtkWidget *tabbed_vowel_check;
+	GtkWidget *tabbed_prefix_check;
+	GtkWidget *autocompletion_enabled_checkbutton;
+	GtkWidget *completion_sort_optionmenu;
+	GtkWidget *show_zoom_check;
+	GtkWidget *show_spinner_check;
+        GtkWidget *proxy_use_checkbutton;
+	GtkWidget *http_proxy_entry;
+	GtkWidget *ssl_proxy_entry;
+	GtkWidget *ftp_proxy_entry;
+	GtkWidget *http_proxy_port_spin;
+	GtkWidget *ssl_proxy_port_spin;
+	GtkWidget *ftp_proxy_port_spin;
+	GtkWidget *no_proxies_for_entry;
+	GtkWidget *disk_cache_entry;
+	GtkWidget *mem_cache_entry;
+	GtkWidget *lang_encode;
+	GtkWidget *serif_font_fpicker;
+	GtkWidget *sansserif_font_fpicker;
+	GtkWidget *monospace_font_fpicker;
+	GtkWidget *size_font_spin;
+	GtkWidget *default_font_optionmenu;
+	GtkWidget *history_expire_spin;
+	GtkWidget *use_own_fonts_checkbutton;
+	GtkWidget *image_optionmenu;
+	GtkWidget *ftp_optionmenu;
+	GtkWidget *ftp_program_optionmenu;
+	GtkWidget *ftp_command_entry;
+	GtkWidget *show_go_button_check;
+	GtkWidget *show_up_button_check;
+	GtkWidget *show_new_button_check;
+	GtkWidget *show_home_button_check;
+	GtkWidget *show_drag_event_box_check;
+	GtkWidget *show_toolbars_in_fullscreen_check;
+	GtkWidget *show_menubar_in_fullscreen_check;
+	GtkWidget *show_statusbar_in_fullscreen_check;
+	GtkWidget *use_external_source_viewer_check;
+	GtkWidget *external_source_viewer_entry;
+	GtkWidget *crash_optionmenu;
+	GtkWidget *allow_popups_checkbutton;
+	GtkWidget *statusbar_rewrite_check;
+	GtkWidget *windows_title_entry;
+	GtkWidget *fullscreen_stacking_checkbutton;
+	GtkWidget *myportal_stylesheet_entry;
+	GtkWidget *spinner_dir_entry;
+	GtkWidget *autodetect_charset_optionmenu;
+	GtkWidget *default_charset_combo;
+	GtkWidget *default_charset_entry;
+	GtkWidget *lang_list;
+	GtkWidget *background_color;
+	GtkWidget *text_color;
+	GtkWidget *unvisited_link_color;
+	GtkWidget *visited_link_color;
+	GtkWidget *use_own_colors_checkbutton;
+	GtkWidget *underline_links_checkbutton;
+	GtkWidget *http_version_optionmenu;
+	GtkWidget *proxy_keepalive_checkbutton;
+	GtkWidget *history_popup_checkbutton;
+	GtkWidget *middle_button_optionmenu;
+	GtkWidget *right_button_optionmenu;
+	GtkWidget *cookie_behavior_optionmenu;
+	GtkWidget *cookie_manager_button;
+	GtkWidget *warn_cookie_checkbutton;
+	GtkWidget *passwords_save_checkbutton;
+	GtkWidget *wheel_nokey_action_optionmenu;
+	GtkWidget *wheel_nokey_step_spinbutton;
+	GtkWidget *wheel_nokey_default_step_checkbutton;
+	GtkWidget *wheel_alt_action_optionmenu;
+	GtkWidget *wheel_alt_step_spinbutton;
+	GtkWidget *wheel_alt_default_step_checkbutton;
+	GtkWidget *wheel_ctrl_action_optionmenu;
+	GtkWidget *wheel_ctrl_step_spinbutton;
+	GtkWidget *wheel_ctrl_default_step_checkbutton;
+	GtkWidget *wheel_shift_action_optionmenu;
+	GtkWidget *wheel_shift_step_spinbutton;
+	GtkWidget *wheel_shift_default_step_checkbutton;
+	GtkWidget *crash_behaviour_optionmenu;
+	GtkWidget *links_drag_checkbutton;
+	GtkWidget *cache_compare_optionmenu;
+	GtkWidget *nautilus_icons_checkbutton;
+} PreferencesDialog;
+
+/**
+ * HistoryView: a history view (either dialog or dockeD)
+ */
+typedef struct
+{
+	GaleonWindow *window;            /* parent window */
+	GtkWidget *view;                 /* either the dock or the dialog */
+	GtkWidget *ctree;
+	GtkWidget *host_checkbutton;
+	GtkWidget *time_optionmenu;
+	GtkWidget *search_entry;
+	GtkWidget *dock_close_button;
+	GtkWidget *dock_detach_button;
+	gboolean group;
+	gchar *search;
+	gint time;
+	GtkWidget *sibling;              /* used in tree construction */
+	GHashTable *host_to_node;        /* maps HistoryHosts to tree nodes */
+	GHashTable *item_to_node;        /* maps HistoryItems to tree nodes */
+	HistoryItem *selected_item;
+} HistoryView;
+
+/**
+ * HistoryHost: a group of history elements assembled by host
+ */
+struct _HistoryHost
+{
+	/* NB: these must stay the same as HistoryItem */
+	gchar *url; /* NOTE: unused! */
 	gchar *title;
-	gchar *url;
 	GTime first;
 	GTime last;
 	gint visits;
-} HistoryItem;
+
+	/* tree specific data */
+	GtkCTreeNode *node;
+	GtkCTreeNode *sibling;
+
+	/* stored zoom for this host */
+	gint zoom;
+
+	/* dominant child -- the location visited the most of this server */
+	HistoryItem *dominant_item;
+};
+
+/**
+ * HistoryItem: an item in the history
+ */
+struct _HistoryItem
+{
+	/* NB: these must stay the same as HistoryItem */
+	gchar *url;
+	gchar *title;
+	GTime first;
+	GTime last;
+	gint visits;
+
+	/* parent host */
+	HistoryHost *host;
+};
 
 /**
  * PixmapData: a single pixmap (stored on the X server)
@@ -522,6 +744,11 @@ typedef enum
 	BM_AUTOBOOKMARKS
 } BookmarkType;
 
+/**
+ * Forward declaration
+ */
+typedef struct _BookmarkTreeItem BookmarkTreeItem;
+
 /**
  * BookmarkItem: a single entry in bookmarks
  */
@@ -551,8 +778,9 @@ struct _BookmarkItem
 	/* the list of children if this is a folder */
 	GList *list;
 
-	/* a handle to find the bookmark in the editor dialog */
-	GtkCTreeNode *tree_item; 
+	/* a handles to find the bookmark in the editor dialogs */
+	/* this is a list of BookmarkTreeItem */
+	GList *tree_items;
 
 	/* parent category (NULL for root bookmark) */
 	BookmarkItem *parent; 
@@ -584,17 +812,6 @@ struct _BookmarkItem
 
 };
 
-/**
- * BookmarkDragItem: the representation of a bookmark when it's passing
- * through drag-and-drop.
- */
-typedef struct
-{
-	GaleonWindow *window;
-	BookmarkItem *bookmark;
-	GtkToolbar   *toolbar;
-} BookmarkDragItem;
-
 /**
  * BookmarksEditorControls: representation of a bookmarks editor.
  */
@@ -605,7 +822,10 @@ typedef struct
 
 	/* the widgets of the editor */
 	GtkWidget *dialog;
+	GtkWidget *menubar;
 	GtkWidget *toolbar;
+	GtkWidget *menubar_dock;
+	GtkWidget *toolbar_dock;
 	GtkWidget *ctree;
 	GtkWidget *vpane;
 	GtkWidget *edit_frame;
@@ -621,16 +841,27 @@ typedef struct
 	GtkWidget *time_modified_entry;
 	GtkWidget *time_visited_entry;
 
-	/* FIXME: comment */
-	BookmarkItem *selection;
-	BookmarkItem *last_pressed; /* for dnd */
-	gboolean lock_edit_controls;
-	gboolean dirty;
-	void (*save_function) (void);
+	BookmarkItem *selection;    /* The current selected bookmark */
+	BookmarkItem *last_pressed; /* For dnd */
+	gboolean lock_edit_controls; /* Don't update bookmark when editing 
+				      * fields. Set to TRUE to avoid recursion
+				      */
+	
 	GaleonWindow *window;
 
+	/* the clipboard data owned by this editor, if any */
+	gchar *clipboard;
+
 } BookmarksEditorControls;
 
+/**
+ * Pair of controls / tree_item for a bookmark
+ */
+struct _BookmarkTreeItem {
+	BookmarksEditorControls *controls;
+	GtkCTreeNode *tree_item; 	
+};
+
 /**
  * BookmarksEditorControls_type types of bookmark editor dialogs
  */
@@ -657,9 +888,6 @@ enum BookmarksDnDTargetType
  */
 typedef struct
 {
-#ifndef CVS_SOURCE
-	gchar *number;
-#endif	
 	gchar *type;
 	gchar *domain;
 	gboolean remove;
@@ -682,30 +910,28 @@ typedef struct
  * Password: a password manager entry
  */
 
-#ifdef CVS_SOURCE
-typedef struct {
+typedef struct
+{
 	gchar *host;
 	gchar *username;
-	
 	gboolean remove;
 } Password;
 
 /**
  * PasswordType: To distinguish actual passwords from blocked password sites
  */
-  
-typedef enum {
-	PASSWORD,
+typedef enum
+{
+	PASSWORD, /* FIXME: prefix  */
 	REJECT
 } PasswordType;
-#endif
 
 /**
  * CookiePage: For selecting which page in the Cookie manager is active
  */
-
-typedef enum {
-	COOKIES = 0,
+typedef enum
+{
+	COOKIES = 0, /* FIXME: prefix */
 	COOKIESITES = 1,
 	IMAGESITES = 2,
 	PASSWORDS = 3,
@@ -713,18 +939,19 @@ typedef enum {
 } CookiePage;
 
 /**
- * MimeAskActionDialog: the representation of dialogs used to ask
- * about actions on MIME types
+ * WheelModKey: modifiers that can be used in conjuction with the mouse wheel
  */
-typedef struct
+typedef enum
 {
-	gchar *url;
-	gchar *mime_type;
-	GaleonEmbed *embed;
-	GladeXML *gxml;
-} MimeAskActionDialog;
+	WHEEL_MOD_NOKEY,
+	WHEEL_MOD_ALT,
+	WHEEL_MOD_CTRL,
+	WHEEL_MOD_SHIFT
+} WheelModKey;
 
 /**
  * FileCallbackFunc: the type of functions used in file callbacks
  */
 typedef void (*FileCallbackFunc) (const gchar *file, gpointer data);
+
+#endif /* __galeon_types_h */
