@@ -17,32 +17,28 @@
  */
 
 #include "galeon.h"
+#include "window.h"
+#include "prefs.h"
+#include "history.h"
+#include "embed.h"
 #include "menubar.h"
+#include "misc.h"
+#include "bookmarks.h"
+#include "print.h"
+#include "find.h"
+#include "mozilla.h"
+#include "xlink.h"
 
-/**
- * window_menu_select_dir_activate_cb: show the download directory dialog (menu)
- */
-void
-window_menu_select_dir_activate_cb (GtkMenuItem *menuitem,
-				    GaleonWindow *window)
-{
-	GtkWidget *dialog;
-	char *olddir;
-	GladeXML *gxml;
+#include <string.h>
+#include <gtkmozembed.h>
+#include <gtk/gtkspinbutton.h>
+#include <libgnomeui/gnome-about.h>
+#include <libgnome/gnome-help.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
 
-	return_if_not_window (window);
-
-	gxml = glade_widget_new ("dirselection", &dialog, window);
-
-	olddir = gnome_config_get_string("/galeon/Downloading/download_dir");
-
-	gtk_file_selection_set_filename(GTK_FILE_SELECTION (dialog), olddir);
-  
-	gtk_widget_set_sensitive(GTK_WIDGET(GTK_FILE_SELECTION(
-						dialog)->file_list), FALSE);
-	gtk_widget_show(dialog);
-	window_set_layer(dialog);
-}
+void 
+window_menu_recent_activate_cb (GtkMenuItem *menuitem, gpointer user_data);
 
 /**
  * window_menu_prefs_activate_cb: show preferences dialog
@@ -72,7 +68,219 @@ void
 window_menu_find_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
-	embed_show_find_dialog (window->active_embed);
+	find_show_dialog (window);
+}
+
+/*
+ * Find which widget is currently in focus
+ */
+static GtkWidget *
+window_menu_edit_find_focus(GaleonWindow *window)
+{
+	GtkWidget *widget;
+	GList *tb_entry;
+
+	if (GTK_WIDGET_HAS_FOCUS ( window -> toolbar_entry ) )
+		return(window -> toolbar_entry);
+
+	/* Search through the toolbars, then down each toolbar
+	** looking for a focused widget
+	*/
+
+	for (widget=NULL, tb_entry = g_list_first(window->bookmarks_toolbars);
+		widget==NULL && tb_entry!=NULL;
+		tb_entry = g_list_next(tb_entry)) {
+
+		GtkToolbar *toolbar=(GtkToolbar *) tb_entry->data;
+		GList *tbc_entry;
+
+		/* Step through the toolbar children */
+		for (tbc_entry = g_list_first(toolbar->children);
+			widget==NULL && tbc_entry!=NULL;
+			tbc_entry = g_list_next(tbc_entry)) {
+
+			GtkToolbarChild *toolbarchild =
+				(GtkToolbarChild *) tbc_entry->data;
+
+			if (toolbarchild->type == GTK_TOOLBAR_CHILD_WIDGET &&
+				GTK_WIDGET_HAS_FOCUS (toolbarchild->widget)) {
+
+				widget=toolbarchild->widget;
+			}
+		}
+	}
+
+	if (widget)
+		return(widget);
+
+	/* Add getting the embed Mozilla widget here */
+
+	return(NULL);
+}
+
+/*
+ * Called before the Cut menu item is exposed. We allow it to be sensitive
+ * if we are currently in an updateable field and there is something selected
+ */
+static gboolean
+window_menu_edit_cut_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+	GaleonWindow *window=data;
+
+	if (event->type == GDK_EXPOSE) {
+		gboolean sensitive=FALSE;
+		GtkWidget *focus_widget;
+
+		focus_widget=window_menu_edit_find_focus(window);
+		if (GTK_IS_EDITABLE(focus_widget)) {
+			GtkEditable *editable = GTK_EDITABLE(focus_widget);
+
+			if (editable->has_selection && editable->editable) {
+				sensitive=TRUE;
+			}
+		}
+		gtk_widget_set_sensitive(widget, sensitive);
+	}
+
+	return(FALSE);
+}
+
+/*
+ * Called before the Copy menu item is exposed. We allow it to be sensitive
+ * if there is something selected
+ */
+static gboolean
+window_menu_edit_copy_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+	GaleonWindow *window=data;
+
+	if (event->type == GDK_EXPOSE) {
+		gboolean sensitive=FALSE;
+		GtkWidget *focus_widget;
+
+		focus_widget=window_menu_edit_find_focus(window);
+		if (GTK_IS_EDITABLE(focus_widget)) {
+			GtkEditable *editable = GTK_EDITABLE(focus_widget);
+
+			if (editable->has_selection) {
+				sensitive=TRUE;
+			}
+		}
+		gtk_widget_set_sensitive(widget, sensitive);
+	}
+
+	return(FALSE);
+}
+
+/*
+ * Called before the Paste menu item is exposed. We allow it to be sensitive
+ * if the current field is updateable and there is something in the clipboard.
+ * Note the call to galeon_x_clipboard_present().
+ */
+static gboolean
+window_menu_edit_paste_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+	GaleonWindow *window=data;
+
+	if (event->type == GDK_EXPOSE) {
+		gboolean sensitive=FALSE;
+		GtkWidget *focus_widget;
+
+		focus_widget=window_menu_edit_find_focus(window);
+		if (GTK_IS_EDITABLE(focus_widget)) {
+			GtkEditable *editable = GTK_EDITABLE(focus_widget);
+			if (editable->editable && galeon_x_clipboard_present()) {
+				sensitive=TRUE;
+			}
+		}
+		gtk_widget_set_sensitive(widget, sensitive);
+	}
+
+	return(FALSE);
+}
+
+/*
+ * Add event handlers for the expose event onto the cut/copy/paste items
+ * on the edit menu
+ */
+void
+window_menu_edit_add_events(GaleonWindow *window)
+{
+	GnomeUIInfo *editmenu = edit_menu_uiinfo;
+	gint count;
+
+	for (count=0; editmenu[count].type != GNOME_APP_UI_ENDOFINFO; count++) {
+		gpointer cb = editmenu[count].moreinfo;
+
+		if (cb == window_menu_edit_cut_cb) {
+			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
+				"expose_event",
+				GTK_SIGNAL_FUNC (window_menu_edit_cut_event),
+				window);
+		} else if (cb == window_menu_edit_copy_cb) {
+			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
+				"expose_event",
+				GTK_SIGNAL_FUNC (window_menu_edit_copy_event),
+				window);
+		} else if (cb == window_menu_edit_paste_cb) {	
+			gtk_signal_connect ( GTK_OBJECT (editmenu[count].widget),
+				"expose_event",
+				GTK_SIGNAL_FUNC (window_menu_edit_paste_event),
+				window);
+		}
+	}
+
+}
+
+/**
+ * window_menu_edit_cut_cb: Cut selected text into clipboard
+ */
+void
+window_menu_edit_cut_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GtkEditable *editable;
+	GtkWidget *widget;
+
+	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+		return;
+
+	editable = GTK_EDITABLE (widget);
+
+	gtk_editable_cut_clipboard (editable);
+}
+
+/**
+ * window_menu_edit_copy_cb: Copy selected text into clipboard
+ */
+void
+window_menu_edit_copy_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GtkEditable *editable;
+	GtkWidget *widget;
+
+	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+		return;
+
+	editable = GTK_EDITABLE (widget);
+
+	gtk_editable_copy_clipboard (editable);
+}
+
+/**
+ * window_menu_edit_paste_cb: Paste selected text into clipboard
+ */
+void
+window_menu_edit_paste_cb (GtkMenuItem *menuitem, GaleonWindow *window)
+{
+	GtkEditable *editable;
+	GtkWidget *widget;
+
+	if ((widget=window_menu_edit_find_focus(window)) == NULL)
+		return;
+
+	editable = GTK_EDITABLE (widget);
+
+	gtk_editable_paste_clipboard (editable);
 }
 
 /**
@@ -132,16 +340,25 @@ void window_menu_history_activate_cb (GtkMenuItem *menuitem,
 void
 window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
-	static const char *author[] = {
+	static gchar *authors[] = {
 		"Marco Pesenti Gritti <mpeseng@tin.it>",
 		"Remco de Jong <rdj@rdj.cg.nu>",
-		"Ricardo Fernandez Pascual <ric@users.sourceforge.net>",
+		/* if your encoding allows it, use aacute (U00E1)
+		 * for the 'a' of 'Fernandez' */
+		N_("Ricardo Fernandez Pascual <ric@users.sourceforge.net>"),
 		"Nate Case <nd@kracked.com>",
 		"Bob Phan <ex-freek@yahoo.com>",
 		"Matt Aubury <matt@ookypooky.com>",
 		"Joshua Horvath <jmhorvat@home.com>",
-		"Yanko Kaneti <yaneti@declera.com>",
-		"","Graphic arts :", "Toussaint Frederic (fred@linuxgraphic.org)",
+		/* this one is supposed to be in cyrillic */
+		N_("Yanko Kaneti <yaneti@declera.com>"),
+		"Philip Langdale <philipl@mail.utexas.edu>",
+		"Matthew Mueller <donut@azstarnet.com>",
+		" ", /* dont delete the space */
+		N_("Graphic arts: "),
+	       /* if your encoding allows it, use eacute (U00E9)
+		* for the two 'e's of Frederic */
+		N_("Toussaint Frederic (fred@linuxgraphic.org)"),
 		NULL
 	};
 
@@ -151,10 +368,37 @@ window_menu_about_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 
 	if (!about)
 	{
+		gint i, num_authors = sizeof(authors) / sizeof(gchar *);
+		gchar *translated_names[num_authors];
+		gchar *bottom_line;
+
+		if (strcmp (_("Translation:"), "Translation:") == 0) {
+			bottom_line = g_strdup("A GNOME browser based on mozilla");
+		} else {
+			bottom_line = g_strconcat (
+					_("A GNOME browser based on mozilla"),
+					"\n \n",
+			/* Translators should localize the following string
+			 * which will be displayed at the bottom of the about
+			 * box to give credit to the translator(s).
+			 * Translate the "Translation:", add your name and an
+			 * email address. You can span several lines with a
+			 * newline character if needed, but it shouldn't be
+			 * too long; vertical space is limited in the about
+			 * dialog. 
+			 * If you dont translate it nothing will be added */
+					_("Translation:"), NULL);
+		}
+		
+		for (i = 0; i < num_authors ; i++)
+			translated_names[i] = _(authors[i]);
+		
 		about = gnome_about_new (_(PACKAGE), VERSION,
 				 _("Copyright (C) 2000-2001 Marco Pesenti Gritti"),
-				 author, _("A GNOME browser based on mozilla"),
+				 (const gchar **)translated_names, 
+				 bottom_line,
 				 SHARE_DIR"/logo.png");
+		g_free(bottom_line);
 
 		gtk_signal_connect (GTK_OBJECT (about), "destroy",
 			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
@@ -285,7 +529,7 @@ window_menu_back_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_back (window->active_embed->mozEmbed);
+	gtk_moz_embed_go_back (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
@@ -307,7 +551,7 @@ window_menu_forward_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_forward(window->active_embed->mozEmbed);
+	gtk_moz_embed_go_forward(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
@@ -340,7 +584,7 @@ window_menu_stop_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_stop_load(window->active_embed->mozEmbed);
+	gtk_moz_embed_stop_load(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
@@ -355,35 +599,23 @@ window_menu_reload_activate_cb (GtkMenuItem *menuitem, GaleonWindow *window)
 }
 
 /** 
- * window_menu_start_page_activate_cb: start page menu activated
+ * window_menu_home_page_activate_cb: home page menu item activated
  */
 void
-window_menu_start_page_activate_cb (GtkMenuItem *menuitem,
+window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
 				    GaleonWindow *window)
 {
 	return_if_not_window (window);
-	/* FIXME: dont call this directly */
-	window_start_button_clicked_cb (GTK_BUTTON (window->BStart), window);
-}
-
-/** 
- * window_menu_show_temp_bookmarks_activate_cb: show temp bookmarks window
- */
-void
-window_menu_show_temp_bookmarks_activate_cb (GtkMenuItem *menuitem,
-					     GaleonWindow *window)
-{
-	return_if_not_window (window);
-	bookmarks_editor_show_dialog (window, COMPACT_BOOKMARKS_EDITOR);
+	window_go_home (window, FALSE, FALSE);
 }
 
 void
-window_menu_add_temp_bookmark_activate_cb (GtkMenuItem *menuitem,
+window_menu_add_bookmark_activate_cb (GtkMenuItem *menuitem,
 					   GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	embed_add_temporary_bookmark (window->active_embed);
+	embed_add_bookmark_default (window->active_embed);
 }
 
 /** 
@@ -450,7 +682,7 @@ void window_menu_bookmarks_dock_activate_cb (GtkCheckMenuItem *menuitem,
 	}
 	else
 	{
-		bookmarks_editor_hide_dock ();
+		window_undock (window);
 	}
 }
 
@@ -467,7 +699,7 @@ void window_menu_history_dock_activate_cb (GtkCheckMenuItem *menuitem,
 	}
 	else
 	{
-		history_hide_dock ();
+		window_undock (window);
 	}
 }
 
@@ -515,14 +747,6 @@ window_menu_own_fonts_activate_cb (GtkCheckMenuItem *menuitem,
 	preferences_set_use_own_fonts (menuitem->active);
 }
 
-void
-window_menu_galeon_mime_handling_activate_cb (GtkCheckMenuItem *menuitem,
-					      GaleonWindow *window)
-{
-	return_if_not_window (window);
-	preferences_set_handlers_enable_mime (menuitem->active);
-}
-
 void
 window_menu_enable_java_activate_cb (GtkCheckMenuItem *menuitem,
 				     GaleonWindow *window)
@@ -631,7 +855,7 @@ window_cookie_manager_launch_cb (GtkMenuItem *menuitem,
 				 GaleonWindow *window)
 {
 	return_if_not_window (window);
-	cookies_show_manager();
+	cookies_show_manager ();
 }
 
 void
@@ -663,6 +887,14 @@ window_menu_zoom_in_activate_cb (GtkCheckMenuItem *menuitem,
 			      GTK_SPIN_STEP_FORWARD, 10); 
 }
 
+void
+window_menu_zoom_100_activate_cb (GtkCheckMenuItem *menuitem,
+				 GaleonWindow *window)
+{
+	return_if_not_window (window);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (window->zoom_spin), 100);
+}
+
 void
 window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
 				  GaleonWindow *window)
@@ -673,7 +905,7 @@ window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
 }
 
 void 
-window_menu_encoding_activate_cb (GtkMenuItem *menuitem,
+window_menu_encoding_activate_cb (GtkWidget *menuitem,
 				  GaleonWindow *window)
 {
 	char *menulabel;
@@ -688,7 +920,6 @@ window_menu_encoding_activate_cb (GtkMenuItem *menuitem,
 	embed_reload (window->active_embed);
 }
 
-#ifdef CVS_SOURCE
 void
 window_view_passwords_activate_cb (GtkMenuItem *menuItem,
 			           GaleonWindow *window)
@@ -703,4 +934,12 @@ window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
 	return_if_not_window (window);
 	cookies_show_page (PASSWORDSITES);
 }
-#endif
+
+void window_menu_offline_mode_activate_cb (GtkCheckMenuItem *menuitem,
+					   GaleonWindow *window)
+{
+	return_if_not_window (window);
+	return_if_not_embed (window->active_embed);
+
+	mozilla_set_offline(window->active_embed, menuitem->active);
+}
