@@ -16,9 +16,15 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <gtk/gtkcheckmenuitem.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-app-helper.h>
+
 /* from menu_callbacks.c */
 void window_menu_zoom_in_activate_cb (GtkCheckMenuItem *menuitem,
                                       GaleonWindow *window);
+void window_menu_zoom_100_activate_cb (GtkCheckMenuItem *menuitem,
+				       GaleonWindow *window);
 void window_menu_zoom_out_activate_cb (GtkCheckMenuItem *menuitem,
                                        GaleonWindow *window);
 void window_menu_bookmarks_edit_activate_cb (GtkMenuItem *menuitem,
@@ -53,9 +59,7 @@ void window_menu_close_activate_cb (GtkMenuItem *menuitem,
                                     GaleonWindow *window);  
 void window_menu_view_source_mode_activate_cb (GtkCheckMenuItem *menuitem,
 					       GaleonWindow *window);
-void window_menu_show_temp_bookmarks_activate_cb (GtkMenuItem *menuitem,
-                                                  GaleonWindow *window);
-void window_menu_add_temp_bookmark_activate_cb (GtkMenuItem *menuitem,
+void window_menu_add_bookmark_activate_cb (GtkMenuItem *menuitem,
                                                 GaleonWindow *window);
 void window_menu_toolbar_activate_cb (GtkCheckMenuItem *menuitem,
                                       GaleonWindow *window);
@@ -107,12 +111,17 @@ void window_menu_stop_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
 void window_menu_reload_activate_cb (GtkMenuItem *menuitem,
                                      GaleonWindow *window);
-void window_menu_start_page_activate_cb (GtkMenuItem *menuitem,
-                                         GaleonWindow *window);   
+void window_menu_home_page_activate_cb (GtkMenuItem *menuitem,
+                                        GaleonWindow *window);   
 void window_menu_find_activate_cb (GtkMenuItem *menuitem,
                                    GaleonWindow *window);
-void window_menu_select_dir_activate_cb (GtkMenuItem *menuitem,
-					 GaleonWindow *window);
+void window_menu_edit_add_events (GaleonWindow *window);
+void window_menu_edit_cut_cb (GtkMenuItem *menuitem,
+                              GaleonWindow *window);
+void window_menu_edit_copy_cb (GtkMenuItem *menuitem,
+                               GaleonWindow *window);
+void window_menu_edit_paste_cb (GtkMenuItem *menuitem,
+                                GaleonWindow *window);
 void window_menu_prefs_activate_cb (GtkMenuItem *menuitem,
                                     GaleonWindow *window);
 void window_menu_view_source_activate_cb (GtkMenuItem *menuitem,
@@ -133,25 +142,12 @@ void window_block_cookies_from_activate_cb (GtkMenuItem *menuitem,
                                             GaleonWindow *window);
 void window_allow_cookies_from_activate_cb (GtkMenuItem *menuitem,
                                             GaleonWindow *window);
-void bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
-                                           GaleonWindow *window);
-void bookmarks_import_other_activate_cb (GtkMenuItem *menuitem,
-                                         GaleonWindow *window);
-void bookmarks_export_xbel_activate_cb (GtkMenuItem *menuitem,
-					GaleonWindow *window); 
-void bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem,
-                                            GaleonWindow *window);
-void bookmarks_export_mozilla_activate_cb (GtkMenuItem *menuitem,
-                                           GaleonWindow *window); 
-void bookmarks_export_other_activate_cb (GtkMenuItem *menuitem,  
-                                         GaleonWindow *window);
-#ifdef CVS_SOURCE
 void window_view_passwords_activate_cb (GtkMenuItem *menuItem,
                                         GaleonWindow *window);
 void window_view_password_sites_activate_cb (GtkMenuItem *menuItem,
-                                             GaleonWindow *window);                                           
-#endif
-
+                                             GaleonWindow *window);
+void window_menu_offline_mode_activate_cb (GtkCheckMenuItem *menuitem,
+					   GaleonWindow *window);
 /* from window_callbacks.c */
 gboolean window_delete_cb (GtkWidget *widget, GdkEventAny *event,
                            GaleonWindow *window);
@@ -165,11 +161,8 @@ extern GnomeUIInfo load_images_menu_uiinfo[];
 extern GnomeUIInfo animate_always_uiinfo[];
 extern GnomeUIInfo animate_images_menu_uiinfo[];
 extern GnomeUIInfo cookies_menu_uiinfo[];
-#ifdef CVS_SOURCE
 extern GnomeUIInfo passwords_menu_uiinfo[];
-#endif
 extern GnomeUIInfo settings_menu_uiinfo[];
-extern GnomeUIInfo import_menu_uiinfo[];
 extern GnomeUIInfo bookmarks_menu_uiinfo[];
 extern GnomeUIInfo go_menu_uiinfo[];
 extern GnomeUIInfo help_menu_uiinfo[];
