@@ -17,6 +17,10 @@
  */
 
 #include "galeon.h"
+#include "mime.h"
+#include "misc.h"
+
+#include <string.h>
 
 /* the global list of MimeItems */
 GList *mime_types = NULL;
@@ -36,39 +40,25 @@ mime_set_action (const gchar *mime_type, MimeAction action)
 {
 	GList *l;
 	MimeItem *newitem;
+	gboolean exists = FALSE;
+
 	/* change the value if we already have it in the list */
 	for (l = mime_types; l != NULL; l = l->next)
-		if (!strcmp (mime_type, ((MimeItem *) l->data)->name)) {
+		if (!strcmp (mime_type, ((MimeItem *) l->data)->name))
+		{
+			exists = TRUE;
 			((MimeItem *) l->data)->action = action;
-		}
+		};
 	/* not found, add a new mime type */
-	newitem = g_new0 (MimeItem, 1);
-	newitem->name = g_strdup (mime_type);
-	newitem->action = action;
-	mime_types = g_list_prepend (mime_types, newitem);
+	if (!exists)
+	{
+		newitem = g_new0 (MimeItem, 1);
+		newitem->name = g_strdup (mime_type);
+		newitem->action = action;
+		mime_types = g_list_prepend (mime_types, newitem);
+	};
 }
 	
-void
-mime_ask_action (const gchar *url, const gchar *mime_type, 
-		 GaleonEmbed *embed)
-{
-	MimeAskActionDialog *dialog = g_new0 (MimeAskActionDialog, 1);
-	GtkWidget *label;
-	gchar *msg;
-		
-	dialog->url = g_strdup (url);
-	dialog->mime_type = g_strdup (mime_type);
-	dialog->embed = embed;
-	dialog->gxml = glade_widget_new ("mime_ask_action_dialog", NULL,
-					 dialog);
-
-	label = glade_xml_get_widget (dialog->gxml, "mime_ask_action_message");
-	msg = g_strdup_printf (_("The url %s has MIME type %s.\n"
-				 "What do you want to do with it?"), url, mime_type);
-	gtk_label_set_text (GTK_LABEL (label), msg);
-	g_free (msg);
-}
-
 MimeItem *
 mime_item_clone (MimeItem *m)
 {
