@@ -17,6 +17,22 @@
  */
 
 #include "galeon.h"
+#include "mozcallbacks.h"
+#include "embed.h"
+#include "misc.h"
+#include "window.h"
+#include "history.h"
+#include "spinner.h"
+#include "session.h"
+#include "mozilla.h"
+#include "context.h"
+
+#include <time.h>
+#include <string.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-triggers.h>
+#include <libgnome/gnome-i18n.h>
+#include <gdk/gdkkeysyms.h>
 
 /* Flag to enable/disable mouse motion listener */
 static gboolean mouse_listener_active = FALSE;
@@ -28,20 +44,9 @@ typedef struct
 	WrapperMouseEventInfo *info;
 } MouseInfo;
 
+/* local function prototyprs */
 static gint mouse_listener (MouseInfo *minfo);
 
-/* type of drag and drop for links */
-const GtkTargetEntry link_drag_types[] =
-{
-        { "GALEON_URL",    0, DND_TARGET_GALEON_URL   },
-        { "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
-        { "STRING",        0, DND_TARGET_STRING       }
-};
-const gint link_drag_types_num_items = (sizeof (link_drag_types) / 
-					sizeof (GtkTargetEntry));
-GtkTargetList *link_drag_types_tl = NULL;
-
-
 /**
  * mozembed_new_window_cb: GTKMOZEMBED SIGNAL, emitted any time a new 
  * window is requested by the document 
@@ -60,34 +65,18 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	return_if_not_window (window);
 	g_assert (retval != NULL);
 
-	/* check popups aren't filtered */
-	if (!(gnome_config_get_bool (CONF_GENERAL_ALLOW_POPUPS)))
-	{
-		window_update_temp_message (window, _("Popup blocked"));
-		*retval = embed->mozEmbed;
-		return;
-	}
-
 	/* check config */
 	open_in_tab = gnome_config_get_bool (CONF_APPEARANCE_TABBED_POPUPS);
 
 	/* create a new browser */
-	new_embed = embed_create_hidden (embed, !open_in_tab);
-	return_if_not_embed (new_embed);
+	new_embed = embed_create (embed, !open_in_tab, FALSE, chromemask);
 
 	/* set the popup flag */
 	new_embed->is_popup = TRUE;
 
-	/* set chrome settings and turn off tab listing if this is actually
-	 * a new window instead of a new tab */
+	/* turn off tab display if in a new window */
 	if (!open_in_tab)
 	{
-		gtk_moz_embed_set_chrome_mask (new_embed->mozEmbed,
-					       chromemask);
-
-		/* FIXME: this is messy, since the tab display is getting
-		 * turned on in embed_create_hidden.  no flicker visible
-		 * here, though. */
 		gtk_notebook_set_show_tabs 
 			(GTK_NOTEBOOK (new_embed->parent_window->notebook),
 			 FALSE);
@@ -97,6 +86,28 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	*retval = GTK_MOZ_EMBED (new_embed->mozEmbed);
 }
 
+#if MOZILLA_VERSION > VERSION2(0,9)
+void new_window_orphan_cb (GtkMozEmbedSingle *embed,
+			   GtkMozEmbed **retval, guint chromemask,
+			   gpointer data)
+{
+	GaleonEmbed *new_embed;
+
+	/* create a new browser */
+	new_embed = embed_create (NULL, FALSE, FALSE, chromemask);
+
+	/* set the popup flag */
+	new_embed->is_popup = TRUE;
+
+	/* turn off tab display */
+	gtk_notebook_set_show_tabs 
+		(GTK_NOTEBOOK (new_embed->parent_window->notebook), FALSE);
+
+	/* return the new browser to gtkmozembed */
+	*retval = GTK_MOZ_EMBED (new_embed->mozEmbed);
+}
+#endif
+
 /**
  * mozembed_visibility_cb: GTKMOZEMBED SIGNAL, emitted when the toplevel
  * window need to be showed or hidden
@@ -105,8 +116,6 @@ void
 mozembed_visibility_cb (GtkMozEmbed *dummy, gboolean visibility, 
 			GaleonEmbed *embed) 
 {
-	GtkMozEmbedChromeFlags flags;
-
 	return_if_not_embed (embed);
 
 	/* set visiblity of this embed */
@@ -117,24 +126,6 @@ mozembed_visibility_cb (GtkMozEmbed *dummy, gboolean visibility,
 	{
 		embed_wrapper_init (embed);
 	}
-
-	/* FIXME: This is ugly, since the resizing of the mozembed
-	   widget is visible. */
-	/* FIXME: this is also not right for tabbed mode... */
-	/* get and apply the requested chrome settings */
-	flags = gtk_moz_embed_get_chrome_mask (embed->mozEmbed);
-	if (!(flags & GTK_MOZ_EMBED_FLAG_MENUBARON))
-	{
-		window_menubar_hide (embed->parent_window);
-	}
-	if (!(flags & GTK_MOZ_EMBED_FLAG_TOOLBARON))
-	{
-		window_toolbar_hide (embed->parent_window);
-	}
-	if (!(flags & GTK_MOZ_EMBED_FLAG_STATUSBARON))
-	{
-		window_statusbar_hide (embed->parent_window);
-	}
 }
 
 /**
@@ -167,7 +158,8 @@ mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	{
 		g_free (embed->site_location);
 	}
-	embed->site_location = gtk_moz_embed_get_location (embed->mozEmbed);
+	embed->site_location = 
+		gtk_moz_embed_get_location (GTK_MOZ_EMBED(embed->mozEmbed));
 
 	/* update in gui */
 	embed_update_page_location (embed);
@@ -187,7 +179,11 @@ mozembed_title_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	{
 		g_free (embed->site_title);
 	}
-	embed->site_title = gtk_moz_embed_get_title (embed->mozEmbed);
+#if MOZILLA_VERSION > VERSION2(0,9)
+	embed->site_title = mozilla_get_document_title (embed);
+#else
+	embed->site_title = gtk_moz_embed_get_title (GTK_MOZ_EMBED(embed->mozEmbed));
+#endif
 
 	/* update in gui */
 	embed_update_page_title (embed);
@@ -214,13 +210,14 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		spinner_start (window);
 		embed_set_notebook_label_status (embed, TAB_LOADING);
 		embed_progress_clear (embed);
+		embed->when_started = time (NULL);
 	}
 	embed->load_started++;
 
 	/* update the buttons view */
 	if (embed == window->active_embed)
 	{
-		window_update_nav_buttons (window);
+		window_update_nav_controls (window);
 		window_update_status_bar (window);
 	}
 }
@@ -233,26 +230,13 @@ void
 mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	GaleonWindow *window;
-	HistoryItem *hi;
+	gint zoom;
 
 	/* check callback args */
 	return_if_not_embed (embed);
 	window = embed->parent_window;
 	return_if_not_window (window);
 
-	/* add the url to the history */
-	if (embed->bytesLoaded)
-	{
-		hi = history_visited (embed->site_location,
-				      embed->site_title);
-
-		/* add url to recent menu */
-		if (hi != NULL)
-		{
-			history_add_recent_menu (hi, GNOME_APP(window->WMain));
-		}
-	}
-
 	/* set in the embed */
 	embed->load_started--;
 	embed_progress_clear (embed);
@@ -260,6 +244,18 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* stop the window spinner */
 	if (embed->load_started == 0)
 	{
+		/* check zoom setting for the new site */
+		zoom = history_get_zoom (embed->site_location);
+		if (zoom == 0 && embed->zoom_auto_set)
+		{
+			embed_set_zoom (embed, 100);
+		}
+		else if (zoom != 0 && zoom != embed->zoom)
+		{
+			embed_set_zoom (embed, zoom);
+			embed->zoom_auto_set = TRUE;
+		}
+
 		spinner_stop (window);
 		embed_set_notebook_label_status (embed, TAB_NORMAL);
 	}
@@ -267,7 +263,7 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	/* update the buttons view */
 	if (embed == window->active_embed)
 	{
-		window_update_nav_buttons (window);
+		window_update_nav_controls (window);
 		window_update_status_bar (window);
 	}
 }
@@ -280,11 +276,23 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 				    guint status, GaleonEmbed *embed) 
 {
 	GaleonWindow *window;
+	float f;
 
 	return_if_not_embed (embed);
 	window = embed->parent_window;
 	return_if_not_window (window);
 
+	/* sync the zoom levels, this shouldn't be necessary but mozilla
+	 * ignore the zoom level setting until it's loaded its first doc */
+	if (embed->wrapper != NULL)
+	{
+		mozilla_get_zoom(embed, &f);
+		if ((gint)(f * 100.0) != embed->zoom)
+		{
+			mozilla_set_zoom (embed, (float)embed->zoom / 100.0);
+		}
+	}
+
 	/* clear temporary message, if any */
 	if (embed == window->active_embed)
 	{
@@ -293,31 +301,28 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 
 	if (flags & GTK_MOZ_EMBED_FLAG_IS_REQUEST)
 	{
-		if (flags & GTK_MOZ_EMBED_FLAG_REDIRECTING)
+		if (flags & GTK_MOZ_EMBED_FLAG_START)
+		{
+			/* ignore */
+		}
+		else if (flags & GTK_MOZ_EMBED_FLAG_REDIRECTING)
 		{
 			embed->statusMessage = _("Redirecting to site...");
 		}
 		else if (flags & GTK_MOZ_EMBED_FLAG_TRANSFERRING)
 		{
-			embed->statusMessage = _("Transferring data from site...");
+			embed->statusMessage = 
+				_("Transferring data from site...");
 		}
 		else if (flags & GTK_MOZ_EMBED_FLAG_NEGOTIATING)
 		{
-			embed->statusMessage = _("Waiting for authorization...");
+			embed->statusMessage = 
+				_("Waiting for authorization...");
+		}
+		else if (flags & GTK_MOZ_EMBED_FLAG_STOP)
+		{
+			/* ignore */
 		}
-	}
-
-	if (status == GTK_MOZ_EMBED_STATUS_FAILED_DNS)
-	{
-		embed->statusMessage = _("Site not found.");
-	}
-	else if (status == GTK_MOZ_EMBED_STATUS_FAILED_CONNECT)
-	{
-		embed->statusMessage = _("Failed to connect to site.");
-	}
-	else if (status == GTK_MOZ_EMBED_STATUS_FAILED_TIMEOUT)
-	{
-		embed->statusMessage = _("Failed due to connection timeout.");
 	}
 
 	if (flags & GTK_MOZ_EMBED_FLAG_IS_DOCUMENT)
@@ -332,6 +337,34 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy, gint flags,
 		}
 	}
 
+	switch (status)
+	{
+	case GTK_MOZ_EMBED_STATUS_FAILED_DNS:
+		embed->statusMessage = _("Site not found.");
+		break;
+		
+	case GTK_MOZ_EMBED_STATUS_FAILED_CONNECT:
+		embed->statusMessage = _("Failed to connect to site.");
+		break;
+
+	case GTK_MOZ_EMBED_STATUS_FAILED_TIMEOUT:
+		embed->statusMessage = _("Failed due to connection timeout.");
+		break;
+
+	case 0:
+	case GTK_MOZ_EMBED_STATUS_FAILED_USERCANCELED:
+		/* don't do a message for this */
+		break;
+
+	default:
+/* FIXME: find out what these are, but for now it's best not to
+ * alarm users with lots of warning messages -- MattA
+		g_warning ("unhandled status message 0x%08x\n", status);
+ */
+		break;
+	}
+
+	/* display the update */
 	window_update_status_bar (window);
 }
 
@@ -394,7 +427,8 @@ mozembed_link_message_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	return_if_not_window (window);
 	
 	/* get the link message */
-	message = gtk_moz_embed_get_link_message (embed->mozEmbed);
+	message = gtk_moz_embed_get_link_message 
+		(GTK_MOZ_EMBED(embed->mozEmbed));
 
 	/* update browser message */
 	window_update_temp_message (window, message);
@@ -421,7 +455,8 @@ mozembed_js_status_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
  	if (gnome_config_get_bool (CONF_ADVANCED_STATUSBAR_REWRITE))
   	{
 		/* get the javascript message */
-		message = gtk_moz_embed_get_js_status (embed->mozEmbed);
+		message = gtk_moz_embed_get_js_status 
+			(GTK_MOZ_EMBED(embed->mozEmbed));
 		
 		/* update the status bar message */
 		if (message && embed->is_active)
@@ -445,10 +480,6 @@ mozembed_open_uri_cb (GtkMozEmbed *dummy, const char *uri, GaleonEmbed *embed)
 	if (handle_foreign_protocols(uri))
 		return TRUE;
 
-	/* handle foreign mime types, e.g. RPMS (if needed) */
-	if (handle_foreign_mime_types (uri, embed))
-		return TRUE;
-
 	/* we haven't handled it, so let Mozilla try to load it */
 	return FALSE;
 }
@@ -472,13 +503,13 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	info = g_new0 (WrapperMouseEventInfo, 1);
 	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
 	{
+		mozilla_free_context_info_sub(&info->ctx);
 		g_free(info);
 		return FALSE;
 	}
 
-#if MOZILLA_MILESTONE == 81
+	/* FIXME remove this */
 	info->button ++;
-#endif
 
 	if (info->button == 3)
 	{
@@ -507,6 +538,7 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			context_show_menu (embed, &info->ctx);
 	}
 
+	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
 
 	return FALSE;
@@ -534,24 +566,22 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	info = g_new0 (WrapperMouseEventInfo,1);
 	if (!mozilla_get_mouse_event_info (embed, dom_event, info)) 
 	{
+		mozilla_free_context_info_sub(&info->ctx);
 		g_free(info);
 		return FALSE;
 	}
 
-#if MOZILLA_MILESTONE == 81
-	info->button ++;
-#endif
-
 	/* make a sound if a link was clicked */
-	if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link && info->button != 3)
+	if ((info->ctx.context & CONTEXT_LINK) && 
+	    (info->ctx.link && info->button != 3))
 	{
-		gnome_triggers_do ("", "program", "galeon", "url_clicked",
-				   NULL);
+		gnome_triggers_do ("", "program", "galeon", 
+				   "url_clicked", NULL);
 	}
 
 	switch (info->button)
 	{
-	case 3:
+	case 2:
 		if (gnome_config_get_int("/galeon/Mouse/right_button_action")
 		    == 2)
 		{
@@ -560,24 +590,24 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 			{
 				mouse_listener_active = FALSE;
 			}
-			gtk_moz_embed_go_back (embed->mozEmbed);
+			gtk_moz_embed_go_back (GTK_MOZ_EMBED(embed->mozEmbed));
 		}
 		else
 			context_show_menu (embed, &info->ctx);
 
 		handled = TRUE;
 		break;
-	case 2:
+	case 1:
 		if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
 		{
 			if ((info->modifier & SHIFT_KEY) &&
 			    !((info->modifier & ALT_KEY) || 
 			    (info->modifier & CTRL_KEY)))
 				embed_create_from_url (embed, info->ctx.link,
-						       tabbed_mode);
+						       FALSE, tabbed_mode);
 			else
 				embed_create_from_url (embed, info->ctx.link,
-						       !tabbed_mode);
+						       FALSE, !tabbed_mode);
 			handled = TRUE;
 		} 
 		else 
@@ -595,24 +625,29 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		}
 		break;
 
-	case 1: 
+	case 0: 
 		if ((info->ctx.context & CONTEXT_IMAGE) &&
 		    (info->modifier & CTRL_KEY))
 		{
-			embed_save_image_default (embed, info->ctx.img);
+			embed_save_url_default (embed, info->ctx.img);
 			break;
 		}	
-#if MOZILLA_MILESTONE != 81
+		else if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link &&
+						 (info->modifier & SHIFT_KEY))
+		{
+			mozilla_save_link (embed, info->ctx.link);
+		}
 		else if ((info->ctx.context & CONTEXT_LINK) && info->ctx.link)
 		{
 			handled = handle_foreign_protocols (info->ctx.link);
 		}
-#endif
+
 		break;
 	default:
 		break;
 	}
 
+	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
 
 	return handled;
@@ -637,6 +672,7 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	info = g_new0 (WrapperKeyEventInfo,1);
 	if (!mozilla_get_key_event_info(embed, dom_event, info))
 	{
+		mozilla_free_context_info_sub(&info->ctx);
 		g_free(info);
 		return FALSE;
 	}
@@ -680,42 +716,22 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	{
 		switch(info->key)
 		{
-/* FIXME: perhaps we should get rid of these bindings, they conflict
- * with alot of peoples window manager bindings. I'd suggest Ctrl-[ and 
- * Ctrl-], but I'd like to know what other people think. Changing them
- * would make it easier to rebind them under GTK+ and remove some code
- * -- MattA 24/3/2001 */
-/* Nope, bad idea. These are pretty standard accelerators, and anyone
- * used to Netscape will expect them to work.  GTK+ doesn't allow cursor keys
- * to be used as accelerators, so these have to be handled here.
- * --Josh */
 		case DOM_VK_KP_LEFT:
 		case DOM_VK_LEFT:
-			gtk_moz_embed_go_back(embed->mozEmbed);
+			gtk_moz_embed_go_back(GTK_MOZ_EMBED(embed->mozEmbed));
 			handled = TRUE;
 			break;
 		case DOM_VK_KP_RIGHT:
 		case DOM_VK_RIGHT:
-			gtk_moz_embed_go_forward(embed->mozEmbed);
+			gtk_moz_embed_go_forward(GTK_MOZ_EMBED(embed->mozEmbed));
 			handled = TRUE;
 			break;
-#if 0 /* FIXME, remove unless people complain */
-		case DOM_VK_UP:
-			gtk_notebook_prev_page 
-				(GTK_NOTEBOOK (window->notebook));
-			handled = TRUE;
-			break;
-		case DOM_VK_DOWN:
-			gtk_notebook_next_page 
-				(GTK_NOTEBOOK (window->notebook));
-			handled = TRUE;
-			break;
-#endif
 		default:
 			break;
 		}
 	}
-	else if ((info->modifier & ALT_KEY) && !(info->modifier & (CTRL_KEY | SHIFT_KEY)))
+	else if ((info->modifier & ALT_KEY) && 
+		 !(info->modifier & (CTRL_KEY | SHIFT_KEY)))
 	{
 		switch(info->key)
 		{
@@ -737,10 +753,6 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 					       9);
 			handled = TRUE;
 			break;
-		case DOM_VK_EQUALS:
-			gtk_spin_button_spin (GTK_SPIN_BUTTON (window->zoom_spin), GTK_SPIN_STEP_FORWARD, 10);
-			handled = TRUE;
-			break;
 		default:
 			break;
 
@@ -752,18 +764,14 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 		{
 		case DOM_VK_g:
 		case DOM_VK_G:
+		case DOM_VK_l:
+		case DOM_VK_L:
 			gtk_editable_select_region
 				(GTK_EDITABLE(window->toolbar_entry), 0, -1);
 			gtk_window_set_focus (GTK_WINDOW(window->WMain),
 					      window->toolbar_entry);
 			handled = TRUE;
 			break;
-		case DOM_VK_EQUALS:
-			gtk_spin_button_spin
-					(GTK_SPIN_BUTTON(window->zoom_spin),
-					 GTK_SPIN_STEP_FORWARD, 10);
-			handled = TRUE;
-			break;
 		default:
 			break;
 
@@ -847,10 +855,12 @@ gint mozembed_dom_key_press_cb (GtkMozEmbed *dummy, gpointer dom_event,
 	   main window in case it is a menu accelerator key */
 	if (!handled)
 	{
-		handled = gtk_accel_groups_activate (GTK_OBJECT (window->WMain),
-						     info->key, state);
+		handled = 
+			gtk_accel_groups_activate (GTK_OBJECT (window->WMain),
+						   info->key, state);
 	}
 
+	mozilla_free_context_info_sub(&info->ctx);
 	g_free (info);
 
 	/* If it wasn't an accelerator, let Mozilla handle it */
@@ -951,6 +961,7 @@ mozembed_drag_drop_cb (GtkWidget * widget, GdkDragContext *context,
 		       gint x, gint y, GtkSelectionData *selection_data, 
 		       guint info, guint time)
 {
+	g_warning ("unexpected mozembed_drag_drop_cb\n");
 	return FALSE;
 }
 
@@ -966,6 +977,7 @@ mouse_listener (MouseInfo *minfo)
 
 	if (!mouse_listener_active)
 	{
+		mozilla_free_context_info_sub(&minfo->info->ctx);
 		g_free (minfo->info);
 		g_free (minfo);
 
