@@ -16,46 +16,72 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <galeon.h>
+#include "galeon.h"
+#include "mozilla_i18n.h"
+#include "mozilla.h"
+#include "galeon-wrapper.h"
+
+#include <gtkmozembed_internal.h>
+/* FIXME !!! */
+#include <gnome.h>
+#include <time.h>
 
 #include "nsString.h"
 #include "nsIPref.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsIUnicodeEncoder.h"
-#include "galeon-wrapper.h"
-#ifdef CVS_SOURCE
-#include <time.h>
 #include "nsIPrintOptions.h"
 #include "nsGfxCIID.h"
 #include "nsICacheService.h"
 #include "nsIPassword.h"
 #include "nsIPasswordManager.h"
+#include "nsIDOMWindow.h"
+#include "nsIEmbeddingSiteWindow.h"
+#include "nsIWindowWatcher.h"
+#include "nsIWebBrowserChrome.h"
+
 #include "PromptService.h"
-#else
-#include "nsINetDataCacheManager.h"
-#endif
+#include "ContentHandler.h"
+#include "ContentOverrideHandler.h"
+#include "FilePicker.h"
+
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-#ifdef CVS_SOURCE
 static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
-#endif
 
 /* local function prototypes */
-static gboolean mozilla_find_setup(nsITextServicesDocument *aDoc,
-				   PRInt32 *outBlockOffset,
-				   gboolean search_backwards, gboolean start);
-static gchar *mozilla_get_attribute (nsIDOMNode *node, gchar *attribute);
-static nsIDocShell *mozilla_get_primary_docshell (GtkMozEmbed *b);
-static nsIDocShell *mozilla_get_event_docshell (GtkMozEmbed *b, 
-						nsIDOMMouseEvent *event);
-static gboolean mozilla_resolve_url (char *base, char *relative,
-				     char **resolved);
-#ifndef CVS_SOURCE
-	static gchar** split_list (gchar *list);
-	static gchar* join_numbers (GList *cookies);
-#endif
 static char  *convert_ns_string_to_c_string (const nsString & ns_string);
 
+extern "C" gboolean
+mozilla_load_prefs (void)
+{
+#if MOZILLA_VERSION > VERSION2(0,9)
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	g_return_val_if_fail (prefService != nsnull, FALSE);
+
+	nsresult rv = prefService->ReadUserPrefs (nsnull);
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
+}
+
+extern "C" gboolean
+mozilla_save_prefs (void)
+{
+#if MOZILLA_VERSION > VERSION2(0,9)
+	nsCOMPtr<nsIPrefService> prefService = 
+				do_GetService (NS_PREFSERVICE_CONTRACTID);
+	g_return_val_if_fail (prefService != nsnull, FALSE);
+
+	nsresult rv = prefService->SavePrefFile (nsnull);
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
+}
+
 /**
  * mozilla_preference_set: set a string mozilla preference
  */
@@ -125,23 +151,146 @@ mozilla_preference_set_int (const char        *preference_name,
 extern "C" gboolean 
 mozilla_init_promptService (void)
 {
-	/* Create PromptService */
+	if (!pushed_startup) return FALSE;
+
 	nsresult rv = NS_OK;
-#ifdef CVS_SOURCE
 
-#define NS_PROMPTSERVICE_CID \
-   {0xa2112d6a, 0x0e28, 0x421f, {0xb4, 0x6a, 0x25, 0xc0, 0xb3, 0x8, 0xcb, 0xd0}}
 	static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
      
 	nsCOMPtr<nsIFactory> promptFactory;
 	rv = NS_NewPromptServiceFactory(getter_AddRefs(promptFactory));
 	if (NS_FAILED(rv)) return rv;
-	rv = nsComponentManager::RegisterFactory(kPromptServiceCID,
-						 "Prompt Service",
-						 "@mozilla.org/embedcomp/prompt-service;1",
-						 promptFactory,
-						 PR_TRUE);  
-#endif
+	rv = nsComponentManager::
+		RegisterFactory(kPromptServiceCID, "Prompt Service",
+				"@mozilla.org/embedcomp/prompt-service;1",
+				promptFactory, PR_TRUE);  
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+}
+
+/**
+ * mozilla_init_ContentHandler: Initialise Content Handler.
+ */
+extern "C" gboolean 
+mozilla_init_ContentHandler (void)
+{
+	if (!pushed_startup) return FALSE;
+
+	nsresult rv = NS_OK;
+
+	static NS_DEFINE_CID(kContentHandlerCID, NS_IHELPERAPPLAUNCHERDIALOG_IID);
+     
+	nsCOMPtr<nsIFactory> ContentHandlerFactory;
+	rv = NS_NewContentHandlerFactory (getter_AddRefs(ContentHandlerFactory));
+	if (NS_FAILED(rv)) return rv;
+	rv = nsComponentManager::
+		RegisterFactory(kContentHandlerCID,
+				NS_IHELPERAPPLAUNCHERDLG_CLASSNAME,
+				NS_IHELPERAPPLAUNCHERDLG_CONTRACTID,
+				ContentHandlerFactory, PR_TRUE);  
+	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+}
+
+/**
+ * mozilla_register_contentOverrideHandler: Registers an explicit contentHandler
+ * @mime-type: The mime-type to register a contentHandler for.
+ */
+extern "C" gboolean mozilla_register_contentOverrideHandler (const char *mime_type)
+{
+        if (!pushed_startup) return FALSE;
+
+        nsresult rv = NS_OK;
+  
+        static NS_DEFINE_CID(kContentOverrideHandlerCID,
+        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
+     
+        nsCOMPtr<nsIFactory> CHFactory;
+        rv = NS_NewContentOverrideHandlerFactory(getter_AddRefs(CHFactory));
+        if (NS_FAILED(rv)) return FALSE;
+      
+        gchar *handlerName = g_strdup_printf ("Content Handler for %s",
+                                              mime_type);
+        gchar *contractID = g_strdup_printf (NS_CONTENT_HANDLER_CONTRACTID_PREFIX
+                                             "%s",mime_type);
+   
+        rv = nsComponentManager::RegisterFactory(kContentOverrideHandlerCID,
+                                                 handlerName,   
+                                                 contractID,
+                                                 CHFactory,
+                                                 PR_TRUE);
+        g_free (handlerName);
+        g_free (contractID);
+
+        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+}
+
+/**
+ * mozilla_register_stored_contentOverrideHandler: Registers contentHandlers
+ * for all stored mime-types with the override action type.
+ */
+extern "C" gboolean mozilla_register_stored_contentOverrideHandlers (void)
+{  
+
+        gboolean rv = TRUE;
+        for (GList *mime_type = g_list_first(mime_types) ;
+             mime_type!=NULL ;
+             mime_type = g_list_next(mime_type))
+        {
+                if (((MimeItem *)mime_type->data)->action == 
+               		MIME_OVERRIDE_MOZILLA)
+                {
+                        if (!mozilla_register_contentOverrideHandler
+                                (((MimeItem *)mime_type->data)->name))   
+                        {
+                                rv = FALSE;
+                        }
+                }
+        }
+        return rv;
+}
+
+/**
+ * mozilla_unregister_stored_contentOverrideHandler: unregisters contentHandlers
+ * for all stored mime-types
+ */
+extern "C" gboolean mozilla_unregister_stored_contentOverrideHandlers (void)
+{
+        if (!pushed_startup) return FALSE;
+        
+        nsresult rv = NS_OK;
+        static NS_DEFINE_CID(kContentOverrideHandlerCID,
+        		     GALEON_CONTENTOVERRIDEHANDLER_CID);
+                                                 
+        nsCOMPtr<nsIFactory> CHFactory;
+        
+        rv = nsComponentManager::FindFactory (kContentOverrideHandlerCID,
+                                              getter_AddRefs(CHFactory));
+        if (NS_FAILED(rv)) return FALSE;
+ 
+        rv = nsComponentManager::UnregisterFactory (kContentOverrideHandlerCID,
+                                                    CHFactory);
+ 
+        return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+}
+
+/**
+ * mozilla_init_FilePicker: Register FilePicker Factory.
+ */
+extern "C" gboolean 
+mozilla_init_FilePicker (void)
+{
+	if (!pushed_startup) return FALSE;
+
+	nsresult rv = NS_OK;
+
+	static NS_DEFINE_CID(kFilePickerCID, NS_IFILEPICKER_IID);
+     
+	nsCOMPtr<nsIFactory> FilePickerFactory;
+	rv = NS_NewFilePickerFactory (getter_AddRefs(FilePickerFactory));
+	if (NS_FAILED(rv)) return rv;
+	rv = nsComponentManager::RegisterFactory(kFilePickerCID,
+						 G_FILEPICKER_CLASSNAME,
+						 G_FILEPICKER_CONTRACTID,
+						 FilePickerFactory, PR_TRUE);  
 	return NS_SUCCEEDED (rv) ? TRUE : FALSE;
 }
 
@@ -175,77 +324,46 @@ mozilla_wrapper_destroy (GaleonEmbed *embed)
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
-extern "C" gboolean mozilla_find (GaleonEmbed *embed, WrapperSearchProperties *properties)
+extern "C" gboolean mozilla_find (GaleonEmbed *embed, 
+				  WrapperSearchProperties *properties)
 {
 	GaleonWrapper *wrapper;
+	PRUnichar *search_string;		
 	PRBool didFind;
 
 	g_return_val_if_fail (embed != NULL, FALSE);
 	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
 
 	wrapper = (GaleonWrapper *)embed->wrapper;
-
-	wrapper->Find (properties->search_string, properties->match_case, 
+	search_string = mozilla_locale_to_unicode (properties->search_string);
+	wrapper->Find (search_string, properties->match_case, 
 		       properties->backwards, &didFind);
+	g_free (search_string);
 
 	return didFind;
 }
 
-extern "C" gboolean mozilla_save_image (GaleonEmbed *embed, const char *image, const char *file_name)
-{
-#if MOZILLA_MILESTONE == 81
-	nsresult result;
-	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-
-	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
-
-	result = wrapper->SaveURI (image, file_name);
-
-	return NS_SUCCEEDED (result) ? TRUE : FALSE;
-#else
-	nsresult result;
-	gchar *dirname = g_dirname(file_name);
-	gchar *basename =  g_basename(file_name);
-	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-
-	/* check if the file is local */
-	if (strncmp ((gchar *)image, "file:/", 6) == 0)
-	{
-		return copy_file (image, file_name);
-	}
-	else
-	{
-		result = wrapper->SaveURLFromCache 
-			((gchar *)image , dirname, basename);
-
-		/* FIXME use gtm if failed */
-
-		return NS_SUCCEEDED (result) ? TRUE : FALSE;
-	}
-#endif
-}
-
-extern "C" gboolean mozilla_save (GaleonEmbed *embed, const char *file_name)
+extern "C" gboolean mozilla_save_url (GaleonEmbed *embed, const char *url, 
+					const char *file_name)
 {
 	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
 
 	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
 
-	result = wrapper->SaveDocument (file_name);
+	result = wrapper->SaveURI (url, file_name);
 
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
-extern "C" gboolean mozilla_save_main_document (GaleonEmbed *embed, const char *file_name)
+extern "C" gboolean mozilla_save_link (GaleonEmbed *embed, const char *url)
 {
 	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-	char *target;
 
 	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
 
-	result = wrapper->SaveMainDocument (file_name);
+	result = wrapper->SaveLink (url);
 
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
@@ -303,7 +421,6 @@ mozilla_get_zoom (GaleonEmbed *embed, float *f)
 extern "C" gboolean
 mozilla_print (GaleonEmbed *embed, PrintInfo *info)
 {
-#ifdef CVS_SOURCE
 	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)(embed->wrapper);
 
@@ -334,9 +451,6 @@ mozilla_print (GaleonEmbed *embed, PrintInfo *info)
 
 	result = wrapper->Print(options);
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
-#else
-	return FALSE;
-#endif
 }
 
 /**
@@ -390,35 +504,65 @@ mozilla_session_history_go (GaleonEmbed *embed, int index)
 	return NS_SUCCEEDED (result) ? TRUE : FALSE;
 }
 
-extern "C" char* 
+/**
+ * mozilla_get_main_document_url: Get URL of main document
+ * embed: Parent widget of document
+ */
+extern "C" gchar * 
+mozilla_get_main_document_url (GaleonEmbed *embed)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	gchar *url = NULL;
+	nsresult result;
+
+	g_return_val_if_fail (embed->wrapper != NULL, NULL); 
+
+	result = wrapper->GetMainDocumentUrl (&url);
+
+	return NS_SUCCEEDED (result) ? url : NULL;
+}
+
+/**
+ * mozilla_get_document_url: Get URL of focused document (frame)
+ * embed: Parent widget of document
+ */
+extern "C" gchar * 
 mozilla_get_document_url (GaleonEmbed *embed)
 {
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
-	char *url;
+	gchar *url = NULL;
+	nsresult result;
 
-	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+	g_return_val_if_fail (embed->wrapper != NULL, NULL); 
 
-	wrapper->GetDocumentUrl (&url);
-	return url;
+	result = wrapper->GetDocumentUrl (&url);
+
+	return NS_SUCCEEDED (result) ? url : NULL;
 }
 
-#ifndef CVS_SOURCE
+#if MOZILLA_VERSION > VERSION2(0,9)
 /**
- * mozilla_view_source_mode: Turn a embed source view mode
+ * mozilla_get_document_title: get the document title in a locale specific
+ * NULL terminated C string, using the private UniCode interface.
  */
-extern "C" gboolean 
-mozilla_view_source_mode (GaleonEmbed *embed, gboolean view_source)
+extern "C" gchar * 
+mozilla_get_document_title (GaleonEmbed *embed)
 {
-	nsresult result;
-	GaleonWrapper *wrapper = (GaleonWrapper *)(embed->wrapper);
+	PRUnichar *unicode_title;
+	gchar *title;
 
-	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+	/* get the title in unicode */
+	unicode_title = gtk_moz_embed_get_title_unichar 
+		(GTK_MOZ_EMBED(embed->mozEmbed));
 
-	result = wrapper->SetViewSourceMode (view_source ? 
-					     (int)(nsIDocShell::viewSource) : 
-					     (int)(nsIDocShell::viewNormal));
+	/* attempt conversion */
+	title = mozilla_unicode_to_locale (unicode_title);
 
-	return NS_SUCCEEDED (result) ? TRUE : FALSE;
+	/* free unicode version */
+	g_free (unicode_title);
+
+	/* return it */
+	return title;
 }
 #endif
 
@@ -443,7 +587,8 @@ mozilla_get_key_event_info (GaleonEmbed *embed, gpointer event, WrapperKeyEventI
  * 
  */
 extern "C" gboolean
-mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event, WrapperMouseEventInfo *context)
+mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event, 
+			      WrapperMouseEventInfo *context)
 {
 	nsresult result;
 	return_val_if_not_embed (embed, FALSE);
@@ -451,7 +596,8 @@ mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event, WrapperMouseEv
 	g_return_val_if_fail (wrapper, FALSE);
 	g_return_val_if_fail (event, FALSE);
 	
-	result = wrapper->GetMouseEventContext ((nsIDOMMouseEvent*)event, context);
+	result = wrapper->GetMouseEventContext ((nsIDOMMouseEvent*)event, 
+						context);
 
 	/* for some reason we get different numbers on PPC, this fixes
 	 * that up... -- MattA */
@@ -467,6 +613,21 @@ mozilla_get_mouse_event_info (GaleonEmbed *embed, gpointer event, WrapperMouseEv
 	return NS_SUCCEEDED(result) ? TRUE : FALSE;
 }
 
+
+/**
+ * mozilla_free_context_info_sub: free info contained in WrapperContextInfo 
+ * struct.
+ */
+extern "C" void
+mozilla_free_context_info_sub (WrapperContextInfo *ctx)
+{
+	g_free(ctx->linktext);
+
+	/* created by mozilla: */
+	if (ctx->link) nsMemory::Free (ctx->link);
+	if (ctx->img) nsMemory::Free (ctx->img);
+}
+
 /**
  * mozilla_list_cookies: get a list of all saved cookies
  */
@@ -474,7 +635,6 @@ extern "C" GList *
 mozilla_list_cookies (void)
 {
 	GList *cookies = NULL;
-#ifdef CVS_SOURCE
 	nsresult result;
 
 	nsCOMPtr<nsICookieManager> 
@@ -514,36 +674,6 @@ mozilla_list_cookies (void)
 		
 		cookies = g_list_append (cookies, c);
 	}	
-#else
-	char *list;
-	gchar **split;
-	gchar **s;
-
-	nsCOMPtr<nsICookieViewer> 
-			cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
-	
-	cookieviewer->GetCookieValue (&list);
-
-	split = split_list (list);
-	for (s = split; *s != NULL; ++s) {
-		Cookie *c = g_new0 (Cookie, 1);
-
-		c->base.number = *(s++);
-		c->name = *s++;
-		c->value = *s++;
-		c->base.type = *s++;
-		c->base.domain = *s++;
-		c->path = *s++;
-		c->secure = *s++;
-		c->expire = *s;
-                g_strchomp (c->expire);
-
-		cookies = g_list_append (cookies, c);
-	}
-	g_free (split);
-
-	delete list;
-#endif
 	return cookies;
 }
 
@@ -557,8 +687,6 @@ mozilla_get_permissions (int type)
 {
 	GList *permissions = NULL;
 	nsresult result;
-#ifdef CVS_SOURCE
-
 	nsCOMPtr<nsIPermissionManager> 
 		permissionManager = do_CreateInstance (PERMISSIONMANAGER_ID);
 	nsCOMPtr<nsISimpleEnumerator> permissionEnumerator;
@@ -593,32 +721,7 @@ mozilla_get_permissions (int type)
 			permissions = g_list_append (permissions, b);
 		}
 	}
-#else
-	char *list;
-	gchar **split, **s;
 
-	nsCOMPtr<nsICookieViewer> 
-			cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
-	
-	result = cookieviewer->GetPermissionValue (type, &list);
-	g_return_val_if_fail (NS_SUCCEEDED(result), NULL);
-
-	split = split_list (list);
-	for (s = split; *s != NULL; ++s) {
-		CookieBase *b = g_new0 (CookieBase, 1);
-
-		b->number = *s++;
-
-		b->type = g_strndup (*s, 1);
-		b->domain = g_strdup (*s+1);
-		g_free (*s);
-
-		permissions = g_list_append (permissions, b);
-	}
-	
-	g_free (split);
-	delete list;
-#endif
 	return permissions;
 };
 
@@ -631,17 +734,11 @@ mozilla_block_url (const char *imgURL)
 {
 	nsresult result;
 	
-#ifdef	CVS_SOURCE
 	nsCOMPtr<nsIImgManager> 
 		imageManager = do_CreateInstance (IMAGEMANAGER_ID);
 	
 	result = imageManager->Block (imgURL);
-#else
-	nsCOMPtr<nsICookieViewer> 
-		cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
-	
-	result = cookieviewer->BlockImage (imgURL);
-#endif
+
 	g_return_if_fail (NS_SUCCEEDED(result));
 }
 
@@ -654,7 +751,6 @@ mozilla_block_url (const char *imgURL)
 extern "C" void
 mozilla_set_permission (GaleonEmbed *embed, gboolean permit, int type)
 {
-	nsresult result;
 	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
 
 	if (!wrapper) return;
@@ -662,8 +758,6 @@ mozilla_set_permission (GaleonEmbed *embed, gboolean permit, int type)
 	wrapper->SetSitePermission (permit ? PR_TRUE : PR_FALSE, type);
 }
 
-#ifdef CVS_SOURCE
-
 /**
  * mozilla_remove_cookies:
  * @gone: list of cookies which are to be removed
@@ -710,99 +804,6 @@ mozilla_remove_permissions (GList *gone, int type)
 	return TRUE;
 }
 
-#else //CVS_SOURCE
-
-/**
- * mozilla_set_cookies: remove a set of cookies from the store
- * @gone_c: cookies to remove
- * @gone_p: servers to remove from cookie permissions
- * @gone_i: servers to remove from image permissions
- * @block:  whether to block the cookie in future
- */
-extern "C" gboolean
-mozilla_set_cookies (GList *gone_c, GList *gone_p, GList *gone_i,
-		     gboolean block)
-{
-	nsresult result;
-	gchar *value;
-	gchar *gone_cs, *gone_ps, *gone_is, *block_s;
-
-	if (!gone_c && !gone_p && !gone_i) return TRUE;
-
-	gone_cs = join_numbers (gone_c);
-	gone_ps = join_numbers (gone_p);
-	gone_is = join_numbers (gone_i);
-	if (block)
-		block_s = "true";
-	else 
-		block_s = "false";
-
-	value = g_strconcat ("|goneC|", gone_cs,
-			     "|goneP|", gone_ps,
-			     "|goneI|", gone_is,
-			     "|block|", block_s,
-			     "|", NULL);
-
-	g_free (gone_cs);
-	g_free (gone_ps);
-	g_free (gone_is);
-
-	nsCOMPtr<nsICookieViewer> 
-			cookieviewer = do_CreateInstance (COOKIEVIEWER_ID);
-
-	result = cookieviewer->SetValue (value, NULL);
-
-	g_free (value);
-	return NS_SUCCEEDED(result) ? TRUE : FALSE;
-}
-
-/**
- * split_list: split a list of cookies
- */
-static gchar **
-split_list (gchar *list)
-{
-	gchar *delim;
-	gchar **split;
-
-	delim = g_strndup (list, 1);
-	
-	if (!(*list)) return g_new0 (gchar *, 1);
-
-	split = g_strsplit (list+1, delim, -1);
-
-	g_free (delim);
-
-	return split;
-}
-
-/**
- * join_numbers:
- */
-static gchar*
-join_numbers (GList *cookies)
-{
-	gchar **str_array, *join;
-	GList *elem;
-	int i = 0;
-
-	int len = g_list_length (cookies);
-	str_array = g_new (gchar*, len + 2);
-
-	for (elem = cookies; elem != NULL; elem = g_list_next(elem))
-		str_array[i++] = ((CookieBase*)elem->data)->number;
-	if (cookies)
-		str_array[i++] = "";
-	str_array[i] = NULL;
-
-	join = g_strjoinv (",", str_array);
-
-	g_free (str_array);
-	return join;
-}
-
-#endif //CVS_SOURCE
-
 /**
  * mozilla_copy_session_history: copy the session history to another embed
  */
@@ -858,16 +859,19 @@ mozilla_get_charsets (GHashTable **charsets, GList **sorted_charset_titles)
 
 		if (strlen(charset_title_str) == 0)
 		{
+			g_free (charset_title_str);
 			charset_title_str = g_strdup(charset_str);
 		}
 		
 		for (PRUint32 j = 0; j < translated_cscount; j++)
 		{
-			if (g_strcasecmp (charset_title_str, 
-					  charset_trans_array[j]) == 0)
+			if (g_strcasecmp (
+				charset_title_str, 
+				charset_trans_array[j].charset_title) == 0)
 			{
-				g_free(charset_title_str);
-				charset_title_str = (char*)_(charset_trans_array[j]);
+				g_free (charset_title_str);
+				charset_title_str = (char *) 
+					_(charset_trans_array[j].charset_title);
 				break;
 			}
 		}
@@ -898,20 +902,9 @@ mozilla_force_character_set (GaleonEmbed *embed, char *force_character_set)
 extern "C" void
 mozilla_clear_cache (unsigned long cache)
 {
-	nsresult result;
-
-#ifdef CVS_SOURCE
 	nsCOMPtr<nsICacheService> 
 		CacheService = do_CreateInstance (CACHESERVICE_ID);
 	CacheService->EvictEntries(cache);
-	
-#else
-	nsCOMPtr<nsINetDataCacheManager> CacheManager = 
-		do_GetService(NS_NETWORK_CACHE_MANAGER_CONTRACTID, &result);
-	if (NS_FAILED (result)) return;
-
-	CacheManager->Clear(cache);
-#endif
 }
 
 /* This nonsense is needed to get the allocators right */
@@ -944,8 +937,6 @@ mozilla_get_favicon_location (GaleonEmbed *embed, gchar **url)
 	return NS_SUCCEEDED (result);
 }
 
-#ifdef CVS_SOURCE
-
 /**
  * mozilla_list_passwords: get a list of all saved username/password pairs
  */
@@ -953,7 +944,7 @@ extern "C" GList *
 mozilla_list_passwords (PasswordType type)
 {
 	GList *passwords = NULL;
-	nsresult result;
+	nsresult result = NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIPasswordManager> 
 		passwordManager = do_CreateInstance (PASSWORDMANAGER_ID);
@@ -984,7 +975,7 @@ mozilla_list_passwords (PasswordType type)
 		{
 			PRUnichar *unicodeName;
 			nsPassword->GetUser (&unicodeName);
-			p->username = mozilla_unicode_to_utf8 (unicodeName);
+			p->username = mozilla_unicode_to_locale (unicodeName);
 		}
 		
 		passwords = g_list_append (passwords, p);
@@ -1000,7 +991,7 @@ mozilla_list_passwords (PasswordType type)
 extern "C" gboolean 
 mozilla_remove_passwords (GList *gone, PasswordType type)
 {
-	nsresult result;
+	nsresult result = NS_ERROR_FAILURE;
 	nsCOMPtr<nsIPasswordManager> 
 		passwordManager = do_CreateInstance (PASSWORDMANAGER_ID);
 
@@ -1026,8 +1017,6 @@ mozilla_remove_passwords (GList *gone, PasswordType type)
 	return TRUE;
 }
 
-#endif
-
 /**
  * mozilla_unicode_to_utf8: Encodes unicode string to UTF-8
  * @uniStr: The unicode string to encode
@@ -1037,12 +1026,14 @@ mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
 {
 	PRInt32 sSize,dSize;
 	nsresult result;
-#ifdef CVS_SOURCE
-	const nsLocalString str = nsLiteralString (uniStr);
-	sSize = str.Length ();
+
+#if MOZILLA_VERSION > VERSION2(0,9)
+	const nsString str (uniStr);
 #else
-	for (sSize = 0; uniStr[sSize]!=0 ; sSize++);
-#endif	 
+	const nsLocalString str = nsLiteralString (uniStr);
+#endif
+	sSize = str.Length ();
+
 	nsCOMPtr<nsIUnicodeEncoder> 
 		unicodeEncoder = do_CreateInstance (UTF8_ENCODER_ID);
 
@@ -1063,7 +1054,6 @@ mozilla_unicode_to_utf8 (const PRUnichar *uniStr)
 
 	/* Normally a null would need to appended at this point but
 	   cStr was initialised to zero, so there's no need */
-
 	return utfStr;
 }
 
@@ -1086,12 +1076,12 @@ mozilla_unicode_to_locale (const PRUnichar *uniStr)
 		return NULL;
 	}
 
-#ifdef CVS_SOURCE
-	const nsLocalString str = nsLiteralString (uniStr);
-	sSize = str.Length ();
+#if MOZILLA_VERSION > VERSION2(0,9)
+	const nsString str (uniStr);
 #else
-	for (sSize = 0; uniStr[sSize]!=0 ; sSize++);
-#endif	 
+	const nsLocalString str = nsLiteralString (uniStr);
+#endif
+	sSize = str.Length ();
 
 	/* allocate a wide string big enough to hold the unicode string,
 	 * this is necessary since wchar_t is 32-bits with glibc */
@@ -1162,7 +1152,6 @@ mozilla_locale_to_unicode (const gchar *locStr)
 {
 	PRUnichar *uniStr;
 	wchar_t *wide;
-	gchar *output;
 	gint i, count;
 
 	/* sanity */
@@ -1195,3 +1184,46 @@ mozilla_locale_to_unicode (const gchar *locStr)
 	g_free (wide);
 	return uniStr;
 }
+
+/**
+ * mozilla_find_gtk_parent: retrives gtkwidget pointer to parent
+ * of domWindow. It will be the gtkmozembed that holds it.
+ * @domWindow: nsIDOMWindow to find gtk parent of.
+ */
+extern "C" GtkWidget *
+mozilla_find_gtk_parent (nsIDOMWindow *domWindow)
+{
+	nsresult result;
+	
+        nsCOMPtr<nsIWindowWatcher> wwatch 
+		(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
+        if (!wwatch) return nsnull;
+
+        nsCOMPtr<nsIWebBrowserChrome> windowChrome;
+        result = wwatch->GetChromeForWindow (domWindow,
+        				     getter_AddRefs(windowChrome));
+        if (NS_FAILED(result)) return nsnull;
+
+        nsCOMPtr<nsIEmbeddingSiteWindow> window
+        		(do_QueryInterface(windowChrome, &result));
+        if (NS_FAILED(result)) return nsnull;
+
+        GtkWidget *parentWindow;
+        result = window->GetSiteWindow ((void **)&parentWindow);
+        if (NS_FAILED(result)) return nsnull;
+
+        return parentWindow;
+}
+
+extern "C" gboolean
+mozilla_set_offline (GaleonEmbed *embed, gboolean offline)
+{
+	GaleonWrapper *wrapper = (GaleonWrapper *)embed->wrapper;
+	nsresult result;
+
+	g_return_val_if_fail (embed->wrapper != NULL, FALSE); 
+
+	result = wrapper->SetOffline (offline);
+
+	return NS_SUCCEEDED (result) ? TRUE : FALSE;
+}
