@@ -22,7 +22,15 @@
  * Netscape bookmark exporting by Ricardo Fernndez Pascual <ric@users.sourceforge.net>
  */
 
-#include <galeon.h>
+#include "galeon.h"
+#include "bookmarks.h"
+#include "misc.h"
+
+#include <stdlib.h>
+#include <string.h>
+#include <libgnomeui/gnome-dialog-util.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
 
 /* local data types */
 
@@ -116,9 +124,7 @@ netscape_import_bookmarks (GString *filename)
 					? TRUE : FALSE;
 				parent->list = g_list_insert (parent->list, newcat, -1);
 				newcat->parent = parent;
-				if (bookmarks_editor)
-					bookmarks_editor_place_tree_item 
-						(bookmarks_editor, newcat);
+				bookmarks_editor_place_tree_items (newcat);
 			}
 			
 			parent = newcat;
@@ -144,9 +150,7 @@ netscape_import_bookmarks (GString *filename)
 			g_free (parsedname);
 			newsite->parent = parent;
 			parent->list = g_list_insert(parent->list, newsite, -1);
-			if (bookmarks_editor)
-				bookmarks_editor_place_tree_item 
-					(bookmarks_editor, newsite);
+			bookmarks_editor_place_tree_items (newsite);
 			break;
 		case NS_SEPARATOR:
 			/* how to check if a separator is duplicated? */
@@ -154,9 +158,7 @@ netscape_import_bookmarks (GString *filename)
 				(BM_SEPARATOR, TRUE, NULL, NULL, NULL, NULL, NULL);
 			newsite->parent = parent;
 			parent->list = g_list_insert(parent->list, newsite, -1);
-			if (bookmarks_editor)
-				bookmarks_editor_place_tree_item 
-					(bookmarks_editor, newsite);
+			bookmarks_editor_place_tree_items (newsite);
 			break;
 		case NS_FOLDER_END:
 			if (parent != bookmarks_root)
@@ -172,9 +174,7 @@ netscape_import_bookmarks (GString *filename)
 	g_slist_free(urls);
 	g_slist_free(folders);
 		
-	if (bookmarks_editor)
-		bookmarks_editor->dirty = TRUE;
-	bookmarks_save ();
+	bookmarks_updated ();
 
 	return 0;	
 }
@@ -342,6 +342,12 @@ netscape_export_bookmarks_item (FILE *file, BookmarkItem *b)
 		fputs ("\t<DT><A HREF=\"", file);
 		fputs (b->url, file);
 		fputs ("\"", file);
+		if (b->time_added > 0)
+			fprintf (file, " ADD_DATE=\"%d\"", b->time_added);
+		if (b->time_modified > 0) 
+			fprintf (file, " LAST_MODIFIED=\"%d\"", b->time_modified);
+		if (b->time_visited > 0)
+			fprintf (file, " LAST_VISIT=\"%d\"", b->time_visited);
 		/* support a mozilla-only feature, but won't harm netscape */
 		if (strlen (b->nick) > 0) {
 			fputs (" SHORTCUTURL=\"", file);
@@ -353,13 +359,24 @@ netscape_export_bookmarks_item (FILE *file, BookmarkItem *b)
 		fputs (strippedname, file);
 		g_free(strippedname);
 		fputs ("</A>\n", file);
+		if (strlen (b->notes) > 0) {
+			fputs ("<DD>", file);
+			fputs (b->notes, file);
+			fputs ("\n", file);
+		}
 		break;
 	case BM_CATEGORY:
 		fputs ("<DT><H3 ADD_DATE=\"0\">", file); 
 		strippedname = strip_uline_accel (b->name);
 		fputs (strippedname, file);
 		g_free(strippedname);
-		fputs ("</H3>\n<DL><p>\n", file);
+		fputs ("</H3>\n", file);
+		if (strlen (b->notes) > 0) {
+			fputs ("<DD>", file);
+			fputs (b->notes, file);
+			fputs ("\n", file);
+		}
+		fputs ("<DL><p>\n", file);
 		/* FIXME: Alias catogories are not exported */
 		if (!b->alias_of)
 			for (l = b->list; l != NULL; l = l->next) 
