@@ -17,6 +17,19 @@
  */
 
 #include "galeon.h"
+#include "misc.h"
+
+#include <sys/stat.h>
+#include <gtk/gtktogglebutton.h>
+#include <libgnome/gnome-util.h>
+#include <libgnome/gnome-config.h>
+
+#if ENABLE_NLS && HAVE__NL_PAPER_WIDTH_HEIGHT
+#include <langinfo.h>
+
+static gint locale_paper ();
+#endif
+
 
 /**
  * newuser_check: check if this is the first time the user
@@ -38,6 +51,15 @@ newuser_check (void)
 		return FALSE;
 	}
 
+	/* set download dir */
+	gnome_config_set_string ("/galeon/Downloading/download_dir",
+				 g_get_home_dir ());
+
+	/* set default paper type */
+#if ENABLE_NLS && HAVE__NL_PAPER_WIDTH_HEIGHT
+	gnome_config_set_int ("/galeon/Print/paper", locale_paper());
+#endif
+
 	/* never show again! */
 	gnome_config_set_int ("/galeon/General/new_user", 0);
 
@@ -63,7 +85,7 @@ newuser_check (void)
 	}
 
 	/* build the druid */
-	gxml = glade_widget_new ("new_user_druid", NULL, NULL);
+	gxml = glade_widget_new ("galeon.glade", "new_user_druid", NULL, NULL);
 
 	/* lookup some widgets */
 	import_netscape_bookmarks = 
@@ -114,3 +136,21 @@ newuser_check (void)
 	/* indicate to main() not to open an initial window */
 	return TRUE;
 }
+
+#if ENABLE_NLS && HAVE__NL_PAPER_WIDTH_HEIGHT
+static gint 
+locale_paper ()
+{
+	int height;
+	int width;
+
+	height = (int) (long int) nl_langinfo (_NL_PAPER_HEIGHT);
+	width = (int) (long int) nl_langinfo (_NL_PAPER_WIDTH);
+
+	if (height == 297 && width == 210)
+	{
+		return 3;
+	}
+	else return 0;
+}
+#endif
