@@ -17,6 +17,16 @@
  */
 
 #include "galeon.h"
+#include "misc.h"
+#include "embed.h"
+#include "bookmarks.h"
+
+#include <string.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktable.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
 
 /* local function prototypes */
 void new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy);
@@ -175,7 +185,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		/* add each smart bookmark checked */
 		if (google_web_checked)
 		{
-			add_smart (toolbar, "Web", "google",
+			add_smart (toolbar, "Google", "google",
 				   "http://www.google.com/search?q=%s",
 				   SHARE_DIR "/google.png");
 		}
@@ -207,9 +217,13 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 		}
 		if (merriam_websters_checked)
 		{
-			add_smart (toolbar, "Dictionary", "dict",
-				   "http://www.m-w.com/cgi-bin/"
-				   "dictionary?va=%s", NULL);
+			/* URL marked as translatable so that non-English
+			 * dictionaries can be used -- don't forget to
+			 * make the equivalent change in the glade file */
+			add_smart (toolbar, _("Dictionary"), "dict",
+				   /* xgettext: no-c-format */
+				   _("http://www.m-w.com/cgi-bin/"
+				     "dictionary?va=%s"), NULL);
 		}
 		if (freshmeat_checked)
 		{
