@@ -16,16 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifndef __print_h
+#define __print_h
+
 typedef struct
 {
 	gboolean print_to_file;
 	gchar *printer;
 	gchar *file;
 	gint paper;
-	gfloat top_margin;
-	gfloat bottom_margin;
-	gfloat left_margin;
-	gfloat right_margin;
+	gdouble top_margin;
+	gdouble bottom_margin;
+	gdouble left_margin;
+	gdouble right_margin;
 	gint pages;
 	gint from_page;
 	gint to_page;
@@ -34,3 +37,5 @@ typedef struct
 PrintInfo;
 
 void print_show_dialog (GaleonWindow *window);
+
+#endif /* __print_h */
