@@ -17,6 +17,18 @@
  */
 
 #include "galeon.h"
+#include "bookmarks.h"
+#include "session.h"
+#include "embed.h"
+#include "misc.h"
+#include "window.h"
+
+#include <gtkmozembed.h>
+#include <string.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-config.h>
+#include <libgnomeui/gnome-uidefs.h>
+#include <libgnomeui/gnome-dialog-util.h>
 
 /*
  *  Session code 
@@ -143,6 +155,7 @@ session_autoresume (void)
 	/* abort if no windows in session */
 	if (session->root->childs == NULL)
 	{
+		xmlFreeDoc (session);
 		return FALSE;
 	}
 
@@ -254,7 +267,11 @@ session_save (gchar *filename)
 	{
 		/* get this item */
 		GaleonWindow *window = (GaleonWindow *)(w->data);
-		return_if_not_window (window);
+		if (window->magic != GALEON_WINDOW_MAGIC)
+		{
+			g_warning ("not a valid galeon window");
+			continue;
+		}
 
 		/* skip if for some reason it doesn't contain any embeds */
 		if (window->embed_list == NULL)
@@ -283,7 +300,8 @@ session_save (gchar *filename)
 		xmlSetProp (window_node, "height", buffer);
 
 		/* iterate through all the embeds */
-		for (e = window->embed_list; e != NULL; e = g_list_next (e))
+		for (e = g_list_last (window->embed_list);
+		     e != NULL; e = g_list_previous (e))
 		{
 			/* get this item */
 			GaleonEmbed *embed = (GaleonEmbed *)(e->data);
@@ -320,7 +338,7 @@ session_save (gchar *filename)
  * session_load: load a session into a new XML document
  */
 static xmlDocPtr
-session_load(gchar *filename)
+session_load (gchar *filename)
 {
 	xmlDocPtr doc;
 
@@ -342,11 +360,15 @@ static void
 session_resume(xmlDocPtr doc, gboolean override_behaviour)
 {
 	xmlNodePtr i;
+	gint behaviour;
 	gchar *url = NULL;
 	gchar *title = NULL;
-	gint behaviour;
 	BookmarkItem *cat = NULL; 
-	GaleonEmbed *last;
+	gint x = 0, y = 0, width, height;
+	GdkWindow *gdk_window;
+	GaleonWindow *window = NULL;
+	GaleonEmbed *embed;
+	gint embed_count = 0;
 
 	/* so we don't open saved sessions into temporary bookmarks) */
 	if (override_behaviour)
@@ -360,9 +382,9 @@ session_resume(xmlDocPtr doc, gboolean override_behaviour)
 	{
 		bookmarks_editor_show_dialog (NULL, 
 					      COMPACT_BOOKMARKS_EDITOR);
-		cat = add_temp_bookmark (BM_CATEGORY, 
-					 _("URLs opened before crashing"),
-	                                 NULL, NULL);
+		cat = add_bookmark_default (BM_CATEGORY, 
+					    _("URLs opened before crashing"),
+	                                    NULL, NULL);
 	}
 
 	/* iterate over session windows */
@@ -370,13 +392,29 @@ session_resume(xmlDocPtr doc, gboolean override_behaviour)
 	{
 		xmlNodePtr j;
 
-		/* don't put in same window */
-		last = NULL;
+		if (i->childs == NULL)
+		{
+			g_warning ("window with no GaleonEmbeds");
+			continue;
+		}
 
-		/* Proper initialization must occur first */
-		gtk_moz_embed_push_startup ();
-		mozilla_init_promptService ();
+		if (behaviour != 1)
+		{
+			/* create a default window */
+			window = window_create (GTK_MOZ_EMBED_FLAG_ALLCHROME);
 
+			/* get the size of the window */
+			x      = xmlGetIntProp (i, "x");
+			y      = xmlGetIntProp (i, "y");
+			width  = xmlGetIntProp (i, "width");
+			height = xmlGetIntProp (i, "height");
+			
+			/* set the size of the window */
+			gtk_window_set_default_size 
+				(GTK_WINDOW (window->WMain), width, height);
+			window->set_size = TRUE;
+		}
+		
 		/* iterate over session embeds */
 		for (j = i->childs; j != NULL; j = j->next)
 		{
@@ -387,26 +425,41 @@ session_resume(xmlDocPtr doc, gboolean override_behaviour)
 			if (behaviour == 1)
 			{
 				/* add the bookmark */
-				add_temp_bookmark (BM_SITE, title, url, cat);
+				add_bookmark_default (BM_SITE, title, 
+						      url, cat);
 			}
 			else
 			{
 				/* open a browser pointing to this URL */
-				last = embed_create_from_url (last, url,
-							      last == NULL);
+				embed = embed_create_in_window 
+					(window, GTK_MOZ_EMBED_FLAG_ALLCHROME);
+				embed_set_visibility (embed, TRUE);
+				embed_load_url (embed, url);
+				embed_count++;
 			}
+
+			/* free allocated strings */
+			if (url != NULL) xmlFree (url);
+			if (title != NULL) xmlFree (title);
 		}
 
-		/* free allocated strings */
-		if (url != NULL) xmlFree (url);
-		if (title != NULL) xmlFree (title);
+		if (behaviour != 1)
+		{
+			gdk_window = GTK_WIDGET (window->WMain)->window;
+			gdk_window_move (gdk_window, x, y);
+		}
 	}
 
 	/* Add an empty window bookmark to the temp bookmarks window, just
 	   in case all the saved bookmarks are crashers */
 	if (behaviour == 1)
 	{
-		add_temp_bookmark (BM_SITE, N_("Empty browser window"),
-				   "about:blank", cat);
+		add_bookmark_default (BM_SITE, N_("Empty browser window"),
+				      "about:blank", cat);
+	} 
+	else if (embed_count == 0)
+	{
+		g_warning ("having to create default window");
+		embed_create_default (NULL, FALSE);
 	}
 }
