@@ -16,8 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Galeon includes */
 #include "galeon.h"
+#include "spinner.h"
+#include "misc.h"
 
+#include <string.h>
+#include <gtk/gtkbox.h>
+
+/* GNOME includes */
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-animator.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
+
+/* local function prototypes */
 static void free_spinner_list (void);
 static void spinner_fill_iconlist_from_dir (GnomeIconList *icon_list,
 					    gchar *base);
@@ -25,8 +39,8 @@ static void spinner_fill_iconlist_from_dir (GnomeIconList *icon_list,
 /**
  * spinner_create: create a spinner and add it to an hbox
  */
-GtkWidget *
-spinner_create (GtkWidget *hbox)
+void
+spinner_create (GaleonWindow *window)
 {
 	GtkWidget *spinner_button;
 	GtkWidget *spinner;
@@ -36,6 +50,12 @@ spinner_create (GtkWidget *hbox)
 	GdkImlibImage *first;
 	gint width, height;
 
+	/* check window */
+	return_if_not_window (window);
+
+	/* initially no spinner */
+	window->spinner = NULL;
+
 	/* get the directory of the animations from configuration space,
  	 * default to installed share directory */
 	anim_dir = gnome_config_get_string("/galeon/Appearance/spinner_dir="
@@ -53,7 +73,7 @@ spinner_create (GtkWidget *hbox)
 		g_warning ("unable to load first spinner frame: %s\n",
 			   filename);
 		g_free (filename);
-		return NULL;
+		return;
 	}
 	width = first->rgb_width;
 	height = first->rgb_height;
@@ -116,7 +136,7 @@ spinner_create (GtkWidget *hbox)
 			{
 				g_warning("no frames of animation found in "
 					  "directory `%s'", anim_dir);
-				return NULL;
+				return;
 			}
 
 			/* jump out of loading */
@@ -136,17 +156,37 @@ spinner_create (GtkWidget *hbox)
 	}
 
 	/* add it to the hbox */
-	gtk_box_pack_end (GTK_BOX(hbox), spinner_button, FALSE, FALSE, 1);
+	g_return_if_fail (GTK_IS_WIDGET (window->main_toolbar));
+	gtk_box_pack_end (GTK_BOX (window->main_toolbar->parent), 
+			  GTK_WIDGET (spinner_button), FALSE, FALSE, 0);
+
+	/* attach click signal */
+	gtk_signal_connect (GTK_OBJECT (spinner_button), "clicked",
+			    window_spinner_button_clicked_cb, window);
 
 	/* show the spinner */
-	gtk_widget_show(spinner_button);
-	gtk_widget_show(spinner);
+	gtk_widget_show (spinner_button);
+	gtk_widget_show (spinner);
 
 	/* free directory string */
-	g_free(anim_dir);
+	g_free (anim_dir);
 
-	/* return the spinner */
-	return spinner;
+	/* set in window */
+	window->spinner = spinner;
+}
+
+/**
+ * spinner_destroy: destroy the spinner of a window
+ */
+void
+spinner_destroy (GaleonWindow *window)
+{
+	if (window->spinner != NULL)
+	{
+		gnome_animator_stop (GNOME_ANIMATOR (window->spinner));
+		gtk_widget_destroy (window->spinner->parent);
+		window->spinner = NULL;
+	}
 }
 
 /**
@@ -232,13 +272,11 @@ spinner_fill_iconlist_from_dir (GnomeIconList *icon_list, gchar *base)
 		gnome_config_get_string("/galeon/Appearance/spinner_dir="
 					SHARE_DIR "/spinners/pipeon/");	
 
-	rc = gnome_vfs_directory_list_load(&list,
-			base,
-			(GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
-			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
-			NULL);
-
+	rc = gnome_vfs_directory_list_load
+		(&list, base, (GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+			       GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
+			       GNOME_VFS_FILE_INFO_FOLLOW_LINKS), NULL);
+	
 	index = icon_list->icons;
 	for (node = list; node; node = node->next)
 	{
@@ -266,15 +304,13 @@ spinner_fill_iconlist_from_dir (GnomeIconList *icon_list, gchar *base)
 
 		/* Check to see if there is a rest.png */
 		icon = g_concat_dir_and_file (path, "rest.png");
-
 		if (g_file_exists (icon))
 		{
 			spinner_list = g_list_append (spinner_list, 
 						      g_strdup (path));
 
 			/* Name defaults to the directory name */
-			gnome_icon_list_append (icon_list, icon, 
-						g_strdup (info->name));
+			gnome_icon_list_append (icon_list, icon, info->name);
 
 			/* Select the icon configured in prefs */
 			if (strcmp (pref_spinner_path, path) == 0)
@@ -295,11 +331,12 @@ spinner_fill_iconlist_from_dir (GnomeIconList *icon_list, gchar *base)
  * spinner_get_path_from_index: used in prefs_callbacks.c to get the 
  * path of selected icon
  */
-gchar *
-spinner_get_path_from_index (int index)
+const gchar *
+spinner_get_path_from_index (gint index)
 {
 	gchar *path;
-	path = g_list_nth_data(spinner_list, index);
+
+	path = g_list_nth_data (spinner_list, index);
 
 	return path;
 }
