@@ -16,7 +16,33 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* Galeon includes */
 #include "galeon.h"
+#include "toolbar.h"
+#include "spinner.h"
+#include "misc.h"
+#include "window.h"
+#include "bookmarks.h"
+
+#include <string.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktoolbar.h>
+#include <gtk/gtkdnd.h>
+#include <libgnome/gnome-util.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-preferences.h>
+#include <libgnomeui/gnome-dock.h>
+#include <libgnomeui/gnome-entry.h>
+
+/* GNOME includes */
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
+
+extern const GtkTargetEntry drop_types [];
+extern const GtkTargetEntry url_drag_types [];
 
 /* the type of toolbar button definitions */
 typedef struct
@@ -60,17 +86,6 @@ static gchar *theme_path;
 static gchar *theme_name;
 static gboolean use_nautilus_theme;
 
-/* FIXME: this stuff is duplicated several times in Galeon */
-/* gtkmozembed type of drag and drop */
-static const GtkTargetEntry drop_types[] =
-{
-	{ "GALEON_URL",    0, DND_TARGET_GALEON_URL   },
-	{ "_NETSCAPE_URL", 0, DND_TARGET_NETSCAPE_URL },
-	{ "STRING",        0, DND_TARGET_STRING       }
-};
-static const gint drop_types_num_items = (sizeof (drop_types) / 
-					  sizeof (GtkTargetEntry));
-
 /* path to Nautilus theme pref in config space */
 #define NAUTILUS_THEME_PREF "/apps/nautilus/preferences/theme"
 
@@ -98,7 +113,7 @@ const static ToolbarButton back_button =
 const static ToolbarButton up_button =
 {
 	N_("Up"), 
-	N_("Go up a directory level"), 
+	N_("Go up a level from the current location"), 
 	GNOME_STOCK_PIXMAP_UP, 
 	"Up",
 	window_up_button_clicked_cb,		
@@ -131,8 +146,8 @@ const static ToolbarButton home_button =
 	N_("Go to your home page"), 
 	GNOME_STOCK_PIXMAP_HOME, 
 	"Home",
-	window_start_button_clicked_cb,
-	window_start_button_press_event_cb,
+	window_home_button_clicked_cb,
+	window_home_button_press_event_cb,
 	NULL /* drag_data_received */
 };
 const static ToolbarButton stop_button =
@@ -152,7 +167,7 @@ const static ToolbarButton go_button =
 	GNOME_STOCK_PIXMAP_JUMP_TO, 
 	"Go",
 	window_go_button_clicked_cb,
-	NULL /* press_event */,
+	window_go_button_press_cb,
 	NULL /* drag_data_received */
 };
 
@@ -170,27 +185,25 @@ toolbar_create (GaleonWindow *window)
 	/* get theme infos */
 	if (use_theme_pref)
 	{
-		theme_path = toolbar_get_theme_path ();
-		if (theme_path)
+		if ((theme_path = toolbar_get_theme_path ()));
 		{
 			use_nautilus_theme = !toolbar_theme_use_standard_icons();
 		}
 	}
 
 	/* create main toolbar */
-	window->main_dockitem = toolbar_add_docked
-					(window,
-					 toolbar_create_main (window),
-					 GNOME_APP_TOOLBAR_NAME, 1);
+	window->main_dockitem = 
+		toolbar_add_docked (window, toolbar_create_main (window),
+				    GNOME_APP_TOOLBAR_NAME, 1);
 
 	/* create location toolbar, if necessary */
 	if (gnome_config_get_int ("/galeon/Appearance/url_location=1") == 0)
 	{
 		GtkWidget *location_tb = toolbar_create_location (window);
 
-		window->location_dockitem = toolbar_add_docked (window,
-								location_tb,
-								"location", 2);
+		window->location_dockitem = 
+			toolbar_add_docked (window, location_tb, 
+					    "location", 2);
 	}
 
 	/* set the visibility of the optional components */
@@ -204,33 +217,62 @@ toolbar_create (GaleonWindow *window)
 	}
 }
 
+/**
+ * toolbar_recreate: recreate a single toolbar
+ */
+void
+toolbar_recreate (GaleonWindow *window)
+{
+	/* FIXME: why is the necessary */
+	window_clear_url_entry (window);
+
+	/* destroy spinner, this prevents crashes with timeouts */
+	spinner_destroy (window);
+
+	/* destroy main toolbar */
+	gtk_widget_destroy (window->main_dockitem->parent);
+	window->main_dockitem = NULL;
+
+	/* destroy location toolbar */
+	if (window->location_dockitem != NULL)
+	{
+		gtk_widget_destroy (window->location_dockitem->parent);
+		window->location_dockitem = NULL;
+	}
+
+	/* create new toolbar */
+	toolbar_create (window);
+	
+	/* show new toolbars */
+	gtk_widget_show (window->main_dockitem);
+	if (window->location_dockitem != NULL)
+	{
+		gtk_widget_show (window->location_dockitem);
+	}
+
+	/* set the controls properly for this window */
+	window_update_nav_controls (window);
+}
+
 /**
  * toolbar_add_docked: add an horizontal box to the toolbar dock
  */
-GtkWidget *
-toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox, char *name,
-		    int band_num)
+static GtkWidget *
+toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox, 
+		    gchar *name, gint band_number)
 {
 	gboolean toolbar_relief;
 	gboolean toolbar_detachable;
 	GnomeDockItemBehavior props;
 	gboolean url_location;
 
-
 	url_location = gnome_config_get_int 
 		("/galeon/Appearance/url_location=1");
 	toolbar_relief = gnome_preferences_get_toolbar_relief ();
 	toolbar_detachable = gnome_preferences_get_toolbar_detachable ();
 
-	props = GNOME_DOCK_ITEM_BEH_EXCLUSIVE;
-
-/* FIXME: use this code once the toolbar is packed properly
-	if (url_location == 1)
-	{
-		props |= GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL;
-	}
-*/
-	props |= GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL;
+	props = (GNOME_DOCK_ITEM_BEH_EXCLUSIVE |
+		 GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL);
 
 	if (!toolbar_detachable)
 	{
@@ -238,11 +280,13 @@ toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox, char *name,
 	}
 
 	gnome_app_add_docked (GNOME_APP (window->WMain), hbox, name,
-			      props, GNOME_DOCK_TOP, band_num, 0, 0);
-	gtk_container_set_border_width (GTK_CONTAINER(hbox->parent), 2);
+			      props, GNOME_DOCK_TOP, band_number, 0, 0);
+	gtk_container_set_border_width (GTK_CONTAINER(hbox->parent),
+					toolbar_relief ? 2 : 0);
 	gnome_dock_item_set_shadow_type 
 		(GNOME_DOCK_ITEM (hbox->parent), 
 		 toolbar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
+
 	gtk_widget_show (hbox);
 
 	return hbox->parent;
@@ -254,7 +298,6 @@ toolbar_add_docked (GaleonWindow *window, GtkWidget *hbox, char *name,
 static void
 toolbar_update (GaleonWindow *window)
 {
-	GtkWidget *main_toolbar;
 	GtkToolbarStyle style;
 	gboolean toolbar_button_relief;
 
@@ -263,24 +306,30 @@ toolbar_update (GaleonWindow *window)
 	toolbar_button_relief = gnome_preferences_get_toolbar_relief_btn ();
 	style = gnome_config_get_int ("toolbar_type=0");
 
-	main_toolbar = gtk_object_get_data (GTK_OBJECT (window->WMain),
-					    "main_toolbar");
-	if (main_toolbar != NULL)
+	if (!(window->toolBarOn))
 	{
-		gtk_toolbar_set_style (GTK_TOOLBAR (main_toolbar),
-				       style);
-		gtk_toolbar_set_button_relief (GTK_TOOLBAR (main_toolbar),
-					       toolbar_button_relief ? 
-							GTK_RELIEF_NORMAL :
-							GTK_RELIEF_NONE);
+		return;
 	}
 
-	gtk_toolbar_set_style (GTK_TOOLBAR (window->go_toolbar),
-			       style);
-	gtk_toolbar_set_button_relief (GTK_TOOLBAR (window->go_toolbar),
-				       toolbar_button_relief ?
-						GTK_RELIEF_NORMAL :
-						GTK_RELIEF_NONE);
+	if (window->main_toolbar != NULL)
+	{
+		gtk_toolbar_set_style 
+			(GTK_TOOLBAR (window->main_toolbar), style);
+		gtk_toolbar_set_button_relief 
+			(GTK_TOOLBAR (window->main_toolbar),
+			 toolbar_button_relief ? 
+			 GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
+	}
+
+	if (window->go_toolbar != NULL)
+	{
+		gtk_toolbar_set_style 
+			(GTK_TOOLBAR (window->go_toolbar), style);
+		gtk_toolbar_set_button_relief 
+			(GTK_TOOLBAR (window->go_toolbar),
+			 toolbar_button_relief ?
+			 GTK_RELIEF_NORMAL : GTK_RELIEF_NONE);
+	}
 
 	if (gnome_config_get_bool ("show_new_button=FALSE"))
 		gtk_widget_show (window->BNew);
@@ -292,10 +341,20 @@ toolbar_update (GaleonWindow *window)
 	else
 		gtk_widget_hide (window->BUp);
 
+	if (gnome_config_get_bool ("show_home_button=TRUE"))
+		gtk_widget_show (window->BStart);
+	else
+		gtk_widget_hide (window->BStart);
+	
 	if (gnome_config_get_bool ("show_go_button=FALSE"))
 		gtk_widget_show (window->go_toolbar);
 	else
 		gtk_widget_hide (window->go_toolbar);
+
+	if (gnome_config_get_bool ("show_drag_event_box=TRUE"))
+		gtk_widget_show (window->drag_event_box);
+	else
+		gtk_widget_hide (window->drag_event_box);
 		
 	gnome_config_pop_prefix ();
 		
@@ -308,134 +367,45 @@ toolbar_update (GaleonWindow *window)
 	/* create/destroy the spinner */
 	if (gnome_config_get_bool ("/galeon/General/show_spinner=true"))
 	{
-		if (window->spinner != NULL)
-		{
-			gnome_animator_stop (GNOME_ANIMATOR (window->spinner));
-			gtk_widget_destroy (window->spinner->parent);
-		}
-		window->spinner = spinner_create (main_toolbar->parent);
-		if (window->spinner != NULL)
-		{
-			gtk_signal_connect
-					(GTK_OBJECT (window->spinner->parent),
-					 "clicked",
-					 window_spinner_button_clicked_cb,
-					 (gpointer) window);
-		}
+		spinner_destroy (window);
+		spinner_create (window);
 	}
 	else
 	{
-		if (window->spinner != NULL)
-		{
-			gnome_animator_stop (GNOME_ANIMATOR (window->spinner));
-			gtk_widget_destroy (window->spinner->parent);
-			window->spinner = NULL;
-		}
+		spinner_destroy (window);
 	}
 }
 
-/**
- * toolbar_global_recreate: recreate toolbars of all galeon windows
- */
-void
-toolbar_global_recreate (void)
-{
-	GList *l;
-	GaleonWindow *window = (GaleonWindow *) all_windows->data;
-	gint url_location = gnome_config_get_int
-					("/galeon/Appearance/url_location=1");
-	gboolean recreate = FALSE;
-
-	/*  FIXME: The layout still gets lost when you switch between
-	 *	   having the location entry on the main toolbar or on
-	 *	   a separate toolbar.  This is probably not a big deal,
-	 *	   as I doubt this setting is changed very often   --Josh
-	 */
-
-	/* Check the first window in the list to see if we actually need to
-	   recreate the toolbars */
-	if ( (url_location == 0 && window->location_dockitem == NULL) ||
-	     (url_location == 1 && window->location_dockitem != NULL) )
-		recreate = TRUE;
-
-	if (recreate)
-		bookmarks_remove_tb_global ();
-
-	for (l = all_windows; l != NULL; l = l->next)
-	{
-		window = (GaleonWindow *) l->data;
-
-		if (recreate)
-		{
-			/* recreate the toolbars */
-			gtk_editable_delete_text
-					(GTK_EDITABLE (window->toolbar_entry),
-					 0, -1);
-			gtk_widget_destroy (window->main_dockitem->parent);
-			if (window->location_dockitem)
-			{
-				gtk_widget_destroy
-					(window->location_dockitem->parent);
-				window->location_dockitem = NULL;
-			}
-
-			toolbar_create (window);
-
-			gtk_widget_show (window->main_dockitem);
-			if (window->location_dockitem)
-				gtk_widget_show (window->location_dockitem);
-		}
-		else
-		{
-			/* otherwise, just update the existing toolbars */
-			toolbar_update (window);
-		}
-
-		window_update_nav_buttons (window);
-	}
-
-	if (recreate)
-		bookmarks_create_tb_global ();
-
-	/* We need to recreate the menus in case the location of the
-	   extra menu options has changed */
-	bookmarks_update_menu ();
-
-	/* Update the smart bookmarks toolbar entry widgets */
-	bookmarks_tb_update_entries_global ();
-}
-
 /**
  * toolbar_create_main: create main toolbar (buttons)
  */
-GtkWidget*
+static GtkWidget *
 toolbar_create_main (GaleonWindow *window)
 {
+	gint url_location;
+	gboolean show_spinner;
 	GtkWidget *hbox, *toolbar;
 	GtkAdjustment *adj;
-	int url_location;
 
-	gnome_config_push_prefix ("/galeon/Appearance/");
-	url_location = gnome_config_get_int ("url_location=1");
-	gnome_config_pop_prefix ();
+	/* read prefs */
+	url_location = gnome_config_get_int (CONF_APPEARANCE_URL_LOCATION);
+	show_spinner = gnome_config_get_bool (CONF_GENERAL_SHOW_SPINNER);
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	/* build the overall box to pack the toolbar into */
+	hbox = gtk_hbox_new (FALSE, 2);
 
 	/* create buttons toolbar */
-	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, 
-				   GTK_TOOLBAR_BOTH);
-	/* the lazy way */
-	gtk_object_set_data (GTK_OBJECT (window->WMain), "main_toolbar",
-			     toolbar);
+	window->main_toolbar = toolbar =
+		gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 
 	/* pack it in */
-	gtk_box_pack_start (GTK_BOX(hbox), toolbar, FALSE, FALSE, 1);
+	gtk_box_pack_start (GTK_BOX (hbox), toolbar, FALSE, FALSE, 0);
 
 	/* create all of the buttons */
 	window->BNew     = toolbar_button (window, toolbar, &new_button);
 	window->BBack    = toolbar_button (window, toolbar, &back_button);
-	window->BUp      = toolbar_button (window, toolbar, &up_button);
 	window->BForward = toolbar_button (window, toolbar, &forward_button);
+	window->BUp      = toolbar_button (window, toolbar, &up_button);
 	window->BRefresh = toolbar_button (window, toolbar, &reload_button);
 	window->BStart   = toolbar_button (window, toolbar, &home_button);
 	window->BStop    = toolbar_button (window, toolbar, &stop_button);
@@ -443,73 +413,59 @@ toolbar_create_main (GaleonWindow *window)
 	/* create zoom spin entry */
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (100, 1, 999, 10, 25, 10));
 	window->zoom_spin = GTK_WIDGET (gtk_spin_button_new (adj, 1, 0));
+	gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (window->zoom_spin),
+					 GTK_SHADOW_NONE);
 	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
 				    GTK_TOOLBAR_CHILD_WIDGET,
 				    GTK_WIDGET (window->zoom_spin),
 				    NULL, NULL, NULL, NULL, NULL, NULL);
-	gtk_signal_connect (GTK_OBJECT (window->zoom_spin), "changed",
+	gtk_signal_connect (GTK_OBJECT (adj), "value_changed", 
 			    window_zoom_spin_changed_cb, window);
 	gtk_signal_connect (GTK_OBJECT (window->zoom_spin), "key_press_event",
 			    (GtkSignalFunc) window_zoom_spin_key_press_cb, 
 			    window);
 
-	/* create the spinner, if necessary */
-	if (gnome_config_get_bool ("/galeon/General/show_spinner=true"))
-	{
-		window->spinner = spinner_create (hbox);
-
-		if (window->spinner != NULL)
-		{
-			gtk_signal_connect
-			    (GTK_OBJECT (window->spinner->parent), "clicked",
-			     (GtkSignalFunc) window_spinner_button_clicked_cb,
-			     (gpointer) window);
-		}
-	}
-	else
-	{
-			window->spinner = NULL;
-	}
-
 	/* create location related widgets if needed */
 	if (url_location == 1)
 	{
 		toolbar_create_location_widgets (window, hbox, toolbar);
 	}
 
-	/* show the box and toolbar */
-	gtk_widget_show (toolbar);
-	gtk_widget_show (hbox);
+	/* spinner is created later by toolbar_update */
+	window->spinner = NULL;
 
+	/* show the box */
+	gtk_widget_show_all (hbox);
 	return hbox;
 }
 
-void
+static void
 toolbar_add_drag_event_box (GaleonWindow *window, GtkWidget *toolbar, 
 			    GtkWidget *widget)
 {
-	GtkWidget *event_box = gtk_event_box_new ();
+	window->drag_event_box = gtk_event_box_new ();
 
-	gtk_container_add (GTK_CONTAINER(event_box), widget);
-	gtk_container_set_border_width (GTK_CONTAINER(event_box), 3);
+	gtk_container_add (GTK_CONTAINER(window->drag_event_box), widget);
+	gtk_container_set_border_width (GTK_CONTAINER(window->drag_event_box), 4);
 	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
 				    GTK_TOOLBAR_CHILD_WIDGET,
-				    GTK_WIDGET (event_box),
+				    GTK_WIDGET (window->drag_event_box),
 				    NULL, _("Drag page location from here"),
 				    NULL, NULL, NULL, NULL);
-	gtk_signal_connect (GTK_OBJECT(event_box), "drag_data_get",
+	gtk_signal_connect (GTK_OBJECT(window->drag_event_box), 
+			    "drag_data_get", 
 			    window_drag_pixmap_drag_data_get_cb, window);
-	gtk_drag_source_set (GTK_WIDGET (event_box),
+	gtk_drag_source_set (GTK_WIDGET (window->drag_event_box),
 			     GDK_BUTTON1_MASK, url_drag_types, 
 			     url_drag_types_num_items,
 			     GDK_ACTION_COPY|GDK_ACTION_LINK);
-	gtk_widget_show_all (event_box);
+	gtk_widget_show_all (window->drag_event_box);
 }
 
 /*
  * toolbar_create_location_widgets: create location related widgets
  */
-void
+static void
 toolbar_create_location_widgets (GaleonWindow *window, 
 				 GtkWidget *hbox, GtkWidget *toolbar) 
 {
@@ -521,12 +477,20 @@ toolbar_create_location_widgets (GaleonWindow *window,
 	toolbar_button_relief = gnome_preferences_get_toolbar_relief_btn ();
 
 	drag_pixmap = pixmap_data_new_from_file (SHARE_DIR "/i-bookmark.xpm");
-	window->drag_pixmap = gtk_pixmap_new (drag_pixmap->pixmap,
-					      drag_pixmap->mask);
+	if (drag_pixmap != NULL)
+	{
+		window->drag_pixmap = gtk_pixmap_new (drag_pixmap->pixmap,
+						      drag_pixmap->mask);
+		g_free (drag_pixmap);
+	}
+	else
+	{
+		window->drag_pixmap = NULL;
+	}
 	toolbar_add_drag_event_box (window, toolbar, window->drag_pixmap);
 
 	/* create location entry */
-	window->toolbar_gnomeentry = gnome_entry_new ("location_history");
+	window->toolbar_gnomeentry = gnome_entry_new ("LocationHistory");
 	gnome_entry_load_history (GNOME_ENTRY(window->toolbar_gnomeentry));
 
 	/* connect the popwin handlers */
@@ -552,6 +516,17 @@ toolbar_create_location_widgets (GaleonWindow *window,
 	gtk_signal_connect 
 		(GTK_OBJECT(window->toolbar_entry), "drag_data_received",
 		 window_location_entry_drag_data_received_cb, window);
+	gtk_signal_connect_after
+		(GTK_OBJECT(window->toolbar_entry), "button_press_event",
+		 (GtkSignalFunc)window_location_entry_button_press_cb, window);
+
+	/* set url entry drop destination */
+	gtk_drag_dest_set 
+		(GTK_WIDGET (gnome_entry_gtk_entry 
+			     (GNOME_ENTRY (window->toolbar_gnomeentry))),
+		 GTK_DEST_DEFAULT_ALL,
+		 drop_types, drop_types_num_items,
+		 GDK_ACTION_COPY);
 
 	/* create go button toolbar */
 	window->go_toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, 
@@ -564,7 +539,7 @@ toolbar_create_location_widgets (GaleonWindow *window,
 /*
  * toolbar_create_location: create location toolbar
  */
-GtkWidget*
+static GtkWidget*
 toolbar_create_location (GaleonWindow *window)
 {
 	GtkWidget *hbox;
@@ -587,7 +562,7 @@ toolbar_create_location (GaleonWindow *window)
 /*
  * toolbar_button: create and append a button to a toolbar
  */
-GtkWidget *
+static GtkWidget *
 toolbar_button (GaleonWindow *window, GtkWidget *toolbar, 
 		const ToolbarButton *button)
 {
@@ -643,7 +618,8 @@ toolbar_button (GaleonWindow *window, GtkWidget *toolbar,
 	return GTK_WIDGET (widget);
 }
 
-gboolean toolbar_theme_use_standard_icons (void)
+static gboolean 
+toolbar_theme_use_standard_icons (void)
 {
 	xmlDocPtr doc;
 	xmlNodePtr item;
@@ -655,22 +631,27 @@ gboolean toolbar_theme_use_standard_icons (void)
 	if (g_file_exists (filename))
 	{
 		doc = xmlParseFile (filename);
+		g_return_val_if_fail (doc != NULL, FALSE);
+		g_return_val_if_fail (doc->root != NULL, FALSE);
 		item = doc->root->childs;
-		while (item != NULL) {
-			if (!strcmp (item->name, "toolbar")) {
+		while (item != NULL)
+		{
+			if (!strcmp (item->name, "toolbar"))
+			{
 				icon_theme = xmlGetProp (item, "icon_theme");
 				result = (strcmp (icon_theme, "standard")==0);
 				xmlFree (icon_theme);
 			}
 			item = item->next;
 		}
+		xmlFreeDoc (doc);
 	}
 
 	g_free (filename);
 	return result;
 }
 
-GtkWidget *
+static GtkWidget *
 toolbar_get_theme_pixmap (const gchar *pixmap)
 {
 	char *filename;
@@ -693,7 +674,6 @@ toolbar_get_theme_pixmap (const gchar *pixmap)
 			g_free (filename);
 			return NULL;
 		}
-
 	}
 
 	icon = pixmap_data_new_from_file (filename);
@@ -702,6 +682,7 @@ toolbar_get_theme_pixmap (const gchar *pixmap)
 	if (icon != NULL)
 	{
 		widget = gtk_pixmap_new (icon->pixmap, icon->mask);
+		g_free (icon); /* FIXME: better to cache these? */
 		return GTK_WIDGET (widget);
 	}
 	else
@@ -710,9 +691,11 @@ toolbar_get_theme_pixmap (const gchar *pixmap)
 	}
 }
 
-char *
+static char *
 toolbar_get_theme_path (void)
 {
+	extern GConfClient *gconf_client;
+	char *aa_pixmap_path;
 	char *pixmap_path;
 	char *theme;
 	char *dir;
@@ -726,15 +709,30 @@ toolbar_get_theme_path (void)
 
 	theme_name = theme;
 
-	dir = gnome_datadir_file("pixmaps/nautilus/");
+	dir = gnome_datadir_file ("pixmaps/nautilus/");
 
-	if (strcmp(theme_name, "default") == 0) {
-		pixmap_path = g_strconcat (dir, NULL);
-	} else {
+	if (strcmp (theme_name, "default") == 0)
+	{
+		pixmap_path = g_strdup (dir);
+	} 
+	else
+	{
 		pixmap_path = g_strconcat (dir, theme, "/", NULL);
 	}
-
 	g_free (dir);
+
+	aa_pixmap_path = g_concat_dir_and_file (pixmap_path, 
+						"fake-aa-navibar/");
+	if (g_file_exists (aa_pixmap_path))
+	{
+		g_free (pixmap_path);
+		return aa_pixmap_path;
+	}
+	else
+	{
+		g_free (aa_pixmap_path);
+	}
+	
 	if (g_file_exists (pixmap_path))
 	{
 		return pixmap_path;
@@ -745,6 +743,7 @@ toolbar_get_theme_path (void)
 	}
 
 	/* try with the user directory */
+	/* FIXME: look for AA path here too */
 	pixmap_path = g_strconcat (g_get_home_dir (), "/.nautilus/themes/", 
 		theme, "/", NULL);
 	if (g_file_exists (pixmap_path))
