@@ -17,6 +17,23 @@
  */
 
 #include "galeon.h"
+#include "embed.h"
+#include "window.h"
+#include "misc.h"
+#include "auto_completion.h"
+#include "history.h"
+#include "bookmarks.h"
+
+#include <string.h>
+#include <gtkmozembed.h>
+#include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-popup-menu.h>
+#include <libgnomeui/gnome-stock.h>
+#include <libgnomeui/gnome-entry.h>
+#include <libgnomeui/gnome-app-helper.h>
+#include <gdk/gdkkeysyms.h>
 
 /* local function prototypes */
 gboolean window_delete_cb (GtkWidget *widget, GdkEventAny *event, 
@@ -25,9 +42,11 @@ gint window_zoom_spin_timeout_cb (GaleonWindow *window);
 void window_entry_changed_cb (GtkEditable *editable, gboolean *changed); 
 void window_location_gnomeentry_popwin_cb (GtkWidget *widget, 
 					   GaleonWindow *window);
+static void window_handle_go_button (GaleonWindow *window, gboolean new_embed,
+				     gboolean new_window);
 
 GTimer *zoom_timer = NULL;
-#define ZOOM_DELAY 0.75 
+#define ZOOM_DELAY 0.20
 
 /**
  * mozembed_selection_get_cb: get selection on copy link location
@@ -62,16 +81,22 @@ window_selection_received_cb (GtkWidget *widget,
 
 	if (selection_data->data)
 	{
-		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
 		
 		gdk_window_get_pointer (window->WMain->window,
 			NULL, NULL, &modifier);
 			
-		if (modifier & GDK_SHIFT_MASK)
-		  	embed_create_from_url (window->active_embed, selection_data->data,
-				!tabbed_mode);
-		else
+		if (modifier & GDK_SHIFT_MASK) {
+			tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+			if (modifier & GDK_CONTROL_MASK) {
+				tabbed_mode = !tabbed_mode;
+			}
+		  	embed_create_from_url (window->active_embed, 
+		  			       selection_data->data,
+		  			       TRUE, !tabbed_mode);
+		}
+		else {
 			window_load_url (window, selection_data->data);
+		}
 	}
 }
 
@@ -81,15 +106,17 @@ window_selection_received_cb (GtkWidget *widget,
 gboolean
 window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
 {
-	/* location bar history is saved automatically when the widget is
+	extern gboolean fullscreen_active;
+	
+        /* location bar history is saved automatically when the widget is
 	   destroyed. we don't want to save the text currently in the entry */
-	gtk_editable_delete_text (GTK_EDITABLE (window->toolbar_entry),
-				  0, -1);
+	window_clear_url_entry (window);
 
 	/* save window size if we are not in fullscreen and if 
 	 * the window is not a popup */
-	if (!GTK_CHECK_MENU_ITEM (window->view_fullscreen)->active &&
-	    !window->active_embed->is_popup)
+	if (window->active_embed != NULL &&
+	    !window->active_embed->is_popup &&
+	    !fullscreen_active)
 	{	
 		gnome_config_set_int("/galeon/Appearance/winwidth", 
 				     window->WMain->allocation.width);
@@ -99,6 +126,7 @@ window_delete_cb (GtkWidget *widget, GdkEventAny *event, GaleonWindow *window)
 
 	/* close the window */
 	window_close (window);
+
 	return TRUE;
 }
 
@@ -121,11 +149,11 @@ window_back_forward_button_press_cb (GtkButton *button,
 	{
 		if (button == GTK_BUTTON (window->BForward))
 		{
-			menu = create_forward_menu (embed);
+			menu = embed_create_forward_menu (embed);
 		}
 		else if (button == GTK_BUTTON (window->BBack))
 		{
-			menu = create_back_menu (embed);
+			menu = embed_create_back_menu (embed);
 		}
 		else
 		{
@@ -141,28 +169,28 @@ window_back_forward_button_press_cb (GtkButton *button,
 }
 
 /** 
- * window_start_button_clicked_cb: start button clicked
+ * window_home_button_clicked_cb: home button clicked
  */
 void
-window_start_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+window_home_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	window_go_home (window, FALSE, FALSE);
 }
 
 /** 
- * window_start_button_press_event_cb:  Open home page in new window on
+ * window_home_button_press_event_cb:  Open home page in new window on
  * middle click, or display a context menu on right-click.
  */
 gboolean
-window_start_button_press_event_cb (GtkButton *button, GdkEventButton *event,
-				    GaleonWindow *window)
+window_home_button_press_event_cb (GtkButton *button, GdkEventButton *event,
+				   GaleonWindow *window)
 {
-	gboolean tabbed_mode;
-	gboolean shift_modifier;
 	GaleonEmbed *embed;
 	GtkWidget *menu;
 	gint action;
+	gboolean tabbed_mode;
+	gboolean shift_modifier;
 
 	/* homepage context menu */
 	/* although there's only one entry I think it's still necessary
@@ -184,14 +212,17 @@ window_start_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	{
 	case 1:
 		/* nothing -- handled by click callback */
-		break;
+		return FALSE;
 
 	case 2:
-		/* load in new tab or window */
-		tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED);
-		shift_modifier = (event->state & GDK_SHIFT_MASK) ? 0 : 1;
-		window_go_home (window, TRUE, tabbed_mode ^ shift_modifier);
-		break;
+		/* open home page in new window/tab */
+		tabbed_mode =
+			gnome_config_get_bool (CONF_APPEARANCE_TABBED);
+		shift_modifier =
+			(event->state & GDK_SHIFT_MASK) ? 0 : 1;
+		window_go_home (window, TRUE,
+			tabbed_mode ^ shift_modifier);
+		return TRUE;
 
 	case 3:
 		/* show context menu */
@@ -212,14 +243,15 @@ window_start_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 
 		/* destroy the popup menu */
 		gtk_widget_destroy (GTK_WIDGET (menu));
-		break;
-	}
+		return TRUE;
 
-	return TRUE;
+	default:
+		return FALSE;
+	}
 }
 
 /** 
- * window_new_button_clicked_cb: new browser button pressed
+ * window_new_button_press_cb: new browser button pressed
  */
 gboolean
 window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
@@ -229,22 +261,61 @@ window_new_button_press_cb (GtkButton *button, GdkEventButton *event,
 	gint shift_modifier;
 	GdkModifierType modifier;
 	GaleonEmbed *embed;
+	GtkWidget *menu;
+	gint action;
+
+	/* new button context menu */
+	static GnomeUIInfo menu_uiinfo[] =
+	{
+		GNOMEUIINFO_ITEM_STOCK (N_("Open new window"),
+					NULL, NULL, GNOME_STOCK_MENU_NEW),
+		GNOMEUIINFO_ITEM_STOCK (N_("Open new tab"),
+					NULL, NULL, GNOME_STOCK_MENU_NEW),
+		GNOMEUIINFO_END
+	};
 
 	return_val_if_not_window (window, FALSE);
 	embed = window->active_embed;
 	return_val_if_not_embed (embed, FALSE);
 
-	if (event->button != 1 && event->button != 2)
-	{
-		return FALSE;
-	}
-
 	tabbed_mode = gnome_config_get_bool (CONF_APPEARANCE_TABBED) ? 1 : 0;
 	gdk_window_get_pointer (window->WMain->window, NULL, NULL, &modifier);
 	shift_modifier = (modifier & GDK_SHIFT_MASK) ? 0 : 1;
-	embed_create_default (embed, (tabbed_mode ^ shift_modifier));
+	
+	switch (event->button)
+	{
+	case 1:
+		embed_create_default (embed, (tabbed_mode ^ shift_modifier));
+		return TRUE;
+		
+	case 2:
+		/* hmmm, this is probably a bit hacky ;-) -- MattA */
+		embed = embed_create (embed, (tabbed_mode ^ shift_modifier), 
+				      TRUE, GTK_MOZ_EMBED_FLAG_ALLCHROME);
+		gtk_selection_convert (embed->parent_window->WMain,
+				       GDK_SELECTION_PRIMARY,
+				       GDK_SELECTION_TYPE_STRING,
+				       GDK_CURRENT_TIME);
+		return FALSE;
 
-	return TRUE;
+	case 3:
+		/* show context menu */
+		menu = gnome_popup_menu_new (menu_uiinfo);
+		action = gnome_popup_menu_do_popup_modal 
+			(GTK_WIDGET (menu),
+			 menu_position_under_widget, 
+			 button, NULL, NULL);
+		
+		/* do action */
+		if (action != -1)
+		{
+			embed_create_default (embed, action == 0);
+		}
+		return TRUE;
+
+	default:
+		return FALSE;
+	}
 }
 
 /**
@@ -278,7 +349,7 @@ window_new_button_drag_data_received_cb (GtkWidget *widget,
 	case DND_TARGET_STRING:
 		embed_create_from_url (window->active_embed,
 				       selection_data->data,
-				       !tabbed_mode);
+				       TRUE, !tabbed_mode);
 		break;
 
 	default:
@@ -295,7 +366,7 @@ window_back_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_back (window->active_embed->mozEmbed);
+	gtk_moz_embed_go_back (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
@@ -306,7 +377,6 @@ window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 				 GaleonWindow *window)
 {
 	GtkMenu *menu;
-	gint levels;
 
 	/* check arguments */
 	return_val_if_not_window (window, FALSE);
@@ -319,22 +389,15 @@ window_up_button_press_event_cb (GtkButton *button, GdkEventButton *event,
 	}
 
 	/* make menu */
-	menu = create_up_menu (window->active_embed);
+	menu = embed_create_up_menu (window->active_embed);
 	if (menu == NULL)
 	{
 		return TRUE;
 	}
 
 	/* show popup menu */
-	levels = gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
-						  menu_position_under_widget, 
-						  button, NULL, NULL);
-
-	/* go up n levels if requested */
-	if (levels >= 0)
-	{
-		embed_go_up (window->active_embed, levels);
-	}
+	gnome_popup_menu_do_popup_modal (GTK_WIDGET (menu),
+					 NULL, NULL, NULL, NULL);
 
 	/* destroy the popup menu */
 	gtk_widget_destroy (GTK_WIDGET (menu));
@@ -362,7 +425,7 @@ window_forward_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
 	return_if_not_window (window);
 	return_if_not_embed (window->active_embed);
-	gtk_moz_embed_go_forward(window->active_embed->mozEmbed);
+	gtk_moz_embed_go_forward(GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
@@ -448,106 +511,124 @@ void
 window_stop_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 {
 	return_if_not_window (window);
-	gtk_moz_embed_stop_load (window->active_embed->mozEmbed);
+	gtk_moz_embed_stop_load (GTK_MOZ_EMBED(window->active_embed->mozEmbed));
 }
 
 /** 
  * window_location_entry_key_press_cb: key pressed in the url entry
- * FIXME: tidy this up
  */
 gboolean
 window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 				    GaleonWindow *window)
 {
 	static gchar *before_completion = NULL;
-	GtkEntry *entry = GTK_ENTRY(widget);
-	GtkEditable *editable = GTK_EDITABLE(widget);
+	GtkEntry *entry = GTK_ENTRY (widget);
+	GtkEditable *editable = GTK_EDITABLE (widget);
 	gboolean url_dialog = FALSE;
 	GaleonEmbed *embed;
+	guint keyval;
 
 	g_return_val_if_fail (GTK_IS_ENTRY (widget), TRUE);
 	return_val_if_not_window (window, TRUE);
 	embed = window->active_embed;
 	return_val_if_not_embed (embed, TRUE);
 
+	auto_completion_reset();
+
+	/* don't process the standard GtkEntry editing shortcuts */
+	if (event->keyval > GDK_A && event->keyval < GDK_Z)
+		keyval = event->keyval + 0x20;
+	else
+		keyval = event->keyval;
+
+	if ((event->state & GDK_Control_L || event->state & GDK_Control_R) 
+	    && (keyval == GDK_a || keyval == GDK_b || keyval == GDK_c ||
+	        keyval == GDK_d || keyval == GDK_e || keyval == GDK_f ||
+	        keyval == GDK_h || keyval == GDK_k || keyval == GDK_u ||
+	        keyval == GDK_v || keyval == GDK_w || keyval == GDK_x ))
+	{
+		return TRUE;
+	}
+		  
+	/* don't grab alt combos, thus you can still access the menus. */
+	if (event->state & GDK_MOD1_MASK)
+		return FALSE;
+
 	/* check to see if key press is in the window's location bar or
 	   the open url dialog */
 	if (widget != window->toolbar_entry)
 		url_dialog = TRUE;
 
-	auto_completion_reset();
 	if ((event->keyval == GDK_Return) || (event->keyval == GDK_KP_Enter))
 	{
 		if (!url_dialog)
 		{
-			window_go_button_clicked_cb (NULL, window);
+			window_handle_go_button (window, FALSE, FALSE);
 		}
 		return TRUE;
 	}
 
-	if ((event->keyval == GDK_Left) || (event->keyval == GDK_Right))
-	{
-		return TRUE;
-	}
-
+	/* make sure the end key works at all times */ 
 	if ( ( ! ( (event->state & GDK_SHIFT_MASK) ||
 		   (event->state & GDK_CONTROL_MASK) ||
 		   (event->state & GDK_MOD1_MASK) ) 
-	      && (event->keyval == GDK_End))) {
+	      && (event->keyval == GDK_End)))
+	{
 		gtk_editable_select_region(editable, 0, 0); 
 		gtk_editable_set_position(editable, -1);
 		return TRUE;
 	}
-	
-	if (event->keyval == GDK_Tab) {
+
+	switch (event->keyval)
+	{
+	case GDK_Left:
+	case GDK_Right:
+		return TRUE;
+	case GDK_Tab:
+	{
 		gchar *common_prefix = NULL;
 		gchar *text;
 
-		gtk_editable_delete_selection(editable);
-		text = gtk_editable_get_chars(editable, 0, -1);
-		common_prefix = auto_completion_complete_url_extended(text);
-		if (common_prefix) {
+		gtk_editable_delete_selection (editable);
+		text = gtk_editable_get_chars (editable, 0, -1);
+		common_prefix = auto_completion_complete_url_extended (text);
+		if (common_prefix)
+		{
 			if (!before_completion) 
 				before_completion = g_strdup(text);
 
-			gtk_entry_set_text(entry, common_prefix);
-			auto_completion_display_alternatives(window, widget);
-			g_free(common_prefix);
+			gtk_entry_set_text (entry, common_prefix);
+			auto_completion_display_alternatives (window, widget);
+			g_free (common_prefix);
 		}
 		return TRUE;
 	}
-
-	if (event->keyval == GDK_Escape) {
+	case GDK_Escape:
 		auto_completion_reset();
-		if (before_completion) {
-			gtk_entry_set_text(entry, before_completion);
-			g_free(before_completion);
+
+		if (before_completion)
+		{
+			gtk_entry_set_text (entry, before_completion);
+			g_free (before_completion);
 			before_completion = NULL;
-		} else
+		}
+		else
 		{
 			gtk_entry_set_text (entry, embed->site_location);
 		}
 		return TRUE;
-	}
-
-	/* do not toggle statusbar on Ctrl+U/A/E, since this is an editing
-	   shortcut in GtkEntry */
-	if ((event->state & GDK_Control_L || event->state & GDK_Control_R) 
-	    && (event->keyval == GDK_U || event->keyval == GDK_u ||
-		event->keyval == GDK_A || event->keyval == GDK_a ||
-		event->keyval == GDK_E || event->keyval == GDK_e))
-	{
-		return TRUE;
+	default:
+		break;
 	}
 
 	if (event->string[0] > 32 && event->string[0] < 126) 
 	{
 		if (before_completion != NULL) 
 		{
-			g_free(before_completion);
+			g_free (before_completion);
 			before_completion = NULL;
 		}
-		auto_completion_display(window, widget);
+		auto_completion_display (window, widget);
 		return TRUE;
 	}
 
@@ -559,60 +640,92 @@ window_location_entry_key_press_cb (GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 }
 
-/** 
- * window_go_button_clicked_cb: go button clicked
+/**
+ * window_location_entry_button_press_cb: select the whole location field
+ * when the user double clicks on the entry. This is preferable to the
+ * GNOME and X standard way of selecting since it's not selecting english
+ * text but instead a URL.
  */
-void
-window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+gboolean
+window_location_entry_button_press_cb (GtkWidget *entry, 
+				       GdkEventButton *event, 
+				       GaleonWindow *window)
+{
+	if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
+	{
+		gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+		gtk_entry_select_region (GTK_ENTRY (entry), 0, -1);
+
+		/* Ugh, more GTK+ goofiness. Returning TRUE doesn't stop
+		   emission of the signal, so we have to do it ourselves */
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (entry),
+		                              "button-press-event");
+		return TRUE;
+	}
+	return FALSE;
+}
+
+static void
+window_handle_go_button (GaleonWindow *window, gboolean new_embed,
+			 gboolean new_window)
 {
 	gchar *text;
 	gchar *text2;
 	GList *wl;
 	GnomeEntry *ge;
 
+	return_if_not_window (window);
+
 	text = gtk_editable_get_chars (GTK_EDITABLE (window->toolbar_entry),
 				       0, -1);
 
-	/* update the content of the location field of the GaleonEmbed */
-	if (window->active_embed->site_location) 
+	/* make sure the string isn't empty */
+	if (strcmp (text, "") != 0)
 	{
-		g_free (window->active_embed->site_location);
-	}
-	window->active_embed->site_location = g_strdup (text);
+		/* don't save the text in the entry -
+		 * avoids duplicate entries */
+		window_clear_url_entry (window);
+		ge = GNOME_ENTRY (window->toolbar_gnomeentry);
+		gnome_entry_save_history (ge);
+		gtk_entry_set_text (GTK_ENTRY (window->toolbar_entry), text);
 
-	if (strcmp(text,"")!=0)
-	{
-		/* don't save the text in the entry - avoids duplicate entries */
-		gtk_editable_delete_text(GTK_EDITABLE(window->toolbar_entry),
-					 0, -1);
-		ge = GNOME_ENTRY(window->toolbar_gnomeentry);
-		gnome_entry_save_history(ge);
-		gtk_entry_set_text(GTK_ENTRY(window->toolbar_entry), text);
-
-		/* add item to location bars in all other open browser windows */
+		/* add item to location bars in all other
+		 * open browser windows */
 		for (wl = all_windows; wl != NULL; wl = wl->next)
 		{
 			GtkList *list;
 			GList *items;
+			GaleonWindow *awindow;
 			gchar *label;
 			gboolean dup = FALSE;
 
-			if (wl->data == window)
+			awindow = (GaleonWindow *) (wl->data);
+
+			/* check if it's the active window or
+			   if the window is without entry */
+			if (wl->data == window ||
+			    (!awindow->toolbar_gnomeentry))
 				continue;
 
-			ge = GNOME_ENTRY(((GaleonWindow *)
-					   (wl->data))->toolbar_gnomeentry);
+			ge = GNOME_ENTRY(awindow->toolbar_gnomeentry);
+
+			g_return_if_fail (ge);
+
 			list = GTK_LIST(ge->combo.list);
-			for (items = list->children; items != NULL; items = items->next)
+			for (items = list->children; items != NULL;
+			     items = items->next)
 			{
 				gtk_label_get(GTK_LABEL(
-						GTK_BIN(items->data)->child),
-					      &label);
+					GTK_BIN(items->data)->child), &label);
 				if (!strcmp(label, text))
+				{
 					dup = TRUE;
+				}
 			}
 			if (!dup)
+			{
 				gnome_entry_prepend_history(ge, TRUE, text);
+			}
 		}
 
 		text2 = bookmarks_parse_nick (text, NULL);
@@ -622,10 +735,77 @@ window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 			g_free (text);
 			text = text2;
 		}
-	
-		window_load_url (window, text);
-		g_free (text);
 	}
+	/* empty string -- load a blank page */
+	else
+	{
+		g_free (text);
+		text = g_strdup ("about:blank");
+	}
+
+	/* if we need to open in the current embed... */
+	if (!new_embed)
+	{
+		/* update the contents of the location field of the
+		 * GaleonEmbed if it's empty */
+		if (window->active_embed->site_location) 
+		{
+			g_free (window->active_embed->site_location);
+		}
+		window->active_embed->site_location = g_strdup (text);
+		window_load_url (window, text);
+	}
+	/* otherwise, open in a new tab or window */
+	else
+	{
+		embed_create_from_url (window->active_embed, text, 
+				       TRUE, new_window);
+	}
+
+	g_free (text);
+}
+
+
+/** 
+ * window_go_button_press_cb: go button pressed
+ */
+gboolean
+window_go_button_press_cb (GtkButton *button, GdkEventButton *event,
+			   GaleonWindow *window)
+{
+	gboolean tabbed_mode;
+	gboolean shift_modifier;
+
+	return_val_if_not_window (window, FALSE);
+
+	switch (event->button)
+	{
+		case 1:
+			/* nothing -- handled by click callback */
+			break;
+
+		case 2:
+			/* open location in new window/tab */
+			tabbed_mode = gnome_config_get_bool
+				(CONF_APPEARANCE_TABBED);
+			shift_modifier =
+				(event->state & GDK_SHIFT_MASK) ? 0 : 1;
+			window_handle_go_button (window, TRUE,
+						 tabbed_mode ^ shift_modifier);
+			break;
+	}
+
+	return TRUE;
+}
+
+/** 
+ * window_go_button_press_cb: go button clicked
+ */
+void
+window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
+{
+	return_if_not_window (window);
+	window_handle_go_button (window, FALSE, FALSE);
 }
 
 /**
@@ -634,17 +814,41 @@ window_go_button_clicked_cb (GtkButton *button, GaleonWindow *window)
 gint 
 window_zoom_spin_timeout_cb (GaleonWindow *window)
 {
-	return_val_if_not_window (window, FALSE);
-	return_val_if_not_embed (window->active_embed, FALSE);
+	GaleonEmbed *embed;
+	gint zoom;
 
-	if (zoom_timer == NULL) return FALSE;
-	if (g_timer_elapsed (zoom_timer, NULL) >= ZOOM_DELAY) {
-		gint value = atoi (gtk_editable_get_chars (GTK_EDITABLE (window->zoom_spin), 0, -1));
-		g_timer_destroy (zoom_timer);
-		zoom_timer = NULL;
-		mozilla_set_zoom (window->active_embed, (float) value / 100);
+	return_val_if_not_window (window, FALSE);
+	embed = window->active_embed;
+	return_val_if_not_embed (embed, FALSE);
+
+	/* timer still valid? */
+	if (zoom_timer == NULL)
+	{
 		return FALSE;
 	}
+
+	/* okay, we're ready to set */
+	if (g_timer_elapsed (zoom_timer, NULL) >= ZOOM_DELAY)
+	{
+		/* kill off the timer */
+		g_timer_destroy (zoom_timer);
+		zoom_timer = NULL;
+
+		/* get the value of the spin */
+		zoom = gtk_spin_button_get_value_as_int 
+			(GTK_SPIN_BUTTON (window->zoom_spin));
+
+		/* set */
+		embed_set_zoom (embed, zoom);
+
+		/* store for this site */
+		history_set_zoom (embed->site_location, zoom);
+
+		/* done now */
+		return FALSE;
+	}
+
+	/* call me again */
 	return TRUE;
 }
 
@@ -654,7 +858,25 @@ window_zoom_spin_timeout_cb (GaleonWindow *window)
 void 
 window_zoom_spin_changed_cb (GtkEditable *editable, GaleonWindow *window)
 {
-	if (zoom_timer != NULL) g_timer_destroy (zoom_timer);
+	gint zoom;
+
+	/* get the value of the spin */
+	zoom = gtk_spin_button_get_value_as_int 
+		(GTK_SPIN_BUTTON (window->zoom_spin));
+
+	/* check we haven't already registered the change */
+	if (window->active_embed != NULL && window->active_embed->zoom == zoom)
+	{
+		return;
+	}
+
+	/* destroy any existing timer */
+	if (zoom_timer != NULL)
+	{
+		g_timer_destroy (zoom_timer);
+	}
+
+	/* start the new one */
 	zoom_timer = g_timer_new();
 	g_timer_start (zoom_timer);
 	g_timeout_add (50, (GSourceFunc) window_zoom_spin_timeout_cb, window);
@@ -715,14 +937,15 @@ window_location_entry_drag_data_received_cb (GtkWidget *widget,
 {
 	gchar *url = selection_data->data;
 	gint i;
-	
+
 	switch (info) {
 	case DND_TARGET_STRING:
 	case DND_TARGET_NETSCAPE_URL:
 	case DND_TARGET_GALEON_URL:
-		gtk_editable_delete_text (GTK_EDITABLE (window->toolbar_entry), 0, -1);
-		gtk_editable_insert_text (GTK_EDITABLE (window->toolbar_entry), url, 
-					  strlen (url), &i);
+	case DND_TARGET_TEXT_URI_LIST:/* FIXME: won't work right for a list */
+		window_clear_url_entry (window);
+		gtk_editable_insert_text (GTK_EDITABLE (window->toolbar_entry),
+					  url, strlen (url), &i);
 		break;
 	default:
 		g_warning ("Unknown DND type");
