@@ -16,7 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <galeon.h>
+#include "galeon.h"
+#include "misc.h"
+
+#include <string.h>
 #include <gnome-xml/tree.h>
 
 /* Export Prototypes */
